@@ -1,5 +1,5 @@
 /*
- * propget-cmd.c -- Display status information in current directory
+ * propget-cmd.c -- Print properties and values of files/dirs
  *
  * ====================================================================
  * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
@@ -129,7 +129,7 @@ svn_cl__propget (apr_getopt_t *os,
           
           SVN_ERR (stream_write (out, printable_val->data, 
                                  printable_val->len));
-          SVN_ERR (stream_write (out, "\n", 1));
+          SVN_ERR (stream_write (out, APR_EOL_STR, strlen (APR_EOL_STR)));
         }
     }
   else  /* operate on a normal, versioned property (not a revprop) */
@@ -145,6 +145,7 @@ svn_cl__propget (apr_getopt_t *os,
           svn_boolean_t is_url = svn_path_is_url (target);
 
           svn_pool_clear (subpool);
+          SVN_ERR (svn_cl__check_cancel (ctx->cancel_baton));
           SVN_ERR (svn_client_propget (&props, pname_utf8, target,
                                        &(opt_state->start_revision),
                                        opt_state->recursive, ctx, subpool));
@@ -171,27 +172,36 @@ svn_cl__propget (apr_getopt_t *os,
               /* If this is a special Subversion property, it is stored as
                  UTF8, so convert to the native format. */
               if (svn_prop_needs_translation (pname_utf8))
-                SVN_ERR (svn_subst_detranslate_string (&propval, propval,
-                                                       TRUE, subpool));
+                {
+                  SVN_ERR (svn_subst_detranslate_string (&propval, propval,
+                                                         TRUE, subpool));
+                }
               
               if (print_filenames) 
                 {
                   const char *filename_stdout;
+
                   if (! is_url)
-                    SVN_ERR (svn_cmdline_path_local_style_from_utf8
-                             (&filename_stdout, filename, subpool));
+                    {
+                      SVN_ERR (svn_cmdline_path_local_style_from_utf8
+                               (&filename_stdout, filename, subpool));
+                    }
                   else
-                    SVN_ERR (svn_cmdline_cstring_from_utf8
-                             (&filename_stdout, filename, subpool));
+                    {
+                      SVN_ERR (svn_cmdline_cstring_from_utf8
+                               (&filename_stdout, filename, subpool));
+                    }
+
                   SVN_ERR (stream_write (out, filename_stdout,
                                          strlen (filename_stdout)));
                   SVN_ERR (stream_write (out, " - ", 3));
-                } 
+                }
+
               SVN_ERR (stream_write (out, propval->data, propval->len));
               if (! opt_state->strict)
-                SVN_ERR (stream_write (out, "\n", 1));
+                SVN_ERR (stream_write (out, APR_EOL_STR,
+                                       strlen (APR_EOL_STR)));
             }
-          SVN_ERR (svn_cl__check_cancel (ctx->cancel_baton));
         }
       svn_pool_destroy (subpool);
     }
