@@ -32,6 +32,7 @@
 #include "svn_path.h"
 #include "client.h"
 
+#include "svn_private_config.h"
 
 
 /*** Code. ***/
@@ -58,11 +59,11 @@ find_undeletables (void *baton,
   /* Check for error-ful states. */
   if (status->text_status == svn_wc_status_obstructed)
     sb->err = svn_error_createf (SVN_ERR_NODE_UNEXPECTED_KIND, NULL,
-                                 "'%s' is in the way of the resource "
-                                 "actually under version control", path);
+                                 _("'%s' is in the way of the resource "
+                                   "actually under version control"), path);
   else if (! status->entry)
     sb->err = svn_error_createf (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-                                 "'%s' is not under version control", path);
+                                 _("'%s' is not under version control"), path);
   
   else if ((status->text_status != svn_wc_status_normal
             && status->text_status != svn_wc_status_deleted
@@ -71,7 +72,7 @@ find_undeletables (void *baton,
            (status->prop_status != svn_wc_status_none
             && status->prop_status != svn_wc_status_normal))
     sb->err = svn_error_createf (SVN_ERR_CLIENT_MODIFIED, NULL,
-                             "'%s' has local modifications", path);
+                             _("'%s' has local modifications"), path);
 }
 
 
@@ -251,16 +252,17 @@ svn_client_delete (svn_client_commit_info_t **commit_info,
 
           svn_pool_clear (subpool);
           parent_path = svn_path_dirname (path, subpool);
-          /* Let the working copy library handle the PATH. */
-          SVN_ERR (svn_wc_adm_open (&adm_access, NULL, parent_path, 
-                                    TRUE, FALSE, subpool));
-          SVN_ERR (svn_client__wc_delete (path, adm_access, force, 
-                                          FALSE, ctx, subpool));
-          SVN_ERR (svn_wc_adm_close (adm_access));
 
           /* See if the user wants us to stop. */
           if (ctx->cancel_func)
             SVN_ERR (ctx->cancel_func (ctx->cancel_baton));
+
+          /* Let the working copy library handle the PATH. */
+          SVN_ERR (svn_wc_adm_open2 (&adm_access, NULL, parent_path, 
+                                     TRUE, 0, subpool));
+          SVN_ERR (svn_client__wc_delete (path, adm_access, force, 
+                                          FALSE, ctx, subpool));
+          SVN_ERR (svn_wc_adm_close (adm_access));
         }
       svn_pool_destroy (subpool);
     }
