@@ -34,6 +34,8 @@
 #include "svn_ra.h"
 #include "svn_pools.h"
 
+#include "svn_private_config.h"
+
 #include "ra_dav.h"
 
 
@@ -211,18 +213,18 @@ static svn_error_t * handle_resource(merge_ctx_t *mc,
       /* ### shouldn't have happened. we told the server "don't merge" */
       /* ### need something better than APR_EGENERAL */
       return svn_error_createf(APR_EGENERAL, NULL,
-                               "Protocol error: we told the server to not "
-                               "auto-merge any resources, but it said that "
-                               "'%s' was merged", mc->href->data);
+                               _("Protocol error: we told the server not to "
+                                 "auto-merge any resources, but it said that "
+                                 "'%s' was merged"), mc->href->data);
     }
   if (mc->response_parent != ELEM_updated_set)
     {
       /* ### unknown parent for this response(!) */
       /* ### need something better than APR_EGENERAL */
       return svn_error_createf(APR_EGENERAL, NULL,
-                               "Internal error: there is an unknown parent "
-                               "(%d) for the 'DAV:response' element within the"
-                               " MERGE response", mc->response_parent);
+                               _("Internal error: there is an unknown parent "
+                                 "(%d) for the 'DAV:response' element within"
+                                 " the MERGE response"), mc->response_parent);
     }
 #if 0
   /* ### right now, the server isn't sending everything for all resources.
@@ -235,10 +237,10 @@ static svn_error_t * handle_resource(merge_ctx_t *mc,
       /* one or more properties were missing in the DAV:response for the
          resource. */
       return svn_error_createf(APR_EGENERAL, NULL,
-                               "Protocol error: the MERGE response for the "
-                               "'%s' resource did not return all of the "
-                               "properties that we asked for (and need to "
-                               "complete the commit).", mc->href->data);
+                               _("Protocol error: the MERGE response for the "
+                                 "'%s' resource did not return all of the "
+                                 "properties that we asked for (and need to "
+                                 "complete the commit)"), mc->href->data);
     }
 #endif
 
@@ -254,8 +256,8 @@ static svn_error_t * handle_resource(merge_ctx_t *mc,
     {
       /* ### need something better than APR_EGENERAL */
       return svn_error_createf(APR_EGENERAL, NULL,
-                               "A MERGE response for '%s' is not a child "
-                               "of the destination ('%s')",
+                               _("A MERGE response for '%s' is not a child "
+                                 "of the destination ('%s')"),
                                mc->href->data, mc->base_href);
     }
 
@@ -475,8 +477,8 @@ static int end_element(void *userdata, const svn_ra_dav__xml_elm_t *elm,
           {
             /* ### fix this error value */
             mc->err = svn_error_create(APR_EGENERAL, NULL,
-                                       "The MERGE property response had an "
-                                       "error status");
+                                       _("The MERGE property response had an "
+                                         "error status"));
           }
       }
       break;
