@@ -2,7 +2,7 @@
  * props.c :  routines for fetching DAV properties
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -24,6 +24,7 @@
 #define APR_WANT_STRFUNC
 #include <apr_want.h>
 
+#include <ne_socket.h>
 #include <ne_basic.h>
 #include <ne_props.h>
 #include <ne_xml.h>
@@ -37,6 +38,8 @@
 #include "svn_xml.h"
 #include "svn_pools.h"
 
+#include "svn_private_config.h"
+
 #include "ra_dav.h"
 
 
@@ -56,6 +59,7 @@ static const ne_propname starting_props[] =
   { "DAV:", "version-controlled-configuration" },
   { "DAV:", "resourcetype" },
   { SVN_DAV_PROP_NS_DAV, "baseline-relative-path" },
+  { SVN_DAV_PROP_NS_DAV, "repository-uuid"},
   { NULL }
 };
 
@@ -73,7 +77,7 @@ static const ne_propname baseline_props[] =
 /*** Propfind Implementation ***/
 
 typedef struct {
-  ne_xml_elmid id;
+  svn_ra_dav__xml_elmid id;
   const char *name;
   int is_property;      /* is it a property, or part of some structure? */
 } elem_defn;
@@ -85,14 +89,14 @@ static const elem_defn elem_definitions[] =
        propfind_elements[] ***/
 
   /* DAV elements */
-  { NE_ELM_multistatus, "DAV:multistatus", 0 },
-  { NE_ELM_response, "DAV:response", 0 },
-  { NE_ELM_href, "DAV:href", NE_XML_CDATA },
-  { NE_ELM_propstat, "DAV:propstat", 0 },
-  { NE_ELM_prop, "DAV:prop", 0 },
-  { NE_ELM_status, "DAV:status", NE_XML_CDATA },
-  { ELEM_baseline, "DAV:baseline", NE_XML_CDATA },
-  { ELEM_collection, "DAV:collection", NE_XML_CDATA },
+  { ELEM_multistatus, "DAV:multistatus", 0 },
+  { ELEM_response, "DAV:response", 0 },
+  { ELEM_href, "DAV:href", SVN_RA_DAV__XML_CDATA },
+  { ELEM_propstat, "DAV:propstat", 0 },
+  { ELEM_prop, "DAV:prop", 0 },
+  { ELEM_status, "DAV:status", SVN_RA_DAV__XML_CDATA },
+  { ELEM_baseline, "DAV:baseline", SVN_RA_DAV__XML_CDATA },
+  { ELEM_collection, "DAV:collection", SVN_RA_DAV__XML_CDATA },
   { ELEM_resourcetype, "DAV:resourcetype", 0 },
   { ELEM_baseline_coll, SVN_RA_DAV__PROP_BASELINE_COLLECTION, 0 },
   { ELEM_checked_in, SVN_RA_DAV__PROP_CHECKED_IN, 0 },
@@ -110,39 +114,41 @@ static const elem_defn elem_definitions[] =
 };
 
 
-static const struct ne_xml_elm propfind_elements[] = 
+static const svn_ra_dav__xml_elm_t propfind_elements[] = 
 {
   /*** NOTE: Make sure that every item in here is also represented in
        elem_definitions[] ***/
 
   /* DAV elements */
-  { "DAV:", "multistatus", NE_ELM_multistatus, 0 },
-  { "DAV:", "response", NE_ELM_response, 0 },
-  { "DAV:", "href", NE_ELM_href, NE_XML_CDATA },
-  { "DAV:", "propstat", NE_ELM_propstat, 0 },
-  { "DAV:", "prop", NE_ELM_prop, 0 },
-  { "DAV:", "status", NE_ELM_status, NE_XML_CDATA },
-  { "DAV:", "baseline", ELEM_baseline, NE_XML_CDATA },
-  { "DAV:", "baseline-collection", ELEM_baseline_coll, NE_XML_CDATA },
+  { "DAV:", "multistatus", ELEM_multistatus, 0 },
+  { "DAV:", "response", ELEM_response, 0 },
+  { "DAV:", "href", ELEM_href, SVN_RA_DAV__XML_CDATA },
+  { "DAV:", "propstat", ELEM_propstat, 0 },
+  { "DAV:", "prop", ELEM_prop, 0 },
+  { "DAV:", "status", ELEM_status, SVN_RA_DAV__XML_CDATA },
+  { "DAV:", "baseline", ELEM_baseline, SVN_RA_DAV__XML_CDATA },
+  { "DAV:", "baseline-collection", ELEM_baseline_coll, SVN_RA_DAV__XML_CDATA },
   { "DAV:", "checked-in", ELEM_checked_in, 0 },
-  { "DAV:", "collection", ELEM_collection, NE_XML_CDATA },
+  { "DAV:", "collection", ELEM_collection, SVN_RA_DAV__XML_CDATA },
   { "DAV:", "resourcetype", ELEM_resourcetype, 0 },
   { "DAV:", "version-controlled-configuration", ELEM_vcc, 0 },
-  { "DAV:", "version-name", ELEM_version_name, NE_XML_CDATA },
-  { "DAV:", "getcontentlength", ELEM_get_content_length, NE_XML_CDATA },
-  { "DAV:", "creationdate", ELEM_creationdate, NE_XML_CDATA },
-  { "DAV:", "creator-displayname", ELEM_creator_displayname, NE_XML_CDATA },
+  { "DAV:", "version-name", ELEM_version_name, SVN_RA_DAV__XML_CDATA },
+  { "DAV:", "getcontentlength", ELEM_get_content_length,
+    SVN_RA_DAV__XML_CDATA },
+  { "DAV:", "creationdate", ELEM_creationdate, SVN_RA_DAV__XML_CDATA },
+  { "DAV:", "creator-displayname", ELEM_creator_displayname,
+    SVN_RA_DAV__XML_CDATA },
 
   /* SVN elements */
   { SVN_DAV_PROP_NS_DAV, "baseline-relative-path", ELEM_baseline_relpath,
-    NE_XML_CDATA },
+    SVN_RA_DAV__XML_CDATA },
   { SVN_DAV_PROP_NS_DAV, "md5-checksum", ELEM_md5_checksum,
-    NE_XML_CDATA },
+    SVN_RA_DAV__XML_CDATA },
   { SVN_DAV_PROP_NS_DAV, "repository-uuid", ELEM_repository_uuid,
-    NE_XML_CDATA },
+    SVN_RA_DAV__XML_CDATA },
 
   /* Unknowns */
-  { "", "", NE_ELM_unknown, NE_XML_COLLECT },
+  { "", "", ELEM_unknown, SVN_RA_DAV__XML_COLLECT },
 
   { NULL } 
 };
@@ -156,7 +162,7 @@ typedef struct propfind_ctx_t
   const char *encoding; /* property encoding (or NULL) */
   int status; /* status for the current <propstat> (or 0 if unknown). */
   apr_hash_t *propbuffer; /* holds properties until their status is known. */
-  ne_xml_elmid last_open_id; /* the id of the last opened tag. */
+  svn_ra_dav__xml_elmid last_open_id; /* the id of the last opened tag. */
   ne_xml_parser *parser; /* xml parser handling the PROPSET request. */
 
   apr_pool_t *pool;
@@ -166,7 +172,7 @@ typedef struct propfind_ctx_t
 
 /* Look up an element definition ID.  May return NULL if the elem is
    not recognized. */
-static const elem_defn *defn_from_id(ne_xml_elmid id)
+static const elem_defn *defn_from_id(svn_ra_dav__xml_elmid id)
 {
   const elem_defn *defn;
 
@@ -206,54 +212,55 @@ static void assign_rsrc_url(svn_ra_dav_resource_t *rsrc,
 
 
 static int validate_element(void *userdata, 
-                            ne_xml_elmid parent, 
-                            ne_xml_elmid child)
+                            svn_ra_dav__xml_elmid parent, 
+                            svn_ra_dav__xml_elmid child)
 {
   switch (parent)
     {
-    case NE_ELM_root:
-      if (child == NE_ELM_multistatus)
-        return NE_XML_VALID;
+    case ELEM_root:
+      if (child == ELEM_multistatus)
+        return SVN_RA_DAV__XML_VALID;
       else
-        return NE_XML_INVALID;
+        return SVN_RA_DAV__XML_INVALID;
 
-    case NE_ELM_multistatus:
-      if (child == NE_ELM_response)
-        return NE_XML_VALID;
+    case ELEM_multistatus:
+      if (child == ELEM_response)
+        return SVN_RA_DAV__XML_VALID;
       else
-        return NE_XML_DECLINE;
+        return SVN_RA_DAV__XML_DECLINE;
 
-    case NE_ELM_response:
-      if ((child == NE_ELM_href) || (child == NE_ELM_propstat))
-        return NE_XML_VALID;
+    case ELEM_response:
+      if ((child == ELEM_href) || (child == ELEM_propstat))
+        return SVN_RA_DAV__XML_VALID;
       else
-        return NE_XML_DECLINE;
+        return SVN_RA_DAV__XML_DECLINE;
 
-    case NE_ELM_propstat:
-      if ((child == NE_ELM_prop) || (child == NE_ELM_status))
-        return NE_XML_VALID;
+    case ELEM_propstat:
+      if ((child == ELEM_prop) || (child == ELEM_status))
+        return SVN_RA_DAV__XML_VALID;
       else
-        return NE_XML_DECLINE;
+        return SVN_RA_DAV__XML_DECLINE;
 
-    case NE_ELM_prop:
-      return NE_XML_VALID; /* handle all children of <prop> */
+    case ELEM_prop:
+      return SVN_RA_DAV__XML_VALID; /* handle all children of <prop> */
         
     case ELEM_baseline_coll:
     case ELEM_checked_in:
     case ELEM_vcc:
-      if (child == NE_ELM_href)
-        return NE_XML_VALID;
+      if (child == ELEM_href)
+        return SVN_RA_DAV__XML_VALID;
       else
-        return NE_XML_DECLINE; /* not concerned with other types */
+        return SVN_RA_DAV__XML_DECLINE; /* not concerned with other types */
       
     case ELEM_resourcetype:
       if ((child == ELEM_collection) || (child == ELEM_baseline))
-        return NE_XML_VALID;
+        return SVN_RA_DAV__XML_VALID;
       else
-        return NE_XML_DECLINE; /* not concerned with other types (### now) */
+        return SVN_RA_DAV__XML_DECLINE; /* not concerned with other types
+                                           (### now) */
 
     default:
-      return NE_XML_DECLINE;
+      return SVN_RA_DAV__XML_DECLINE;
     }
 
   /* NOTREACHED */
@@ -261,16 +268,16 @@ static int validate_element(void *userdata,
 
 
 static int start_element(void *userdata, 
-                         const struct ne_xml_elm *elm, 
+                         const svn_ra_dav__xml_elm_t *elm, 
                          const char **atts)
 {
   propfind_ctx_t *pc = userdata;
 
   switch (elm->id)
     {
-    case NE_ELM_response:
+    case ELEM_response:
       if (pc->rsrc)
-        return 1;
+        return SVN_RA_DAV__XML_INVALID;
       /* Create a new resource. */
       pc->rsrc = apr_pcalloc(pc->pool, sizeof(*(pc->rsrc)));
       pc->rsrc->pool = pc->pool;
@@ -278,11 +285,11 @@ static int start_element(void *userdata,
       pc->status = 0;
       break;
 
-    case NE_ELM_propstat:
+    case ELEM_propstat:
       pc->status = 0;
       break;
 
-    case NE_ELM_href:
+    case ELEM_href:
       /* Remember this <href>'s parent so that when we close this tag,
          we know to whom the URL assignment belongs.  Could be the
          resource itself, or one of the properties:
@@ -294,7 +301,7 @@ static int start_element(void *userdata,
       pc->rsrc->is_collection = 1;
       break;
 
-    case NE_ELM_unknown:
+    case ELEM_unknown:
       /* these are our user-visible properties, presumably. */
       pc->encoding = ne_xml_get_attr(pc->parser, atts, SVN_DAV_PROP_NS_DAV,
                                      "encoding");
@@ -309,12 +316,12 @@ static int start_element(void *userdata,
 
   /* Remember the last tag we opened. */
   pc->last_open_id = elm->id;
-  return 0;
+  return SVN_RA_DAV__XML_VALID;
 }
 
 
 static int end_element(void *userdata, 
-                       const struct ne_xml_elm *elm,
+                       const svn_ra_dav__xml_elm_t *elm,
                        const char *cdata)
 {
   propfind_ctx_t *pc = userdata;
@@ -327,17 +334,17 @@ static int end_element(void *userdata,
 
   switch (elm->id)
     {
-    case NE_ELM_response:
+    case ELEM_response:
       /* Verify that we've received a URL for this resource. */
       if (!pc->rsrc->url)
-        return 1;
+        return SVN_RA_DAV__XML_INVALID;
 
       /* Store the resource in the top-level hash table. */
       apr_hash_set(pc->props, pc->rsrc->url, APR_HASH_KEY_STRING, pc->rsrc);
       pc->rsrc = NULL;
-      return 0;
+      return SVN_RA_DAV__XML_VALID;
 
-    case NE_ELM_propstat:
+    case ELEM_propstat:
       /* We're at the end of a set of properties.  Do the right thing
          status-wise. */
       if (pc->status)
@@ -362,24 +369,24 @@ static int end_element(void *userdata,
       else if (! pc->status)
         {
           /* No status at all?  Bogosity. */
-          return 1;
+          return SVN_RA_DAV__XML_INVALID;
         }
-      return 0;
+      return SVN_RA_DAV__XML_VALID;
 
-    case NE_ELM_status:
+    case ELEM_status:
       /* Parse the <status> tag's CDATA for a status code. */
       if (ne_parse_statusline(cdata, &status))
-        return 1;
+        return SVN_RA_DAV__XML_INVALID;
       free(status.reason_phrase);
       pc->status = status.code;
-      return 0;
+      return SVN_RA_DAV__XML_VALID;
 
-    case NE_ELM_href:
+    case ELEM_href:
       /* Special handling for <href> that belongs to the <response> tag. */
-      if (rsrc->href_parent == NE_ELM_response)
+      if (rsrc->href_parent == ELEM_response)
         {
           assign_rsrc_url(pc->rsrc, cdata, pc->pool);
-          return 0;
+          return SVN_RA_DAV__XML_VALID;
         }
 
       /* Use the parent element's name, not the href. */
@@ -387,7 +394,7 @@ static int end_element(void *userdata,
 
       /* No known parent?  Get outta here. */
       if (!parent_defn)
-        return 0;
+        return SVN_RA_DAV__XML_VALID;
 
       /* All other href's we'll treat as property values. */
       name = parent_defn->name;
@@ -396,15 +403,15 @@ static int end_element(void *userdata,
 
     default:
       /*** This case is, as usual, for everything not covered by other
-           cases.  ELM->id should be either NE_ELM_unknown, or one of
+           cases.  ELM->id should be either ELEM_unknown, or one of
            the ids in the elem_definitions[] structure.  In this case,
-           we seek to handle properties.  Since NE_ELM_unknown should
+           we seek to handle properties.  Since ELEM_unknown should
            only occur for properties, we will handle that id.  All
            other ids will be searched for in the elem_definitions[]
            structure to determine if they are properties.  Properties,
            we handle; all else hits the road.  ***/
 
-      if (elm->id == NE_ELM_unknown)
+      if (elm->id == ELEM_unknown)
         {
           name = apr_pstrcat(pc->pool, elm->nspace, elm->name, NULL);
         }
@@ -412,33 +419,31 @@ static int end_element(void *userdata,
         {
           defn = defn_from_id(elm->id);
           if (! (defn && defn->is_property))
-            return 0;
+            return SVN_RA_DAV__XML_VALID;
           name = defn->name;          
         }
 
-      /* Handle the property value. */
-      value = svn_string_create(cdata, pc->pool);
+      /* Check for encoding attribute. */
+      if (pc->encoding == NULL) {
+        /* Handle the property value by converting it to string. */
+        value = svn_string_create(cdata, pc->pool);
+        break;
+      }
 
-      /* Is there an encoding on this property?  Handle it. */
-      if (pc->encoding)
-        {
-          if (strcmp(pc->encoding, "base64") == 0)
-            {
-              svn_string_t in;
-              in.data = cdata;
-              in.len = strlen(cdata);
-              value = svn_base64_decode_string(&in, pc->pool);
-            }
-          else /* unknown encoding type! */
-            {
-              return 1;
-            }
-          pc->encoding = NULL;
-        }
-      else /* no encoding, so just transform the CDATA into an svn_string_t. */
-        {
-          value = svn_string_create(cdata, pc->pool);
-        }
+      /* Check for known encoding type */
+      if (strcmp(pc->encoding, "base64") != 0)
+        return SVN_RA_DAV__XML_INVALID;
+
+      /* There is an encoding on this property, handle it.
+       * the braces are needed to allocate "in" on the stack. */
+      {
+        svn_string_t in;
+        in.data = cdata;
+        in.len = strlen(cdata);
+        value = svn_base64_decode_string(&in, pc->pool);
+      }
+
+      pc->encoding = NULL; /* Reset encoding for future attribute(s). */
     }
 
   /*** Handling resource properties from here out. ***/
@@ -447,7 +452,7 @@ static int end_element(void *userdata,
      <propstat>, we'll either dump the props as invalid or move them
      into the resource's property hash. */
   apr_hash_set(pc->propbuffer, name, APR_HASH_KEY_STRING, value);
-  return 0;
+  return SVN_RA_DAV__XML_VALID;
 }
 
 
@@ -519,11 +524,12 @@ svn_error_t * svn_ra_dav__get_props(apr_hash_t **results,
   pc.props = apr_hash_make(pool);
 
   /* Create and dispatch the request! */
-  err = svn_ra_dav__parsed_request(sess, "PROPFIND", url, body->data, 0, 
-                                   set_parser, propfind_elements, 
-                                   validate_element, 
-                                   start_element, end_element, 
-                                   &pc, extra_headers, pool);
+  err = svn_ra_dav__parsed_request_compat(sess, "PROPFIND", url,
+                                          body->data, 0, 
+                                          set_parser, propfind_elements, 
+                                          validate_element, 
+                                          start_element, end_element, 
+                                          &pc, extra_headers, NULL, pool);
 
   ne_buffer_destroy(body);
   *results = pc.props;
@@ -574,7 +580,7 @@ svn_error_t * svn_ra_dav__get_props_resource(svn_ra_dav_resource_t **rsrc,
     {
       /* ### hmmm, should have been in there... */
       return svn_error_createf(APR_EGENERAL, NULL,
-                               "failed to find label \"%s\" for url \"%s\"",
+                               _("Failed to find label '%s' for URL '%s'"),
                                label ? label : "NULL", url_path);
     }
 
@@ -603,7 +609,8 @@ svn_error_t * svn_ra_dav__get_one_prop(const svn_string_t **propval,
     {
       /* ### need an SVN_ERR here */
       return svn_error_createf(SVN_ERR_RA_DAV_PROPS_NOT_FOUND, NULL,
-                               "'%s' was not present on the resource.", name);
+                               _("'%s' was not present on the resource"), 
+                               name);
     }
 
   *propval = value;
@@ -621,37 +628,29 @@ svn_error_t * svn_ra_dav__get_starting_props(svn_ra_dav_resource_t **rsrc,
 }
 
 
-/* Shared helper func: given a public URL which may not exist in HEAD,
-   use SESS to search up parent directories until we can retrieve a
-   *RSRC (allocated in POOL) containing a standard set of "starting"
-   props: {VCC, resourcetype, baseline-relative-path}.  
 
-   Also return *MISSING_PATH (allocated in POOL), which is the
-   trailing portion of the URL that did not exist.  If an error
-   occurs, *MISSING_PATH isn't changed. */
-static svn_error_t * search_for_starting_props(svn_ra_dav_resource_t **rsrc,
-                                               const char **missing_path,
-                                               ne_session *sess,
-                                               const char *url,
-                                               apr_pool_t *pool)
+svn_error_t * 
+svn_ra_dav__search_for_starting_props(svn_ra_dav_resource_t **rsrc,
+                                      const char **missing_path,
+                                      ne_session *sess,
+                                      const char *url,
+                                      apr_pool_t *pool)
 {
-  svn_error_t *err;
+  svn_error_t *err = SVN_NO_ERROR;
   apr_size_t len;
   svn_stringbuf_t *path_s;
   ne_uri parsed_url;
   const char *lopped_path = "";
 
-  /* Split the url into it's component pieces (schema, host, path,
+  /* Split the url into its component pieces (schema, host, path,
      etc).  We want the path part. */
   ne_uri_parse (url, &parsed_url);
-
   path_s = svn_stringbuf_create (parsed_url.path, pool);
 
   /* Try to get the starting_props from the public url.  If the
      resource no longer exists in HEAD, we'll get a failure.  That's
      fine: just keep removing components and trying to get the
      starting_props from parent directories. */
-  
   while (! svn_path_is_empty (path_s->data))
     {
       err = svn_ra_dav__get_starting_props(rsrc, sess, path_s->data,
@@ -659,37 +658,40 @@ static svn_error_t * search_for_starting_props(svn_ra_dav_resource_t **rsrc,
       if (! err)
         break;   /* found an existing parent! */
       
-      if (err->apr_err != SVN_ERR_RA_DAV_REQUEST_FAILED)
-        return err;  /* found a _real_ error */
+      if (err->apr_err != SVN_ERR_RA_DAV_PATH_NOT_FOUND)
+        goto error;  /* found a _real_ error */
 
       /* else... lop off the basename and try again. */
       lopped_path = svn_path_join(svn_path_basename (path_s->data, pool),
-                                  lopped_path,
-                                  pool);
+                                  lopped_path, pool);
       len = path_s->len;
       svn_path_remove_component(path_s);
+
+      /* if we detect an infinite loop, get out. */
       if (path_s->len == len)          
-        /* whoa, infinite loop, get out. */
-        return svn_error_quick_wrap(err,
-                                    "The path was not part of a repository");
-      
+        {
+          err = svn_error_quick_wrap
+            (err, _("The path was not part of a repository"));
+          goto error;
+        }
       svn_error_clear (err);
     }
 
+  /* error out if entire URL was bogus (not a single part of it exists
+     in the repository!)  */
   if (svn_path_is_empty (path_s->data))
     {
-      /* entire URL was bogus;  not a single part of it exists in
-         the repository!  */
-      err = svn_error_createf(SVN_ERR_RA_ILLEGAL_URL, NULL,
-                              "No part of path '%s' was found in "
-                              "repository HEAD.", parsed_url.path);
-      ne_uri_free(&parsed_url);
-      return err;
+      err = svn_error_createf (SVN_ERR_RA_ILLEGAL_URL, NULL,
+                               _("No part of path '%s' was found in "
+                                 "repository HEAD"), parsed_url.path);
+      goto error;
     }
-  ne_uri_free(&parsed_url);
-  
+
   *missing_path = lopped_path;
-  return SVN_NO_ERROR;
+
+ error:
+  ne_uri_free(&parsed_url);
+  return err;
 }
 
 
@@ -707,15 +709,15 @@ svn_error_t *svn_ra_dav__get_vcc(const char **vcc,
   /* ### Someday, possibly look for disk-cached VCC via get_wcprop callback. */
 
   /* Finally, resort to a set of PROPFINDs up parent directories. */
-  SVN_ERR( search_for_starting_props(&rsrc, &lopped_path,
-                                     sess, url, pool) );
+  SVN_ERR( svn_ra_dav__search_for_starting_props(&rsrc, &lopped_path,
+                                                 sess, url, pool) );
 
   vcc_s = apr_hash_get(rsrc->propset,
                        SVN_RA_DAV__PROP_VCC, APR_HASH_KEY_STRING);
   if (! vcc_s)
     return svn_error_create(APR_EGENERAL, NULL,
-                             "The VCC property was not found on the "
-                             "resource.");
+                             _("The VCC property was not found on the "
+                               "resource"));
 
   *vcc = vcc_s->data;
   return SVN_NO_ERROR;
@@ -759,8 +761,8 @@ svn_error_t *svn_ra_dav__get_baseline_props(svn_string_t *bc_relative,
         parent directories.
   */
 
-  SVN_ERR( search_for_starting_props(&rsrc, &lopped_path,
-                                     sess, url, pool) );
+  SVN_ERR( svn_ra_dav__search_for_starting_props(&rsrc, &lopped_path,
+                                                 sess, url, pool) );
   
   vcc = apr_hash_get(rsrc->propset, SVN_RA_DAV__PROP_VCC, APR_HASH_KEY_STRING);
   if (vcc == NULL)
@@ -769,8 +771,8 @@ svn_error_t *svn_ra_dav__get_baseline_props(svn_string_t *bc_relative,
 
       /* ### need an SVN_ERR here */
       return svn_error_create(APR_EGENERAL, NULL,
-                              "The VCC property was not found on the "
-                              "resource.");
+                              _("The VCC property was not found on the "
+                                "resource"));
     }
 
   /* Allocate our own bc_relative path. */
@@ -782,13 +784,16 @@ svn_error_t *svn_ra_dav__get_baseline_props(svn_string_t *bc_relative,
       /* ### better error reporting... */        
       /* ### need an SVN_ERR here */
       return svn_error_create(APR_EGENERAL, NULL,
-                              "The relative-path property was not "
-                              "found on the resource.");
+                              _("The relative-path property was not "
+                                "found on the resource"));
     }
     
-  /* don't forget to tack on the parts we lopped off in order
-     to find the VCC... */
-  my_bc_relative = svn_path_join(relative_path->data, lopped_path, pool);
+  /* don't forget to tack on the parts we lopped off in order to find
+     the VCC...  We are expected to return a URI decoded relative
+     path, so decode the lopped path first. */
+  my_bc_relative = svn_path_join(relative_path->data,
+                                 svn_path_uri_decode (lopped_path, pool),
+                                 pool);
  
   /* if they want the relative path (could be, they're just trying to find
      the baseline collection), then return it */
@@ -848,7 +853,7 @@ svn_error_t *svn_ra_dav__get_baseline_props(svn_string_t *bc_relative,
 
       /* ### send Label hdr, get DAV:baseline-collection [from the baseline] */
 
-      apr_snprintf(label, sizeof(label), "%" SVN_REVNUM_T_FMT, revision);
+      apr_snprintf(label, sizeof(label), "%ld", revision);
 
       /* ### do we want to optimize the props we fetch, based on what the
          ### user asked for? i.e. omit version-name if latest_rev is NULL */
@@ -900,8 +905,8 @@ svn_error_t *svn_ra_dav__get_baseline_info(svn_boolean_t *is_dir,
       /* ### better error reporting... */
       /* ### need an SVN_ERR here */
       return svn_error_create(APR_EGENERAL, NULL,
-                              "DAV:baseline-collection was not present "
-                              "on the baseline resource.");
+                              _("'DAV:baseline-collection' was not present "
+                                "on the baseline resource"));
     }
 
   /* maybe return bc_url to the caller */
@@ -919,8 +924,8 @@ svn_error_t *svn_ra_dav__get_baseline_info(svn_boolean_t *is_dir,
 
           /* ### need an SVN_ERR here */
           return svn_error_create(APR_EGENERAL, NULL,
-                                  "DAV:version-name was not present on the "
-                                  "baseline resource.");
+                                  _("'DAV:version-name' was not present"
+                                    " on the baseline resource"));
         }
       *latest_rev = SVN_STR_TO_REV(vsn_name->data);
     }
@@ -984,13 +989,9 @@ do_setprop(ne_buffer *body,
     }
   else
     {
-#ifdef SVN_DAV_FEATURE_BINARY_PROPS
       const svn_string_t *base64ed = svn_base64_encode_string(value, pool);
       encoding = " V:encoding=\"base64\"";
       xml_safe = base64ed->data;
-#else /* SVN_DAV_FEATURE_BINARY_PROPS */
-      xml_safe = value->data;
-#endif /* SVN_DAV_FEATURE_BINARY_PROPS */
     }
 
   ne_buffer_concat(body, "<", xml_tag_name, encoding, ">", 
@@ -1064,12 +1065,20 @@ svn_ra_dav__do_proppatch (svn_ra_session_t *ras,
   ne_set_request_body_buffer(req, body->data, ne_buffer_size(body));
   ne_add_request_header(req, "Content-Type", "text/xml; charset=UTF-8");
 
-  /* run the request and get the resulting status code (and svn_error_t) */
-  err = svn_ra_dav__request_dispatch(&code, req, ras->sess, "PROPPATCH",
-                                     url,
-                                     207 /* Multistatus */,
-                                     0 /* nothing else allowed */,
-                                     pool);
+  code = ne_simple_request(ras->sess, req);
+
+  if (code == NE_OK)
+    {
+      err = SVN_NO_ERROR;
+    }
+  else
+    {
+      /* WebDAV spec says that if any part of a PROPPATCH fails, the
+         entire 'commit' is rejected.  */
+      err = svn_error_create
+        (SVN_ERR_RA_DAV_PROPPATCH_FAILED, NULL,
+         _("At least one property change failed; repository is unchanged"));
+    }
 
   ne_buffer_destroy(body);
   return err;
@@ -1078,10 +1087,10 @@ svn_ra_dav__do_proppatch (svn_ra_session_t *ras,
 
 
 svn_error_t *
-svn_ra_dav__do_check_path(svn_node_kind_t *kind,
-                          void *session_baton,
+svn_ra_dav__do_check_path(void *session_baton,
                           const char *path,
                           svn_revnum_t revision,
+                          svn_node_kind_t *kind,
                           apr_pool_t *pool)
 {
   svn_ra_session_t *ras = session_baton;
@@ -1131,23 +1140,8 @@ svn_ra_dav__do_check_path(svn_node_kind_t *kind,
       else
         *kind = svn_node_file;
     }
-  else  /* some error, read the comment below */
+  else if (err->apr_err == SVN_ERR_RA_DAV_PATH_NOT_FOUND)
     {
-      /* ### This is way too general.  We should only convert the
-       * error to `svn_node_none' if we're sure that's what the error
-       * means; for example, the test used to be this
-       *
-       *   (err && (err->apr_err == SVN_ERR_RA_DAV_PROPS_NOT_FOUND))
-       *
-       * which seemed reasonable...
-       *
-       * However, right now svn_ra_dav__get_props() returns a generic
-       * error when the entity doesn't exist.  It's APR_EGENERAL or
-       * something like that, and ne_get_status(req)->code == 500, not
-       * 404.  I don't know whether this is something that can be
-       * improved just in that function, or if the server will need to
-       * be more descriptive about the error.  Greg, thoughts?
-       */
 
       svn_error_clear (err);
       *kind = svn_node_none;
