@@ -157,28 +157,6 @@ svn_config__open_file (FILE **pfile,
                        const char *mode,
                        apr_pool_t *pool);
 
-/** A callback function used in enumerating config sections.
- *
- * See @c svn_config_enumerate_sections for the details of this type.
- */
-typedef svn_boolean_t (*svn_config__section_enumerator_t)
-       (const char *name, void *baton);
-
-/** Enumerate the sections, passing @a baton and the current section's name to
- * @a callback.  Continue the enumeration if @a callback returns @c TRUE.
- * Return the number of times @a callback was called.
- *
- * ### See kff's comment to @c svn_config_enumerate.  It applies to this
- * function, too. ###
- *
- * @a callback's @a name and @a name parameters are only valid for the
- * duration of the call.          
- */
-int svn_config__enumerate_sections (svn_config_t *cfg,
-                                   svn_config__section_enumerator_t callback,
-                                   void *baton);
-
-
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
