@@ -21,6 +21,24 @@
 #include <stdio.h>
 #include <assert.h>
 
+#ifndef WIN32
+#include <unistd.h>
+#ifndef APR_GSETID
+/* Needed for fallback setgid code in dir_make */
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <errno.h>
+#endif
+#endif
+
+#ifndef APR_STATUS_IS_EPERM
+#ifdef EPERM
+#define APR_STATUS_IS_EPERM(s)   ((s) == EPERM)
+#else
+#define APR_STATUS_IS_EPERM(s)   (0)
+#endif
+#endif
+
 #include <apr_lib.h>
 #include <apr_pools.h>
 #include <apr_file_io.h>
@@ -39,6 +57,7 @@
 #include "svn_pools.h"
 #include "svn_utf.h"
 #include "svn_config.h"
+#include "svn_private_config.h"
 
 
 /*
@@ -80,6 +99,7 @@
 static svn_error_t *
 io_check_path (const char *path,
                svn_boolean_t resolve_symlinks,
+               svn_boolean_t *is_special_p,
                svn_node_kind_t *kind,
                apr_pool_t *pool)
 {
@@ -87,6 +107,7 @@ io_check_path (const char *path,
   apr_finfo_t finfo;
   apr_status_t apr_err;
   const char *path_apr;
+  svn_boolean_t is_special = FALSE;
 
   /* Make path appropriate for error messages in advance. */
   path = svn_path_local_style (path, pool);
@@ -97,7 +118,7 @@ io_check_path (const char *path,
 
   flags = resolve_symlinks ? APR_FINFO_MIN : (APR_FINFO_MIN | APR_FINFO_LINK);
   apr_err = apr_stat (&finfo, path_apr, flags, pool);
-
+  
   if (APR_STATUS_IS_ENOENT (apr_err))
     *kind = svn_node_none;
   else if (APR_STATUS_IS_ENOTDIR (apr_err))
@@ -110,13 +131,16 @@ io_check_path (const char *path,
     *kind = svn_node_file;
   else if (finfo.filetype == APR_DIR)
     *kind = svn_node_dir;
-#if 0
-  else if (finfo.filetype == APR_LINK)
-    *kind = svn_node_symlink;  /* we support symlinks someday, but not yet */
-#endif /* 0 */
+  else if (finfo.filetype == APR_LNK)
+    {
+      is_special = TRUE;
+      *kind = svn_node_file;
+    }
   else
     *kind = svn_node_unknown;
 
+  *is_special_p = is_special;
+  
   return SVN_NO_ERROR;
 }
 
@@ -126,7 +150,8 @@ svn_io_check_resolved_path (const char *path,
                             svn_node_kind_t *kind,
                             apr_pool_t *pool)
 {
-  return io_check_path (path, TRUE, kind, pool);
+  svn_boolean_t ignored;
+  return io_check_path (path, TRUE, &ignored, kind, pool);
 }
 
 svn_error_t *
@@ -134,9 +159,18 @@ svn_io_check_path (const char *path,
                    svn_node_kind_t *kind,
                    apr_pool_t *pool)
 {
-  return io_check_path (path, FALSE, kind, pool);
+  svn_boolean_t ignored;
+  return io_check_path (path, FALSE, &ignored, kind, pool);
 }
 
+svn_error_t *
+svn_io_check_special_path (const char *path,
+                           svn_node_kind_t *kind,
+                           svn_boolean_t *is_special,
+                           apr_pool_t *pool)
+{
+  return io_check_path (path, FALSE, is_special, kind, pool);
+}
 
 svn_error_t *
 svn_io_open_unique_file (apr_file_t **f,
@@ -166,7 +200,7 @@ svn_io_open_unique_file (apr_file_t **f,
          conditional, but it's not worth the clarity loss.)
 
          If the first attempt fails, the first number will be "2".
-         This is good, since "1" would misleadingly imply that that
+         This is good, since "1" would misleadingly imply that
          the second attempt was actually the first... and if someone's
          got conflicts on their conflicts, we probably don't want to
          add to their confusion :-). */
@@ -198,7 +232,7 @@ svn_io_open_unique_file (apr_file_t **f,
               apr_status_t apr_err_2 = apr_stat (&finfo, unique_name_apr,
                                                  APR_FINFO_TYPE, pool);
 
-              if (APR_STATUS_IS_SUCCESS (apr_err_2)
+              if (!apr_err_2
                   && (finfo.filetype == APR_DIR))
                 continue;
 
@@ -224,6 +258,150 @@ svn_io_open_unique_file (apr_file_t **f,
                             "Unable to make name for '%s'", path);
 }
 
+svn_error_t *
+svn_io_create_unique_link (const char **unique_name_p,
+                           const char *path,
+                           const char *dest,
+                           const char *suffix,
+                           apr_pool_t *pool)
+{
+#ifdef HAVE_SYMLINK  
+  unsigned int i;
+  const char *unique_name;
+  const char *unique_name_apr;
+  int rv;
+
+  for (i = 1; i <= 99999; i++)
+    {
+      apr_status_t apr_err;
+
+      /* Special case the first attempt -- if we can avoid having a
+         generated numeric portion at all, that's best.  So first we
+         try with just the suffix; then future tries add a number
+         before the suffix.  (A do-while loop could avoid the repeated
+         conditional, but it's not worth the clarity loss.)
+
+         If the first attempt fails, the first number will be "2".
+         This is good, since "1" would misleadingly imply that
+         the second attempt was actually the first... and if someone's
+         got conflicts on their conflicts, we probably don't want to
+         add to their confusion :-). */
+      if (i == 1)
+        unique_name = apr_psprintf (pool, "%s%s", path, suffix);
+      else
+        unique_name = apr_psprintf (pool, "%s.%u%s", path, i, suffix);
+
+      /* Hmmm.  Ideally, we would append to a native-encoding buf
+         before starting iteration, then convert back to UTF-8 for
+         return. But I suppose that would make the appending code
+         sensitive to i18n in a way it shouldn't be... Oh well. */
+      SVN_ERR (svn_path_cstring_from_utf8 (&unique_name_apr, unique_name,
+                                           pool));
+
+      do {
+        rv = symlink (dest, unique_name_apr);
+      } while (rv == -1 && APR_STATUS_IS_EINTR (apr_get_os_error ()));
+      
+      apr_err = apr_get_os_error();
+      
+      if (rv == -1 && APR_STATUS_IS_EEXIST (apr_err))
+        continue;
+      else if (rv == -1 && apr_err)
+        {
+          /* On Win32, CreateFile failswith an "Access Denied" error
+             code, rather than "File Already Exists", if the colliding
+             name belongs to a directory. */
+          if (APR_STATUS_IS_EACCES (apr_err))
+            {
+              apr_finfo_t finfo;
+              apr_status_t apr_err_2 = apr_stat (&finfo, unique_name_apr,
+                                                 APR_FINFO_TYPE, pool);
+
+              if (!apr_err_2
+                  && (finfo.filetype == APR_DIR))
+                continue;
+
+              /* Else ignore apr_err_2; better to fall through and
+                 return the original error. */
+            }
+
+          *unique_name_p = NULL;
+          return svn_error_wrap_apr (apr_err, "Can't open '%s'", unique_name);
+        }
+      else
+        {
+          *unique_name_p = unique_name;
+          return SVN_NO_ERROR;
+        }
+    }
+
+  *unique_name_p = NULL;
+  return svn_error_createf (SVN_ERR_IO_UNIQUE_NAMES_EXHAUSTED,
+                            NULL,
+                            "Unable to make name for '%s'", path);
+#else
+  return svn_error_create (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
+                           "Symbolic links are not supported on this "
+                           "platform");
+#endif  
+}
+
+svn_error_t *
+svn_io_read_link (svn_string_t **dest,
+                  const char *path,
+                  apr_pool_t *pool)
+{
+#ifdef HAVE_READLINK  
+  char buf[1024];
+  int rv;
+  
+  do {
+    rv = readlink (path, buf, sizeof(buf));
+  } while (rv == -1 && APR_STATUS_IS_EINTR (apr_get_os_error ()));
+
+  if (rv == -1)
+    return svn_error_wrap_apr
+      (apr_get_os_error (), "Can't read contents of link");
+
+  *dest = svn_string_ncreate (buf, rv, pool);
+  
+  return SVN_NO_ERROR;
+#else
+  return svn_error_create (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
+                           "Symbolic links are not supported on this "
+                           "platform");
+#endif  
+}
+
+
+svn_error_t *
+svn_io_copy_link (const char *src,
+                  const char *dst,
+                  apr_pool_t *pool)
+
+{
+#ifdef HAVE_READLINK
+  svn_string_t *link_dest;
+  const char *dst_tmp;
+
+  /* Notice what the link is pointing at... */
+  SVN_ERR (svn_io_read_link (&link_dest, src, pool));
+
+  /* Make a tmp-link pointing at the same thing. */
+  SVN_ERR (svn_io_create_unique_link (&dst_tmp, dst, link_dest->data,
+                                      ".tmp", pool));
+  
+  /* Move the tmp-link to link. */
+  return svn_io_file_rename (dst_tmp, dst, pool);
+
+#else
+  return svn_error_create (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
+                           "Symbolic links are not supported on this "
+                           "platform");
+#endif
+}
+
+
 #if 1 /* TODO: Remove this code when APR 0.9.6 is released. */
 #include "apr_env.h"
 
@@ -434,10 +612,10 @@ svn_error_t *svn_io_copy_dir_recursively (const char *src,
 {
   svn_node_kind_t kind;
   apr_status_t status;
-  apr_hash_t *dirents;
-  apr_hash_index_t *hi;
   const char *dst_path;
-  const char *dst_path_apr;
+  apr_dir_t *this_dir;
+  apr_finfo_t this_entry;
+  apr_int32_t flags = APR_FINFO_TYPE | APR_FINFO_NAME;
 
   /* Make a subpool for recursion */
   apr_pool_t *subpool = svn_pool_create (pool);
@@ -465,61 +643,81 @@ svn_error_t *svn_io_copy_dir_recursively (const char *src,
                               "Destination '%s' already exists",
                               dst_path);
   
-  SVN_ERR (svn_path_cstring_from_utf8 (&dst_path_apr, dst_path, pool));
-
   /* Create the new directory. */
   /* ### TODO: copy permissions? */
-  status = apr_dir_make (dst_path_apr, APR_OS_DEFAULT, pool);
-  if (status)
-    return svn_error_createf (status, NULL,
-                              "Unable to create directory '%s'",
-                              dst_path);
+  SVN_ERR (svn_io_dir_make (dst_path, APR_OS_DEFAULT, pool));
 
   /* Loop over the dirents in SRC.  ('.' and '..' are auto-excluded) */
-  SVN_ERR (svn_io_get_dirents (&dirents, src, subpool));
+  SVN_ERR (svn_io_dir_open (&this_dir, src, subpool));
 
-  for (hi = apr_hash_first (subpool, dirents); hi; hi = apr_hash_next (hi))
+  for (status = apr_dir_read (&this_entry, flags, this_dir);
+       status == APR_SUCCESS;
+       status = apr_dir_read (&this_entry, flags, this_dir))
     {
-      const void *key;
-      void *val;
-      const char *entryname;
-      svn_node_kind_t *entrykind;
-      const char *src_target;
-
-      /* Get next entry and its kind */
-      apr_hash_this (hi, &key, NULL, &val);
-      entryname = key;
-      entrykind = val;
-
-      if (cancel_func)
-        SVN_ERR (cancel_func (cancel_baton));
-
-      /* Telescope the entryname onto the source dir. */
-      src_target = svn_path_join (src, entryname, subpool);
-
-      /* If it's a file, just copy it over. */
-      if (*entrykind == svn_node_file)
+      if ((this_entry.name[0] == '.')
+          && ((this_entry.name[1] == '\0')
+              || ((this_entry.name[1] == '.')
+                  && (this_entry.name[2] == '\0'))))
         {
-          /* Telescope and de-telescope the dst_target in here */
-          const char *dst_target
-            = svn_path_join (dst_path, entryname, subpool);
-          SVN_ERR (svn_io_copy_file (src_target, dst_target,
-                                     copy_perms, subpool));
+          continue;
         }
-      else if (*entrykind == svn_node_dir)  /* recurse */
+      else
         {
-          SVN_ERR (svn_io_copy_dir_recursively (src_target,
-                                                dst_path,
-                                                entryname,
-                                                copy_perms,
-                                                cancel_func,
-                                                cancel_baton,
-                                                subpool));
-        }
+          const char *src_target, *src_target_utf8, *dst_target_utf8;
 
-      /* ### someday deal with other node kinds? */
+          if (cancel_func)
+            SVN_ERR (cancel_func (cancel_baton));
+
+          /* Telescope the entryname onto the source dir. */
+          src_target = svn_path_join (src, this_entry.name, subpool);
+          SVN_ERR (svn_path_cstring_to_utf8 (&src_target_utf8, src_target,
+                                             subpool));
+          
+          if (this_entry.filetype == APR_REG) /* regular file */
+            {
+              const char *dst_target = svn_path_join (dst_path,
+                                                      this_entry.name,
+                                                      subpool);
+              SVN_ERR (svn_path_cstring_to_utf8 (&dst_target_utf8, dst_target,
+                                                 subpool));
+              SVN_ERR (svn_io_copy_file (src_target_utf8, dst_target_utf8,
+                                         copy_perms, subpool));
+            }
+          else if (this_entry.filetype == APR_LNK) /* symlink */
+            {
+              const char *dst_target = svn_path_join (dst_path,
+                                                      this_entry.name,
+                                                      subpool);
+              SVN_ERR (svn_path_cstring_to_utf8 (&dst_target_utf8, dst_target,
+                                                 subpool));
+              SVN_ERR (svn_io_copy_link (src_target_utf8, dst_target_utf8,
+                                         subpool));
+            }
+          else if (this_entry.filetype == APR_DIR) /* recurse */
+            {
+              const char *entryname_utf8;
+              SVN_ERR (svn_path_cstring_to_utf8 (&entryname_utf8,
+                                                 this_entry.name, subpool));
+              SVN_ERR (svn_io_copy_dir_recursively 
+                       (src_target_utf8,
+                        dst_path,
+                        entryname_utf8,
+                        copy_perms,
+                        cancel_func,
+                        cancel_baton,
+                        subpool));
+            }
+          /* ### support other APR node types someday?? */
+
+        }
     }
-    
+
+  if (! (APR_STATUS_IS_ENOENT (status)))
+    return svn_error_wrap_apr (status, "Can't read directory '%s'", src);
+
+  status = apr_dir_close (this_dir);
+  if (status)
+    return svn_error_wrap_apr (status, "Error closing directory '%s'", src);
 
   /* Free any memory used by recursion */
   apr_pool_destroy (subpool);
@@ -531,8 +729,7 @@ svn_error_t *svn_io_copy_dir_recursively (const char *src,
 svn_error_t *
 svn_io_make_dir_recursively (const char *path, apr_pool_t *pool)
 {
-  const char *path_apr;
-  apr_status_t apr_err;
+  svn_error_t *err;
   char *dir;
 
   if (svn_path_is_empty (path))
@@ -540,8 +737,6 @@ svn_io_make_dir_recursively (const char *path, apr_pool_t *pool)
        so we do nothing, per docs. */
     return SVN_NO_ERROR;
 
-  SVN_ERR (svn_path_cstring_from_utf8 (&path_apr, path, pool));
-
 #if 0
   /* ### Use this implementation if/when apr_dir_make_recursive is
      available on all platforms, not just on Unix. --xbc */
@@ -554,25 +749,23 @@ svn_io_make_dir_recursively (const char *path, apr_pool_t *pool)
 #else
 
   /* Try to make PATH right out */
-  apr_err = apr_dir_make (path_apr, APR_OS_DEFAULT, pool);
+  err = svn_io_dir_make (path, APR_OS_DEFAULT, pool);
 
-  /* It's OK if PATH exists */
-  if (!apr_err || APR_STATUS_IS_EEXIST(apr_err))
-    return SVN_NO_ERROR;
-
-  if (APR_STATUS_IS_ENOENT(apr_err))
+  if (! err || APR_STATUS_IS_EEXIST (err->apr_err))
     {
-      /* Missing an intermediate dir */
+      /* We succeeded, or path already exists; either way we're done. */
+      svn_error_clear (err);
+      return SVN_NO_ERROR;
+    }
+  else if (APR_STATUS_IS_ENOENT (err->apr_err))
+    {
+      /* Missing an intermediate dir. */
       dir = svn_path_dirname (path, pool);
       SVN_ERR (svn_io_make_dir_recursively (dir, pool));
-
-      apr_err = apr_dir_make (path_apr, APR_OS_DEFAULT, pool);
-      if (!apr_err)
-        return SVN_NO_ERROR;
+      return svn_io_dir_make (path, APR_OS_DEFAULT, pool);
     }
-
-  /* If we get here, there must be an apr_err. */
-  return svn_error_wrap_apr (apr_err, "Can't make '%s'", path);
+  else
+    return err;
 #endif
 }
 
@@ -617,7 +810,7 @@ svn_io_file_affected_time (apr_time_t *apr_time,
 {
   apr_finfo_t finfo;
 
-  SVN_ERR (svn_io_stat (&finfo, path, APR_FINFO_MIN, pool));
+  SVN_ERR (svn_io_stat (&finfo, path, APR_FINFO_MIN | APR_FINFO_LINK, pool));
 
   *apr_time = finfo.mtime;
 
@@ -782,6 +975,27 @@ svn_io_set_file_read_write (const char *path,
   return SVN_NO_ERROR;
 }
 
+/* Given the file specified by PATH_APR, attempt to create an
+   identical version of it owned by the current user.  This is done by
+   moving it to a temporary location, copying the file back to its old
+   path, then deleting the temporarily moved version.  All temporary
+   allocations are done in POOL. */
+static svn_error_t *
+reown_file (const char *path_apr,
+            apr_pool_t *pool)
+{
+  apr_file_t *fp;
+  const char *unique_name;
+
+  SVN_ERR (svn_io_open_unique_file (&fp, &unique_name, path_apr,
+                                    ".tmp", FALSE, pool));
+  SVN_ERR (svn_io_file_close (fp, pool));
+  SVN_ERR (svn_io_file_rename (path_apr, unique_name, pool));
+  SVN_ERR (svn_io_copy_file (unique_name, path_apr, TRUE, pool));
+  SVN_ERR (svn_io_remove_file (unique_name, pool));
+
+  return SVN_NO_ERROR;
+}
 
 svn_error_t *
 svn_io_set_file_executable (const char *path,
@@ -831,12 +1045,33 @@ svn_io_set_file_executable (const char *path,
           status = apr_file_perms_set (path_apr, perms_to_set);
           if (status)
             {
-              if (ignore_enoent && APR_STATUS_IS_ENOENT (status))
+              if (APR_STATUS_IS_EPERM (status))
+                {
+                  /* We don't have permissions to change the
+                     permissions!  Try a move, copy, and delete
+                     workaround to see if we can get the file owned by
+                     us.  If these succeed, try the permissions set
+                     again.
+
+                     Note that we only attempt this in the
+                     stat-available path.  This assumes that the
+                     move-copy workaround will only be helpful on
+                     platforms that implement apr_stat. */
+                  SVN_ERR (reown_file (path_apr, pool));
+                  status = apr_file_perms_set (path_apr, perms_to_set);
+                }
+
+              if (status)
+                {
+                  if (ignore_enoent && APR_STATUS_IS_ENOENT (status))
+                    return SVN_NO_ERROR;
+                  else if (status != APR_ENOTIMPL)
+                    return svn_error_wrap_apr (status,
+                                               "Can't change executability of "
+                                               "file '%s'", path);
+                }
+              else
                 return SVN_NO_ERROR;
-              else if (status != APR_ENOTIMPL)
-                return svn_error_wrap_apr (status,
-                                           "Can't change executability of "
-                                           "file '%s'", path);
             }
           else
             return SVN_NO_ERROR;
@@ -928,6 +1163,14 @@ svn_io__file_clear_and_close (void *arg)
 svn_error_t *svn_io_file_lock (const char *lock_file,
                                svn_boolean_t exclusive,
                                apr_pool_t *pool)
+{
+  return svn_io_file_lock2(lock_file, exclusive, FALSE, pool);
+}
+
+svn_error_t *svn_io_file_lock2 (const char *lock_file,
+                                svn_boolean_t exclusive,
+                                svn_boolean_t nonblocking,
+                                apr_pool_t *pool)
 {
   int locktype = APR_FLOCK_SHARED;
   apr_file_t *lockfile_handle;
@@ -941,6 +1184,9 @@ svn_error_t *svn_io_file_lock (const char *lock_file,
   if (locktype == APR_FLOCK_EXCLUSIVE)
     flags |= APR_WRITE;
 
+  if (nonblocking == TRUE)
+    locktype |= APR_FLOCK_NONBLOCK;
+
   SVN_ERR (svn_io_file_open (&lockfile_handle, lock_file, flags,
                              APR_OS_DEFAULT,
                              pool));
@@ -967,6 +1213,56 @@ svn_error_t *svn_io_file_lock (const char *lock_file,
 }
 
 
+
+/* Data consistency/coherency operations. */
+
+static svn_error_t *
+do_io_file_wrapper_cleanup (apr_file_t *file, apr_status_t status, 
+                            const char *op, apr_pool_t *pool);
+
+svn_error_t *svn_io_file_flush_to_disk (apr_file_t *file,
+                                        apr_pool_t *pool)
+{
+  apr_os_file_t filehand;
+
+  /* First make sure that any user-space buffered data is flushed. */
+  SVN_ERR (do_io_file_wrapper_cleanup (file, apr_file_flush (file),
+                                       "flush", pool));
+
+  apr_os_file_get (&filehand, file);
+    
+  /* Call the operating system specific function to actually force the
+     data to disk. */
+  {
+#ifdef WIN32
+      
+      if (! FlushFileBuffers (filehand))
+        return svn_error_wrap_apr
+          (apr_get_os_error (), "Can't flush file to disk.");
+      
+#else
+      int rv;
+
+      do {
+        rv = fsync (filehand);
+      } while (rv == -1 && APR_STATUS_IS_EINTR (apr_get_os_error ()));
+
+      /* If the file is in a memory filesystem, fsync() may return
+         EINVAL.  Presumably the user knows the risks, and we can just
+         ignore the error. */
+      if (rv == -1 && APR_STATUS_IS_EINVAL (apr_get_os_error ()))
+        return SVN_NO_ERROR;
+
+      if (rv == -1)
+        return svn_error_wrap_apr
+          (apr_get_os_error (), "Can't flush file to disk.");
+
+#endif
+  }
+  return SVN_NO_ERROR;
+}
+    
+
 
 /* TODO write test for these two functions, then refactor. */
 
@@ -980,7 +1276,6 @@ svn_stringbuf_from_file (svn_stringbuf_t **result,
   if (filename[0] == '-' && filename[1] == '\0')
     return svn_error_create
         (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-         "svn_stringbuf_from_file: "
          "Reading from stdin is currently broken, so disabled");
 
   SVN_ERR (svn_io_file_open (&f, filename, APR_READ, APR_OS_DEFAULT, pool));
@@ -1076,11 +1371,41 @@ svn_io_remove_file (const char *path, apr_pool_t *pool)
 }
 
 
+/*
+ Mac OS X has a bug where if you're readding the contents of a
+ directory via readdir in a loop, and you remove one of the entries in
+ the directory and the directory has 338 or more files in it you will
+ skip over some of the entries in the directory.  Needless to say,
+ this causes problems if you are using this kind of loop inside a
+ function that is recursively deleting a directory, because when you
+ get around to removing the directory it will still have something in
+ it.
+
+ This works around the problem by inserting a rewinddir after we
+ remove each item in the directory, which makes the problem go away.
+
+ See http://subversion.tigris.org/issues/show_bug.cgi?id=1896 for more
+ discussion.
+*/
+#if defined(__APPLE__) && defined(__MACH__)
+#define MACOSX_REWINDDIR_HACK(dir, path)                                   \
+  do                                                                       \
+    {                                                                      \
+      apr_status_t apr_err  = apr_dir_rewind (dir);                        \
+      if (apr_err)                                                         \
+        return svn_error_wrap_apr (apr_err, "Can't rewind directory '%s'", \
+                                   path);                                  \
+    }                                                                      \
+  while (0)
+#else
+#define MACOSX_REWINDDIR_HACK(dir, path) do {} while (0)
+#endif
+
+
 /* Neither windows nor unix allows us to delete a non-empty
    directory.  
 
    This is a function to perform the equivalent of 'rm -rf'. */
-
 svn_error_t *
 svn_io_remove_dir (const char *path, apr_pool_t *pool)
 {
@@ -1125,10 +1450,12 @@ svn_io_remove_dir (const char *path, apr_pool_t *pool)
                                              subpool));
           
           fullpath = svn_path_join (path, entry_utf8, pool);
-          
+
           if (this_entry.filetype == APR_DIR)
             {
               SVN_ERR (svn_io_remove_dir (fullpath, subpool));
+
+              MACOSX_REWINDDIR_HACK (this_dir, path);
             }
           else if (this_entry.filetype == APR_REG)
             {
@@ -1138,6 +1465,8 @@ svn_io_remove_dir (const char *path, apr_pool_t *pool)
               if (err)
                 return svn_error_createf (err->apr_err, err,
                                           "Can't remove '%s'", fullpath);
+
+              MACOSX_REWINDDIR_HACK (this_dir, path);
             }
         }
     }
@@ -1174,7 +1503,6 @@ svn_io_get_dirents (apr_hash_t **dirents,
   /* These exist so we can use their addresses as hash values! */
   static const svn_node_kind_t static_svn_node_file = svn_node_file;
   static const svn_node_kind_t static_svn_node_dir = svn_node_dir;
-  static const svn_node_kind_t static_svn_node_unknown = svn_node_unknown;
 
   *dirents = apr_hash_make (pool);
   
@@ -1207,7 +1535,7 @@ svn_io_get_dirents (apr_hash_t **dirents,
             /* ### symlinks, etc. will fall into this category for now.
                someday subversion will recognize them. :)  */
             apr_hash_set (*dirents, name, APR_HASH_KEY_STRING,
-                          &static_svn_node_unknown);
+                          &static_svn_node_file);
         }
     }
 
@@ -1491,8 +1819,9 @@ svn_io_run_diff3 (const char *dir,
     SVN_ERR (svn_config_get_config (&config, pool));
     cfg = config ? apr_hash_get (config, SVN_CONFIG_CATEGORY_CONFIG,
                                  APR_HASH_KEY_STRING) : NULL;
-    svn_config_get_bool (cfg, &has_arg, SVN_CONFIG_SECTION_HELPERS, 
-                         SVN_CONFIG_OPTION_DIFF3_HAS_PROGRAM_ARG, TRUE);
+    SVN_ERR (svn_config_get_bool (cfg, &has_arg, SVN_CONFIG_SECTION_HELPERS,
+                                  SVN_CONFIG_OPTION_DIFF3_HAS_PROGRAM_ARG,
+                                  TRUE));
     if (has_arg)
       {
         const char *diff_cmd, *diff_utf8;
@@ -1515,7 +1844,7 @@ svn_io_run_diff3 (const char *dir,
   /* Run diff3, output the merged text into the scratch file. */
   SVN_ERR (svn_io_run_cmd (dir, diff3_utf8, args, 
                            exitcode, NULL, 
-                           FALSE, /* clean environment */
+                           TRUE, /* keep environment */
                            NULL, merged, NULL,
                            pool));
 
@@ -1803,13 +2132,30 @@ svn_io_file_rename (const char *from_path, const char *to_path,
    should be set on the newly created directory. */
 static svn_error_t *
 dir_make (const char *path, apr_fileperms_t perm,
-          svn_boolean_t hidden, apr_pool_t *pool)
+          svn_boolean_t hidden, svn_boolean_t sgid, apr_pool_t *pool)
 {
   apr_status_t status;
   const char *path_apr;
 
   SVN_ERR (svn_path_cstring_from_utf8 (&path_apr, path, pool));
 
+  /* APR doesn't like "" directories */
+  if (path_apr[0] == '\0')
+    path_apr = ".";
+
+#if (APR_OS_DEFAULT & APR_WSTICKY)
+  /* The APR shipped with httpd 2.0.50 contains a bug where
+     APR_OS_DEFAULT encompasses the setuid, setgid, and sticky bits.
+     There is a special case for file creation, but not directory
+     creation, so directories wind up getting created with the sticky
+     bit set.  (There is no such thing as a setuid directory, and the
+     setgid bit is apparently ignored at mkdir() time.)  If we detect
+     this problem, work around it by unsetting those bits if we are
+     passed APR_OS_DEFAULT. */
+  if (perm == APR_OS_DEFAULT)
+    perm &= ~(APR_USETID | APR_GSETID | APR_WSTICKY);
+#endif
+
   status = apr_dir_make (path_apr, perm, pool);
 
   if (status)
@@ -1827,20 +2173,55 @@ dir_make (const char *path, apr_fileperms_t perm,
     }
 #endif
 
+#if defined(APR_GSETID)
+  if (sgid)
+    {
+      apr_finfo_t finfo;
+
+      status = apr_stat (&finfo, path_apr, APR_FINFO_PROT, pool);
+
+      if (status)
+        return svn_error_wrap_apr (status, "Can't stat directory '%s'", path);
+
+      /* Per our contract, don't do error-checking.  Some filesystems
+       * don't support the sgid bit, and that's okay. */
+      apr_file_perms_set (path_apr, finfo.protection | APR_GSETID);
+    }
+#elif !defined (WIN32)
+  /* APR_GSETID appears in APR 0.9.5, so we need some fallback code
+     until Subversion can require 0.9.5. */
+  if (sgid)
+  {
+    struct stat st;
+
+    if (stat (path_apr, &st) != 0)
+      return svn_error_wrap_apr (APR_FROM_OS_ERROR (errno),
+                                 "Can't stat new directory '%s'", path);
+    chmod (path_apr, (st.st_mode & ~S_IFMT) | S_ISGID);
+  }
+#endif
+
   return SVN_NO_ERROR;
 }
 
 svn_error_t *
 svn_io_dir_make (const char *path, apr_fileperms_t perm, apr_pool_t *pool)
 {
-  return dir_make (path, perm, FALSE, pool);
+  return dir_make (path, perm, FALSE, FALSE, pool);
 }
 
 svn_error_t *
 svn_io_dir_make_hidden (const char *path, apr_fileperms_t perm,
                         apr_pool_t *pool)
 {
-  return dir_make (path, perm, TRUE, pool);
+  return dir_make (path, perm, TRUE, FALSE, pool);
+}
+
+svn_error_t *
+svn_io_dir_make_sgid (const char *path, apr_fileperms_t perm,
+                      apr_pool_t *pool)
+{
+  return dir_make (path, perm, FALSE, TRUE, pool);
 }
 
 
@@ -2079,7 +2460,7 @@ svn_io_dir_empty (svn_boolean_t *is_empty_p,
 
   status = apr_dir_is_empty (path_apr, pool);
 
-  if (APR_STATUS_IS_SUCCESS (status))
+  if (!status)
     *is_empty_p = TRUE;
   else if (APR_STATUS_IS_ENOTEMPTY (status))
     *is_empty_p = FALSE;
@@ -2129,7 +2510,7 @@ svn_io_write_version_file (const char *path,
 
   /* And finally remove the perms to make it read only */
   SVN_ERR (svn_io_set_file_read_only (path, FALSE, pool));
- 
+  
   return SVN_NO_ERROR;
 }
 
