@@ -2,7 +2,7 @@
  * opt.c :  option and argument parsing for Subversion command lines
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -25,6 +25,7 @@
 #include <apr_general.h>
 #include <apr_lib.h>
 
+#include "svn_cmdline.h"
 #include "svn_version.h"
 #include "svn_types.h"
 #include "svn_wc.h"
@@ -35,6 +36,8 @@
 #include "svn_utf.h"
 #include "svn_time.h"
 
+#include "svn_private_config.h"
+
 
 /*** Code. ***/
 
@@ -94,7 +97,7 @@ svn_opt_subcommand_takes_option (const svn_opt_subcommand_desc_t *command,
 /* Print the canonical command name for CMD, and all its aliases, to
    STREAM.  If HELP is set, print CMD's help string too, in which case
    obtain option usage from OPTIONS_TABLE. */
-static void
+static svn_error_t *
 print_command_info (const svn_opt_subcommand_desc_t *cmd,
                     const apr_getopt_option_t *options_table,
                     svn_boolean_t help, 
@@ -105,7 +108,7 @@ print_command_info (const svn_opt_subcommand_desc_t *cmd,
   apr_size_t i;
 
   /* Print the canonical command name. */
-  fputs (cmd->name, stream);
+  SVN_ERR (svn_cmdline_fputs (cmd->name, stream, pool));
 
   /* Print the list of aliases. */
   first_time = TRUE;
@@ -115,24 +118,25 @@ print_command_info (const svn_opt_subcommand_desc_t *cmd,
         break;
 
       if (first_time) {
-        fprintf (stream, " (");
+        SVN_ERR (svn_cmdline_fputs (" (", stream, pool));
         first_time = FALSE;
       }
       else
-        fprintf (stream, ", ");
+        SVN_ERR (svn_cmdline_fputs (", ", stream, pool));
       
-      fprintf (stream, "%s", cmd->aliases[i]);
+      SVN_ERR (svn_cmdline_fputs (cmd->aliases[i], stream, pool));
     }
 
   if (! first_time)
-    fprintf (stream, ")");
+    SVN_ERR (svn_cmdline_fputs (")", stream, pool));
   
   if (help)
     {
       const apr_getopt_option_t *option;
       svn_boolean_t have_options = FALSE;
 
-      fprintf (stream, ": %s", cmd->help);
+      SVN_ERR (svn_cmdline_fprintf (stream, pool, ": %s",
+                                    dgettext (PACKAGE_NAME, cmd->help)));
 
       /* Loop over all valid option codes attached to the subcommand */
       for (i = 0; i < SVN_OPT_MAX_OPTIONS; i++)
@@ -141,7 +145,8 @@ print_command_info (const svn_opt_subcommand_desc_t *cmd,
             {
               if (have_options == FALSE)
                 {
-                  fprintf (stream, "\nValid options:\n");
+                  SVN_ERR (svn_cmdline_fputs (_("\nValid options:\n"),
+                                              stream, pool));
                   have_options = TRUE;
                 }
 
@@ -155,14 +160,17 @@ print_command_info (const svn_opt_subcommand_desc_t *cmd,
                 {
                   const char *optstr;
                   svn_opt_format_option (&optstr, option, TRUE, pool);
-                  fprintf (stream, "  %s\n", optstr);
+                  SVN_ERR (svn_cmdline_fprintf (stream, pool, "  %s\n",
+                                                optstr));
                 }
             }
         }
 
       if (have_options)
-        fprintf (stream, "\n");
+        SVN_ERR (svn_cmdline_fprintf (stream, pool, "\n"));
     }
+
+  return SVN_NO_ERROR;
 }
 
 
@@ -174,22 +182,34 @@ svn_opt_print_generic_help (const char *header,
                             apr_pool_t *pool, FILE *stream)
 {
   int i = 0;
+  svn_error_t *err;
 
   if (header)
-    fprintf (stream, "%s", header);
+    if ((err = svn_cmdline_fputs (header, stream, pool)))
+      goto print_error;
   
   while (cmd_table[i].name) 
     {
-      fprintf (stream, "   ");
-      print_command_info (cmd_table + i, opt_table, FALSE, pool, stream);
-      fprintf (stream, "\n");
+      if ((err = svn_cmdline_fputs ("   ", stream, pool))
+          || (err = print_command_info (cmd_table + i, opt_table, FALSE,
+                                        pool, stream))
+          || (err = svn_cmdline_fputs ("\n", stream, pool)))
+        goto print_error;
       i++;
     }
 
-  fprintf (stream, "\n");
-  
+  if ((err = svn_cmdline_fputs ("\n", stream, pool)))
+    goto print_error;
+
   if (footer)
-    fprintf (stream, "%s", footer);
+    if ((err = svn_cmdline_fputs (footer, stream, pool)))
+      goto print_error;
+
+  return;
+
+ print_error:
+  svn_handle_error (err, stderr, FALSE);
+  svn_error_clear (err);
 }
 
 
@@ -218,7 +238,8 @@ svn_opt_format_option (const char **string,
     opts = apr_pstrcat (pool, opts, " arg", NULL);
 
   if (doc)
-    opts = apr_psprintf (pool, "%-24s : %s", opts, opt->description);
+    opts = apr_psprintf (pool, "%-24s : %s", opts,
+                         dgettext( PACKAGE_NAME, opt->description));
 
   *string = opts;
 }
@@ -232,11 +253,18 @@ svn_opt_subcommand_help (const char *subcommand,
 {
   const svn_opt_subcommand_desc_t *cmd =
     svn_opt_get_canonical_subcommand (table, subcommand);
+  svn_error_t *err;
     
   if (cmd)
-    print_command_info (cmd, options_table, TRUE, pool, stdout);
+    err = print_command_info (cmd, options_table, TRUE, pool, stdout);
   else
-    fprintf (stderr, "\"%s\": unknown command.\n\n", subcommand);
+    err = svn_cmdline_fprintf (stderr, pool,
+                               _("\"%s\": unknown command.\n\n"), subcommand);
+  
+  if (err) {
+    svn_handle_error (err, stderr, FALSE);
+    svn_error_clear (err);
+  }
 }
 
 
@@ -251,23 +279,13 @@ svn_opt_subcommand_help (const char *subcommand,
  *
  *   - For "head", set REVISION->kind to svn_opt_revision_head.
  *
- *   - For "first", set REVISION->kind to svn_opt_revision_number
- *     and REVISION->value.number to 0.  ### iffy, but might be useful
- *     when mixed with dates ###
- *
  *   - For "prev", set REVISION->kind to svn_opt_revision_previous.
  *
  *   - For "base", set REVISION->kind to svn_opt_revision_base.
  *
- *   - For "committed" or "changed", set REVISION->kind to
- *     svn_opt_revision_committed.
+ *   - For "committed", set REVISION->kind to svn_opt_revision_committed.
  *
  * If match, return 0, else return -1 and don't touch REVISION.
- *
- * ### should we enforce a requirement that users write out these
- * words in full?  Actually, we probably will need to start enforcing
- * it as date parsing gets more sophisticated and the chances of a
- * first-letter overlap between a valid date and a valid word go up.
  */
 static int
 revision_from_word (svn_opt_revision_t *revision, const char *word)
@@ -276,11 +294,6 @@ revision_from_word (svn_opt_revision_t *revision, const char *word)
     {
       revision->kind = svn_opt_revision_head;
     }
-  else if (strcasecmp (word, "first") == 0)
-    {
-      revision->kind = svn_opt_revision_number;
-      revision->value.number = 0;
-    }
   else if (strcasecmp (word, "prev") == 0)
     {
       revision->kind = svn_opt_revision_previous;
@@ -289,8 +302,7 @@ revision_from_word (svn_opt_revision_t *revision, const char *word)
     {
       revision->kind = svn_opt_revision_base;
     }
-  else if ((strcasecmp (word, "committed") == 0)
-           || (strcasecmp (word, "changed") == 0))
+  else if (strcasecmp (word, "committed") == 0)
     {
       revision->kind = svn_opt_revision_committed;
     }
@@ -303,25 +315,35 @@ revision_from_word (svn_opt_revision_t *revision, const char *word)
 
 /* Parse one revision specification.  Return pointer to character
    after revision, or NULL if the revision is invalid.  Modifies
-   str, so make sure to pass a copy of anything precious. */
-static char *parse_one_rev (svn_opt_revision_t *revision, char *str)
+   str, so make sure to pass a copy of anything precious.  Uses
+   POOL for temporary allocation. */
+static char *parse_one_rev (svn_opt_revision_t *revision, char *str,
+                            apr_pool_t *pool)
 {
   char *end, save;
-  time_t tm;
 
   if (*str == '{')
     {
+      svn_boolean_t matched;
+      apr_time_t tm;
+      svn_error_t *err;
+
       /* Brackets denote a date. */
       str++;
       end = strchr (str, '}');
       if (!end)
         return NULL;
       *end = '\0';
-      tm = svn_parse_date (str, NULL);
-      if (tm == -1)
-        return NULL;
+      err = svn_parse_date (&matched, &tm, str, apr_time_now (), pool);
+      if (err)
+        {
+          svn_error_clear (err);
+          return NULL;
+        }
+      if (!matched)
+        return NULL; 
       revision->kind = svn_opt_revision_date;
-      apr_time_ansi_put (&(revision->value.date), tm);
+      revision->value.date = tm;
       return end + 1;
     }
   else if (apr_isdigit (*str))
@@ -365,11 +387,11 @@ svn_opt_parse_revision (svn_opt_revision_t *start_revision,
   /* Operate on a copy of the argument. */
   left_rev = apr_pstrdup (pool, arg);
 
-  right_rev = parse_one_rev (start_revision, left_rev);
+  right_rev = parse_one_rev (start_revision, left_rev, pool);
   if (right_rev && *right_rev == ':')
     {
       right_rev++;
-      end = parse_one_rev (end_revision, right_rev);
+      end = parse_one_rev (end_revision, right_rev, pool);
       if (!end || *end != '\0')
         return -1;
     }
@@ -427,7 +449,7 @@ svn_opt_parse_num_args (apr_array_header_t **args_p,
       if (os->ind >= os->argc)
         {
           return svn_error_create (SVN_ERR_CL_ARG_PARSING_ERROR, 
-                                   0, "too few arguments");
+                                   0, _("Too few arguments"));
         }
       array_push_str (args, os->argv[os->ind++], pool);
     }
@@ -446,7 +468,7 @@ svn_opt_parse_all_args (apr_array_header_t **args_p,
 
   if (os->ind > os->argc)
     {
-      return svn_error_create (SVN_ERR_CL_ARG_PARSING_ERROR, 0, "");
+      return svn_error_create (SVN_ERR_CL_ARG_PARSING_ERROR, 0, NULL);
     }
   while (os->ind < os->argc)
     {
@@ -458,22 +480,11 @@ svn_opt_parse_all_args (apr_array_header_t **args_p,
 }
 
 
-/* Parse a working-copy or url PATH, looking for an "@" sign, e.g.
-
-         foo/bar/baz@13
-         http://blah/bloo@27
-         blarg/snarf@HEAD
-
-   If an "@" is found, return the two halves in *TRUEPATH and *REV,
-   allocating in POOL.
-
-   If no "@" is found, set *TRUEPATH to PATH and *REV to kind 'unspecified'.
-*/
-static svn_error_t *
-parse_path (svn_opt_revision_t *rev,
-            const char **truepath,
-            const char *path /* UTF-8! */,
-            apr_pool_t *pool)
+svn_error_t *
+svn_opt_parse_path (svn_opt_revision_t *rev,
+                    const char **truepath,
+                    const char *path /* UTF-8! */,
+                    apr_pool_t *pool)
 {
   int i;
   apr_pool_t *subpool = svn_pool_create (pool);
@@ -483,6 +494,10 @@ parse_path (svn_opt_revision_t *rev,
      screwed-up filenames that might *actually* contain @-signs.  :-) */
   for (i = (strlen (path) - 1); i >= 0; i--)
     {
+      /* If we hit a path separator, stop looking. */
+      if (path[i] == '/')
+        break;
+
       if (path[i] == '@')
         {
           const char *native_rev;
@@ -494,7 +509,7 @@ parse_path (svn_opt_revision_t *rev,
                                       &end_revision,
                                       native_rev, subpool))
             return svn_error_createf (SVN_ERR_CL_ARG_PARSING_ERROR, NULL,
-                                      "Syntax error parsing revision \"%s\"",
+                                      _("Syntax error parsing revision '%s'"),
                                       path + i + 1);
 
           *truepath = apr_pstrndup (pool, path, i);
@@ -540,7 +555,7 @@ svn_opt_args_to_target_array (apr_array_header_t **targets_p,
       const char *raw_target = os->argv[os->ind];
       SVN_ERR (svn_utf_cstring_to_utf8
                ((const char **) apr_array_push (input_targets),
-                raw_target, NULL, pool));
+                raw_target, pool));
     }
 
   if (known_targets)
@@ -566,13 +581,26 @@ svn_opt_args_to_target_array (apr_array_header_t **targets_p,
       if (svn_path_is_url (utf8_target))
         {
           /* No need to canonicalize a URL's case or path separators. */
-          if (! svn_path_is_uri_safe (utf8_target))
+
+          /* Convert to URI. */
+          target = svn_path_uri_from_iri (utf8_target, pool);
+          /* Auto-escape some ASCII characters. */
+          target = svn_path_uri_autoescape (target, pool);
+
+          /* The above doesn't guarantee a valid URI. */
+          if (! svn_path_is_uri_safe (target))
             return svn_error_createf (SVN_ERR_BAD_URL, 0,
-                                      "URL `%s' is not properly URI-encoded",
+                                      _("URL '%s' is not properly URI-encoded"),
                                       utf8_target);
-                                      
+
+          /* Verify that no backpaths are present in the URL. */
+          if (svn_path_is_backpath_present (target))
+            return svn_error_createf (SVN_ERR_BAD_URL, 0,
+                                      _("URL '%s' contains a '..' element"),
+                                      utf8_target);
+          
           /* strip any trailing '/' */
-          target = svn_path_canonicalize (utf8_target, pool);
+          target = svn_path_canonicalize (target, pool);
         }
       else  /* not a url, so treat as a path */
         {
@@ -597,7 +625,7 @@ svn_opt_args_to_target_array (apr_array_header_t **targets_p,
             ;
           else
             return svn_error_createf (apr_err, NULL,
-                                      "Error resolving case of '%s'.",
+                                      _("Error resolving case of '%s'"),
                                       utf8_target);
 
           /* convert back to UTF-8. */
@@ -628,7 +656,7 @@ svn_opt_args_to_target_array (apr_array_header_t **targets_p,
       if (output_targets->nelts > 0)
         {
           path = ((const char **) (output_targets->elts))[0];
-          SVN_ERR (parse_path (&temprev, &path, path, pool));
+          SVN_ERR (svn_opt_parse_path (&temprev, &path, path, pool));
           if (temprev.kind != svn_opt_revision_unspecified)
             {
               ((const char **) (output_targets->elts))[0] = 
@@ -640,7 +668,7 @@ svn_opt_args_to_target_array (apr_array_header_t **targets_p,
       if (output_targets->nelts > 1)
         {
           path = ((const char **) (output_targets->elts))[1];
-          SVN_ERR (parse_path (&temprev, &path, path, pool));
+          SVN_ERR (svn_opt_parse_path (&temprev, &path, path, pool));
           if (temprev.kind != svn_opt_revision_unspecified)
             {
               ((const char **) (output_targets->elts))[1] = 
@@ -669,31 +697,25 @@ print_version_info (const char *pgm_name,
                     svn_boolean_t quiet,
                     apr_pool_t *pool)
 {
-  static const char info[] =
-    "Copyright (C) 2000-2003 CollabNet.\n"
-    "Subversion is open source software, see http://subversion.tigris.org/\n";
-
   if (quiet)
     {
-      printf ("%s\n", SVN_VER_NUMBER);
+      SVN_ERR (svn_cmdline_printf (pool, "%s\n", SVN_VER_NUMBER));
       return SVN_NO_ERROR;
     }
 
-  printf ("%s, version %s\n", pgm_name, SVN_VERSION);
-  printf ("   compiled %s, %s\n\n", __DATE__, __TIME__);
-  printf ("%s\n", info);
+  SVN_ERR (svn_cmdline_printf (pool, _("%s, version %s\n"
+                                       "   compiled %s, %s\n\n"), pgm_name,
+                               SVN_VERSION, __DATE__, __TIME__));
+  SVN_ERR (svn_cmdline_fputs (_("Copyright (C) 2000-2004 CollabNet.\n"
+                                "Subversion is open source software, see"
+                                " http://subversion.tigris.org/\n"
+                                "This product includes software developed by "
+                                "CollabNet (http://www.Collab.Net/).\n\n"),
+                              stdout, pool));
 
   if (footer)
     {
-      const char *footer_native;
-
-      /* ### Back when this code lived in help-cmd.c, prior to rev
-       * 3250 or so, this was the only part of the version output that
-       * did utf8-to-native conversion.  So I've kept that conversion,
-       * but shouldn't the rest of the code do something similar?
-       */
-      SVN_ERR (svn_utf_cstring_from_utf8 (&footer_native, footer, pool));
-      printf ("%s\n", footer_native);
+      SVN_ERR (svn_cmdline_printf (pool, "%s\n", footer));
     }
 
   return SVN_NO_ERROR;
@@ -734,7 +756,8 @@ svn_opt_print_help (apr_getopt_t *os,
                                 pool,
                                 stdout);
   else                                       /* unknown option or cmd */
-    fprintf (stderr, "Type `%s help' for usage.\n", pgm_name);
+    SVN_ERR (svn_cmdline_fprintf (stderr, pool,
+                                  _("Type '%s help' for usage.\n"), pgm_name));
 
   return SVN_NO_ERROR;
 }
