@@ -22,6 +22,7 @@
 #include <assert.h>
 
 #include <apr_file_info.h>
+#include <apr_lib.h>
 
 #include "svn_string.h"
 #include "svn_path.h"
@@ -89,50 +90,6 @@ svn_path_local_style (const char *path, apr_pool_t *pool)
 }
 
 
-
-/* Identify trailing '/' and "/." suffix of PATH.  LEN is the number of
-   characters in PATH.  Returns the number characters without the
-   suffix. */
-static apr_size_t
-discount_trailing_dot_slash (const char *path,
-                             apr_size_t len)
-{
-  while (len > 0)
-    if (path[len - 1] == '/')
-      --len;
-    else if (len > 1
-             && SVN_PATH_IS_PLATFORM_EMPTY(path + len - 1, 1)
-             && path[len - 2] == '/')
-      len -= 2;
-    else
-      break;
-
-  return len;
-}
-
-const char *
-svn_path_canonicalize (const char *path, apr_pool_t *pool)
-{
-  /* At some point this could eliminate redundant components.
-     For now, it just makes sure there is no trailing slashes
-     or dots, and converts "." to "".  */
-
-  apr_size_t len, orig_len;
-
-  orig_len = strlen (path);
-  len = discount_trailing_dot_slash (path, orig_len);
-  if (len == 0 && orig_len > 0 && path[0] == '/')
-    len = 1;
-
-  if (SVN_PATH_IS_PLATFORM_EMPTY (path, len))
-    return SVN_EMPTY_PATH;      /* the canonical empty path */
-
-  if (len == orig_len)
-    return path;
-
-  return apr_pstrmemdup (pool, path, len);
-}
-
 
 #ifndef NDEBUG
 static svn_boolean_t
@@ -308,6 +265,55 @@ char *svn_path_join_many (apr_pool_t *pool, const char *base, ...)
 
 
 
+apr_size_t
+svn_path_component_count (const char *path)
+{
+  apr_size_t count = 0;
+
+  assert (is_canonical (path, strlen (path)));
+
+  while (*path)
+    {
+      const char *start;
+
+      while (*path == '/')
+        ++path;
+
+      start = path;
+      
+      while (*path && *path != '/')
+        ++path;
+
+      if (path != start)
+        ++count;
+    }
+
+  return count;
+}
+
+/* Return the length of substring necessary to encompass the entire
+ * previous path segment in PATH, which should be a LEN byte string.
+ *
+ * A trailing slash will not be included in the returned length except
+ * in the case in which PATH is absolute and there are no more
+ * previous segments.
+ */
+static apr_size_t
+previous_segment (const char *path,
+                  apr_size_t len)
+{
+  if (len == 0)
+    return 0;
+
+  while (len > 0 && path[--len] != '/')
+    ;
+
+  if (len == 0 && path[0] == '/')
+    return 1;
+  else
+    return len;
+}
+
 void
 svn_path_add_component (svn_stringbuf_t *path, 
                         const char *component)
@@ -333,28 +339,19 @@ svn_path_add_component (svn_stringbuf_t *path,
 void
 svn_path_remove_component (svn_stringbuf_t *path)
 {
-  apr_size_t len;
-
   assert (is_canonical (path->data, path->len));
 
-  while (path->len > 0 && path->data[path->len - 1] != '/')
-    --path->len;
-
+  path->len = previous_segment(path->data, path->len);
   path->data[path->len] = '\0';
+}
 
-  /* ### Right now the input path could contain redundant components,
-     since svn_path_canonicalize doesn't remove them.  So we have to
-     check and strip those off. */
-  len = discount_trailing_dot_slash (path->data, path->len);
-  if (len == 0 && path->len > 0 && path->data[0] == '/')
-    len = 1;
-
-  if (SVN_PATH_IS_PLATFORM_EMPTY (path->data, len))
-    svn_stringbuf_set (path, SVN_EMPTY_PATH);
-  else
+void
+svn_path_remove_components (svn_stringbuf_t *path, apr_size_t n)
+{
+  while (n > 0)
     {
-      path->len = len;
-      path->data[path->len] = '\0';
+      svn_path_remove_component (path);
+      n--;
     }
 }
 
@@ -362,23 +359,11 @@ svn_path_remove_component (svn_stringbuf_t *path)
 char *
 svn_path_dirname (const char *path, apr_pool_t *pool)
 {
-  apr_size_t canonical_len, len = strlen (path);
+  apr_size_t len = strlen (path);
 
   assert (is_canonical (path, len));
 
-  while (len > 0 && path[len - 1] != '/')
-    --len;
-
-  /* ### Should canonicalization strip "//" and "/./" substrings? */
-  canonical_len = discount_trailing_dot_slash (path, len);
-
-  if (canonical_len == 0 && len > 0 && path[0] == '/')
-    canonical_len = 1;
-
-  if (SVN_PATH_IS_PLATFORM_EMPTY (path, canonical_len))
-    return apr_pmemdup (pool, SVN_EMPTY_PATH, sizeof (SVN_EMPTY_PATH));
-
-  return apr_pstrmemdup (pool, path, canonical_len);
+  return apr_pstrmemdup (pool, path, previous_segment(path, len));
 }
 
 
@@ -694,6 +679,27 @@ svn_path_is_single_path_component (const char *name)
 }
 
 
+svn_boolean_t
+svn_path_is_backpath_present (const char *path)
+{
+  int len = strlen (path);
+  
+  if (! strcmp (path, ".."))
+    return TRUE;
+
+  if (! strncmp (path, "../", 3))
+    return TRUE;
+  
+  if (strstr (path, "/../") != NULL)
+    return TRUE;
+
+  if (len >= 3
+      && (! strncmp (path + len - 3, "/..", 3)))
+    return TRUE;
+
+  return FALSE;
+}
+
 
 /*** URI Stuff ***/
 
@@ -823,12 +829,7 @@ svn_path_is_uri_safe (const char *path)
       /* Allow '%XX' (where each X is a hex digit) */
       if (path[i] == '%')
         {
-          if ((((path[i + 1] >= '0') && (path[i + 1] <= '9'))
-               || ((path[i + 1] >= 'a') && (path[i + 1] <= 'f'))
-               || ((path[i + 1] >= 'A') && (path[i + 1] <= 'F')))
-              && (((path[i + 2] >= '0') && (path[i + 2] <= '9'))
-                  || ((path[i + 2] >= 'a') && (path[i + 2] <= 'f'))
-                  || ((path[i + 2] >= 'A') && (path[i + 2] <= 'F'))))
+          if (apr_isxdigit (path[i + 1]) && apr_isxdigit (path[i + 2]))
             {
               i += 2;
               continue;
@@ -844,9 +845,11 @@ svn_path_is_uri_safe (const char *path)
   return TRUE;
 }
   
-
-const char *
-svn_path_uri_encode (const char *path, apr_pool_t *pool)
+/* URI-encode each character c in PATH for which TABLE[c] is 0.
+   If no encoding was needed, return PATH, else return a new string allocated
+   in POOL. */
+static const char *
+uri_escape (const char *path, const int table[], apr_pool_t *pool)
 {
   svn_stringbuf_t *retstr;
   apr_size_t i, copied = 0;
@@ -856,7 +859,7 @@ svn_path_uri_encode (const char *path, apr_pool_t *pool)
   for (i = 0; path[i]; i++)
     {
       c = (unsigned char)path[i];
-      if (uri_char_validity[c])
+      if (table[c])
         continue;
 
       /* If we got here, we're looking at a character that isn't
@@ -883,17 +886,91 @@ svn_path_uri_encode (const char *path, apr_pool_t *pool)
       copied = i + 1;
     }
 
+  /* If we didn't encode anything, we don't need to duplicate the string. */
+  if (retstr->len == 0)
+    return path;
+
   /* Anything left to copy? */
   if (i - copied)
     svn_stringbuf_appendbytes (retstr, path + copied, i - copied);
 
-  /* Null-terminate this bad-boy. */
-  svn_stringbuf_ensure (retstr, retstr->len + 1);
-  retstr->data[retstr->len] = 0;
+  /* retstr is null-terminated either by sprintf or the svn_stringbuf
+     functions. */
 
   return retstr->data;
 }
 
+const char *
+svn_path_uri_encode (const char *path, apr_pool_t *pool)
+{
+  const char *ret;
+
+  ret = uri_escape (path, uri_char_validity, pool);
+
+  /* Our interface guarantees a copy. */
+  if (ret == path)
+    return apr_pstrdup (pool, path);
+  else
+    return ret;
+}
+
+static const int iri_escape_chars[256] = {
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+
+  /* 128 */
+  0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0
+};
+
+const char *
+svn_path_uri_from_iri (const char *iri, apr_pool_t *pool)
+{
+  return uri_escape (iri, iri_escape_chars, pool);
+}
+
+const int uri_autoescape_chars[256] = {
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  0, 1, 0, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 0, 1, 0, 1,
+
+  /* 64 */
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 0, 1, 0, 1,
+  0, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 0, 0, 0, 1, 1,
+
+  /* 128 */
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+
+  /* 192 */
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+  1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
+};
+
+const char *
+svn_path_uri_autoescape (const char *uri, apr_pool_t *pool)
+{
+  return uri_escape (uri, uri_autoescape_chars, pool);
+}
 
 const char *
 svn_path_uri_decode (const char *path, apr_pool_t *pool)
@@ -923,7 +1000,8 @@ svn_path_uri_decode (const char *path, apr_pool_t *pool)
            * RFC 2396, section 3.3  */
           c = ' ';
         }
-      else if (c == '%')
+      else if (c == '%' && apr_isxdigit (path[i + 1])
+               && apr_isxdigit (path[i+2]))
         {
           char digitz[3];
           digitz[0] = path[++i];
@@ -953,7 +1031,6 @@ svn_path_url_add_component (const char *url,
   return svn_path_join (url, svn_path_uri_encode (component, pool), pool);
 }
 
-
 svn_error_t *
 svn_path_get_absolute(const char **pabsolute,
                       const char *relative,
@@ -1031,6 +1108,94 @@ svn_path_split_if_file(const char *path,
   return SVN_NO_ERROR;
 }
 
+
+const char *
+svn_path_canonicalize (const char *path, apr_pool_t *pool)
+{
+  char *canon, *dst;
+  const char *src;
+  apr_size_t seglen;
+  apr_size_t canon_segments = 0;
+  svn_boolean_t uri;
+
+  dst = canon = apr_pcalloc (pool, strlen (path) + 1);
+
+  /* Copy over the URI shema if present. */
+  src = skip_uri_schema (path);
+  if (src)
+    {
+      uri = TRUE;
+      memcpy (dst, path, src - path);
+      dst += (src - path);
+    }
+  else
+    {
+      uri = FALSE;
+      src = path;
+    }
+
+  /* If this is an absolute path, then just copy over the initial
+     separator character. */
+  if (*src == '/')
+    {
+      *(dst++) = *(src++);
+
+#ifdef WIN32
+      /* On Windows permit two leading separator characters which means an
+       * UNC path.  However, a double slash in a URI after the scheme is never
+       * valid. */
+      if (!uri && *src == '/')
+        *(dst++) = *(src++);
+#endif /* WIN32 */
+      
+    }
+
+  while (*src)
+    {
+      /* Parse each segment, find the closing '/' */
+      const char *next = src;
+      while (*next && (*next != '/'))
+        ++next;
+
+      seglen = next - src;
+
+      if (seglen == 0 || (seglen == 1 && src[0] == '.'))
+        {
+          /* Noop segment, so do nothing. */
+        }
+      else
+        {
+          /* An actual segment, append it to the destination path */
+          if (*next)
+            seglen++;
+          memcpy (dst, src, seglen);
+          dst += seglen;
+          canon_segments++;
+        }
+
+      /* Skip over trailing slash to the next segment. */
+      src = next;
+      if (*src)
+        src++;
+    }
+
+  /* Remove the trailing slash. */
+  if ((canon_segments > 0 || uri) && *(dst - 1) == '/')
+    dst--;
+  
+  *dst = '\0';
+
+#ifdef WIN32
+  /* Skip leading double slashes when there are less than 2
+   * canon segments. UNC paths *MUST* have two segments. */
+  if (canon_segments < 2 && canon[0] == '/' && canon[1] == '/')
+    return canon + 1;
+#endif /* WIN32 */
+
+  return canon;
+}
+
+
 
 /** Get APR's internal path encoding. */
 static svn_error_t *
