@@ -2,7 +2,7 @@
  * subst.c :  generic eol/keyword substitution routines
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -28,6 +28,8 @@
 #include <apr_file_io.h>
 #include <apr_strings.h>
 #include <apr_lib.h>
+
+#include "svn_cmdline.h"
 #include "svn_types.h"
 #include "svn_delta.h"
 #include "svn_string.h"
@@ -39,6 +41,16 @@
 #include "svn_io.h"
 #include "svn_hash.h"
 #include "svn_subst.h"
+#include "svn_pools.h"
+
+#include "svn_private_config.h"
+
+/**
+ * The textual elements of a detranslated special file.  One of these
+ * strings must appear as the first element of any special file as it
+ * exists in the repository or the text base.
+ */
+#define SVN_SUBST__SPECIAL_LINK_STR "link"
 
 void 
 svn_subst_eol_style_from_value (svn_subst_eol_style_t *style,
@@ -86,7 +98,7 @@ svn_subst_eol_style_from_value (svn_subst_eol_style_t *style,
 
 /* A helper function to convert the date property to something suitable for 
    printing out.  If LONG_P is TRUE, use the long format, otherwise use a 
-   shorter one. */
+   shorter one.  Returns a UTF8 encoded cstring. */
 static svn_error_t *
 date_prop_to_human (const char **human, svn_boolean_t long_p, apr_time_t when,
                     apr_pool_t *pool)
@@ -131,6 +143,7 @@ svn_subst_build_keywords (svn_subst_keywords_t *kw,
       const char *keyword = APR_ARRAY_IDX (keyword_tokens, i, const char *);
 
       if ((! strcmp (keyword, SVN_KEYWORD_REVISION_LONG))
+          || (! strcmp (keyword, SVN_KEYWORD_REVISION_MEDIUM))
           || (! strcasecmp (keyword, SVN_KEYWORD_REVISION_SHORT)))
         {
           kw->revision = svn_string_create (rev, pool);
@@ -345,6 +358,12 @@ translate_keyword (char *buf,
                                    expand ? keywords->revision : NULL))
         return TRUE;
 
+      if (translate_keyword_subst (buf, len,
+                                   SVN_KEYWORD_REVISION_MEDIUM,
+                                   (sizeof (SVN_KEYWORD_REVISION_MEDIUM)) - 1,
+                                   expand ? keywords->revision : NULL))
+        return TRUE;
+
       if (translate_keyword_subst (buf, len,
                                    SVN_KEYWORD_REVISION_SHORT,
                                    (sizeof (SVN_KEYWORD_REVISION_SHORT)) - 1,
@@ -448,9 +467,7 @@ translate_newline (const char *eol_str,
       if ((! repair) &&
           ((*src_format_len != newline_len) ||
            (strncmp (src_format, newline_buf, newline_len)))) 
-        return svn_error_create
-          (SVN_ERR_IO_INCONSISTENT_EOL, NULL,
-           "inconsistent line-endings in source stream, repair flag is off.");
+        return svn_error_create (SVN_ERR_IO_INCONSISTENT_EOL, NULL, NULL);
     }
   else
     {
@@ -532,7 +549,6 @@ svn_subst_keywords_differ (const svn_subst_keywords_t *a,
 }
 
 
-/* ### for docstring:  a translation request MUST be set! */
 svn_error_t *
 svn_subst_translate_stream (svn_stream_t *s, /* src stream */
                             svn_stream_t *d, /* dst stream */
@@ -541,9 +557,8 @@ svn_subst_translate_stream (svn_stream_t *s, /* src stream */
                             const svn_subst_keywords_t *keywords,
                             svn_boolean_t expand)
 {
-  svn_error_t *err = SVN_NO_ERROR;
-  svn_error_t *read_err, *close_err;
-  char c;
+  char buf[SVN_STREAM_CHUNK_SIZE + 1];
+  const char *p, *interesting;
   apr_size_t len, readlen;
   apr_size_t eol_str_len = eol_str ? strlen (eol_str) : 0;
   char       newline_buf[2] = { 0 };
@@ -555,224 +570,99 @@ svn_subst_translate_stream (svn_stream_t *s, /* src stream */
 
   /* The docstring requires that *some* translation be requested. */
   assert (eol_str || keywords);
+  interesting = (eol_str && keywords) ? "$\r\n" : eol_str ? "\r\n" : "$";
 
-  /* Copy bytes till the cows come home (or until one of them breaks a
-     leg, at which point you should trot out to the range with your
-     trusty sidearm, put her down, and consider steak dinners for the
-     next two weeks). */
-  while (err == SVN_NO_ERROR)
+  readlen = sizeof (buf) - 1;
+  while (readlen == sizeof (buf) - 1)
     {
-      /* Read a byte from SRC */
-      readlen = 1;
-      read_err = svn_stream_read (s, &c, &readlen);
+      SVN_ERR (svn_stream_read (s, buf, &readlen));
+      buf[readlen] = '\0';
 
-      if (read_err)
+      /* At the beginning of this loop, assume that we might be in an
+       * interesting state, i.e. with data in the newline or keyword
+       * buffer.  First try to get to the boring state so we can copy
+       * a run of boring characters; then try to get back to the
+       * interesting state by processing an interesting character,
+       * and repeat. */
+      for (p = buf; p < buf + readlen;)
         {
-          /* Oops, something bad happened. */
-          err = read_err;
-          goto cleanup;
-        }
-      else if (readlen < 1)
-        {
-          /* A short read means we've reached the end of the stream.
-             Close up shop.  This means flushing our temporary
-             streams.  Since we shouldn't have data in *both*
-             temporary streams, order doesn't matter here.  However,
-             the newline buffer will need to be translated.  */
+          /* Try to get to the boring state, if necessary. */
           if (newline_off)
             {
-              if ((err = translate_newline (eol_str, eol_str_len, 
-                                            src_format, &src_format_len,
-                                            newline_buf, newline_off,
-                                            d, repair)))
-                goto cleanup;
-            }
-          if (((len = keyword_off)) && 
-              ((err = translate_write (d, keyword_buf, len))))
-            goto cleanup;
-          
-          /* Close the source and destination streams. */
-          close_err = svn_stream_close (s);
-          if (close_err)
-            goto cleanup;
-
-          close_err = svn_stream_close (d);
-          if (close_err)
-            goto cleanup;
-          
-          /* All done, all streams closed, all is well, and all that. */
-          return SVN_NO_ERROR;
-        }
-    
-
-      /* Handle the byte. */
-      switch (c)
-        {
-        case '$':
-          /* A-ha!  A keyword delimiter!  */
-
-          /* If we are currently collecting up a possible newline
-             string, this puts an end to that collection.  Flush the
-             newline buffer (translating as necessary) and move
-             along. */
-          if (newline_off)
-            {
-              if ((err = translate_newline (eol_str, eol_str_len, 
-                                            src_format, &src_format_len,
-                                            newline_buf, newline_off,
-                                            d, repair)))
-                goto cleanup;
+              if (*p == '\n')
+                newline_buf[newline_off++] = *p++;
+              SVN_ERR (translate_newline (eol_str, eol_str_len, src_format,
+                                          &src_format_len, newline_buf,
+                                          newline_off, d, repair));
               newline_off = 0;
             }
-
-          /* If we aren't paying attention to keywords, just skip the
-             rest of this stuff. */
-          if (! keywords)
+          else if (keyword_off && *p == '$')
             {
-              if ((err = translate_write (d, (const void *)&c, 1)))
-                goto cleanup;
-              break;
-            }
-
-          /* Put this character into the keyword buffer. */
-          keyword_buf[keyword_off++] = c;
-
-          /* If this `$' is the beginning of a possible keyword, we're
-             done with it for now.  */
-          if (keyword_off == 1)
-            break;
-
-          /* Else, it must be the end of one!  Attempt to translate
-             the buffer. */
-          len = keyword_off;
-          if (translate_keyword (keyword_buf, &len, expand, keywords))
-            {
-              /* We successfully found and translated a keyword.  We
-                 can write out this buffer now. */
-              if ((err = translate_write (d, keyword_buf, len)))
-                goto cleanup;
+              /* If translation fails, treat this '$' as a starting '$'. */
+              keyword_buf[keyword_off++] = '$';
+              if (translate_keyword (keyword_buf, &keyword_off, expand,
+                                     keywords))
+                p++;
+              else
+                keyword_off--;
+              SVN_ERR (translate_write (d, keyword_buf, keyword_off));
               keyword_off = 0;
             }
-          else
+          else if (keyword_off == SVN_KEYWORD_MAX_LEN - 1
+                   || (keyword_off && (*p == '\r' || *p == '\n')))
             {
-              /* No keyword was found here.  We'll let our
-                 "terminating `$'" become a "beginning `$'" now.  That
-                 means, write out all the keyword buffer (except for
-                 this `$') and reset it to hold only this `$'.  */
-              if ((err = translate_write (d, keyword_buf, keyword_off - 1)))
-                goto cleanup;
-              keyword_buf[0] = c;
-              keyword_off = 1;
-            }
-          break;
-
-        case '\n':
-        case '\r':
-          /* Newline character.  If we currently bagging up a keyword
-             string, this pretty much puts an end to that.  Flush the
-             keyword buffer, then handle the newline. */
-          if ((len = keyword_off))
-            {
-              if ((err = translate_write (d, keyword_buf, len)))
-                goto cleanup;
+              /* No closing '$' found; flush the keyword buffer. */
+              SVN_ERR (translate_write (d, keyword_buf, keyword_off));
               keyword_off = 0;
             }
-
-          if (! eol_str)
+          else if (keyword_off)
             {
-              /* Not doing newline translation...just write out the char. */
-              if ((err = translate_write (d, (const void *)&c, 1)))
-                goto cleanup;
+              keyword_buf[keyword_off++] = *p++;
+              continue;
+            }
+
+          /* We're in the boring state; look for interest characters.
+           * For lack of a memcspn(), manually skip past NULs. */
+          len = 0;
+          while (1)
+            {
+              len += strcspn (p + len, interesting);
+              if (p[len] != '\0' || p + len == buf + readlen)
+                break;
+              len++;
+            }
+          if (len)
+            SVN_ERR (translate_write (d, p, len));
+          p += len;
+
+          /* Set up state according to the interesting character, if any. */
+          switch (*p)
+            {
+            case '$':
+              keyword_buf[keyword_off++] = *p++;
               break;
-            }
-
-          /* If we aren't yet tracking the development of a newline
-             string, begin so now. */
-          if (! newline_off)
-            {
-              newline_buf[newline_off++] = c;
+            case '\r':
+              newline_buf[newline_off++] = *p++;
               break;
-            }
-          else
-            {
-              /* We're already tracking a newline string, so let's see
-                 if this is part of the same newline, or the start of
-                 a new one. */
-              char c0 = newline_buf[0];
-
-              if ((c0 == c) || ((c0 == '\n') && (c == '\r')))
-                {
-                  /* The first '\n' (or '\r') is the newline... */
-                  if ((err = translate_newline (eol_str, eol_str_len, 
-                                                src_format, &src_format_len,
-                                                newline_buf, 1,
-                                                d, repair)))
-                    goto cleanup;
-
-                  /* ...the second '\n' (or '\r') is at least part of our next
-                     newline. */
-                  newline_buf[0] = c;
-                  newline_off = 1;
-                }
-              else 
-                {
-                  /* '\r\n' is our newline */
-                  newline_buf[newline_off++] = c;
-                  if ((err = translate_newline (eol_str, eol_str_len, 
-                                                src_format, &src_format_len,
-                                                newline_buf, 2,
-                                                d, repair)))
-                    goto cleanup;
-                  newline_off = 0;
-                }
-            }
-          break;
-
-        default:
-          /* If we're currently bagging up a keyword string, we'll
-             add this character to the keyword buffer.  */
-          if (keyword_off)
-            {
-              keyword_buf[keyword_off++] = c;
-              
-              /* If we've reached the end of this buffer without
-                 finding a terminating '$', we just flush the buffer
-                 and continue on. */
-              if (keyword_off >= SVN_KEYWORD_MAX_LEN)
-                {
-                  if ((err = translate_write (d, keyword_buf, keyword_off)))
-                    goto cleanup;
-                  keyword_off = 0;
-                }
-              break;
-            }
-
-          /* If we're in a potential newline separator, this character
-             terminates that search, so we need to flush our newline
-             buffer (translating as necessary) and then output this
-             character.  */
-          if (newline_off)
-            {
-              if ((err = translate_newline (eol_str, eol_str_len, 
-                                            src_format, &src_format_len,
-                                            newline_buf, newline_off,
-                                            d, repair)))
-                goto cleanup;
+            case '\n':
+              newline_buf[newline_off++] = *p++;
+              SVN_ERR (translate_newline (eol_str, eol_str_len, src_format,
+                                          &src_format_len, newline_buf,
+                                          newline_off, d, repair));
               newline_off = 0;
+              break;
             }
-
-          /* Write out this character. */
-          if ((err = translate_write (d, (const void *)&c, 1)))
-            goto cleanup;
-          break; 
-
-        } /* switch (c) */
+        }
     }
 
-  return SVN_NO_ERROR;
+  if (newline_off)
+    SVN_ERR (translate_newline (eol_str, eol_str_len, src_format,
+                                &src_format_len, newline_buf, newline_off, d,
+                                repair));
+  if (keyword_off)
+    SVN_ERR (translate_write (d, keyword_buf, keyword_off));
 
- cleanup:
-  return err;
+  return SVN_NO_ERROR;
 }
 
 
@@ -808,11 +698,9 @@ svn_subst_translate_cstring (const char *src,
                                     eol_str, repair, keywords, expand);
   if (err)
     {
-      svn_stream_close (src_stream);
-      svn_stream_close (dst_stream);      
-      return 
-        svn_error_create (err->apr_err, err,
-                          "stringbuf translation failed");
+      svn_error_clear (svn_stream_close (src_stream));
+      svn_error_clear (svn_stream_close (dst_stream));
+      return err;
     }
 
   /* clean up nicely. */
@@ -833,70 +721,262 @@ svn_subst_copy_and_translate (const char *src,
                               const svn_subst_keywords_t *keywords,
                               svn_boolean_t expand,
                               apr_pool_t *pool)
+{
+  return svn_subst_copy_and_translate2 (src, dst, eol_str, repair, keywords,
+                                        expand, FALSE, pool);
+}
+
+
+/* Given a special file at SRC, generate a textual representation of
+   it in a normal file at DST.  Perform all allocations in POOL. */
+static svn_error_t *
+detranslate_special_file (const char *src,
+                          const char *dst,
+                          apr_pool_t *pool)
 {
   const char *dst_tmp;
-  apr_status_t apr_err;
+  svn_string_t *buf;
+  apr_file_t *s, *d;
+  svn_stream_t *src_stream, *dst_stream;
+  apr_finfo_t finfo;
+  
+  /* First determine what type of special file we are
+     detranslating. */
+  SVN_ERR (svn_io_stat (&finfo, src, APR_FINFO_MIN | APR_FINFO_LINK, pool));
+
+  /* Open a temporary destination that we will eventually atomically
+     rename into place. */
+  SVN_ERR (svn_io_open_unique_file (&d, &dst_tmp, dst,
+                                    ".tmp", FALSE, pool));
+
+  dst_stream = svn_stream_from_aprfile (d, pool);
+  
+  switch (finfo.filetype) {
+  case APR_REG:
+    /* Nothing special to do here, just copy the original file's
+       contents. */
+    SVN_ERR (svn_io_file_open (&s, src, APR_READ | APR_BUFFERED,
+                               APR_OS_DEFAULT, pool));
+    src_stream = svn_stream_from_aprfile (d, pool);
+
+    SVN_ERR (svn_stream_copy (src_stream, dst_stream, pool));
+    break;
+  case APR_LNK:
+    /* Determine the destination of the link. */
+    SVN_ERR (svn_io_read_link (&buf, src, pool));
+
+    SVN_ERR (svn_stream_printf (dst_stream, pool, "link %s",
+                                buf->data));
+    break;
+  default:
+    abort ();
+  }
+
+  SVN_ERR (svn_io_file_close (d, pool));
+
+  /* Do the atomic rename from our temporary location. */
+  SVN_ERR (svn_io_file_rename (dst_tmp, dst, pool));
+  
+  return SVN_NO_ERROR;
+}
+
+
+/* Given a file containing a repository representation of a special
+   file in SRC, create the appropriate special file at location DST.
+   Perform all allocations in POOL. */
+static svn_error_t *
+create_special_file (const char *src,
+                     const char *dst,
+                     apr_pool_t *pool)
+{
+  svn_stringbuf_t *contents;
+  char *identifier, *remainder;
+  const char *dst_tmp, *src_tmp = NULL;
+  svn_error_t *err;
+  svn_node_kind_t kind;
+  svn_boolean_t is_special;
+
+  /* Check to see if we are being asked to create a special file from
+     a special file.  If so, do a temporary detranslation and work
+     from there. */
+  SVN_ERR (svn_io_check_special_path (src, &kind, &is_special, pool));
+
+  if (is_special)
+    {
+      apr_file_t *fp;
+      
+      SVN_ERR (svn_io_open_unique_file (&fp, &src_tmp, dst, ".tmp", FALSE,
+                                        pool));
+      SVN_ERR (svn_io_file_close (fp, pool));
+      SVN_ERR (detranslate_special_file (src, src_tmp, pool));
+      src = src_tmp;
+    }
+  
+  /* Read in the detranslated file. */
+  SVN_ERR (svn_stringbuf_from_file (&contents, src, pool));
+
+  /* If there was just a temporary detranslation, remove it now. */
+  if (src_tmp)
+    SVN_ERR (svn_io_remove_file (src_tmp, pool));
+      
+  /* Separate off the identifier.  The first space character delimits
+     the identifier, after which any remaining characters are specific
+     to the actual special device being created. */
+  identifier = contents->data;
+  for (remainder = identifier; *remainder; remainder++)
+    {
+      if (*remainder == ' ')
+        {
+          *remainder = '\0';
+          remainder++;
+          break;
+        }
+    }
+           
+  if (! strcmp (identifier, SVN_SUBST__SPECIAL_LINK_STR))
+    {
+      /* For symlinks, the type specific data is just a filesystem
+         path that the symlink should reference. */
+      err = svn_io_create_unique_link (&dst_tmp, dst, remainder,
+                                       ".tmp", pool);
+    }
+  else
+    {
+      /* We should return a valid error here. */
+      return svn_error_createf (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
+                                "Unsupported special file type '%s'",
+                                identifier);
+    }
+
+  /* If we had an error, check to see if it was because this type of
+     special device is not supported. */
+  if (err)
+    {
+      if (err->apr_err == SVN_ERR_UNSUPPORTED_FEATURE)
+        {
+          apr_file_t *fp;
+          
+          svn_error_clear (err);
+          /* Fall back to just copying the text-base. */
+          SVN_ERR (svn_io_open_unique_file (&fp, &dst_tmp, dst, ".tmp", FALSE,
+                                            pool));
+          SVN_ERR (svn_io_file_close (fp, pool));
+          SVN_ERR (svn_io_copy_file (src, dst_tmp, TRUE, pool));
+        }
+      else
+        return err;
+    }
+
+  /* Do the atomic rename from our temporary location. */
+  SVN_ERR (svn_io_file_rename (dst_tmp, dst, pool));
+
+  return SVN_NO_ERROR;
+}
+
+
+svn_error_t *
+svn_subst_copy_and_translate2 (const char *src,
+                               const char *dst,
+                               const char *eol_str,
+                               svn_boolean_t repair,
+                               const svn_subst_keywords_t *keywords,
+                               svn_boolean_t expand,
+                               svn_boolean_t special,
+                               apr_pool_t *pool)
+{
+  const char *dst_tmp = NULL;
   svn_stream_t *src_stream, *dst_stream;
   apr_file_t *s = NULL, *d = NULL;  /* init to null important for APR */
-  svn_error_t *err, *err2;
+  svn_error_t *err;
+  apr_pool_t *subpool;
+  svn_node_kind_t kind;
+  svn_boolean_t path_special;
+
+  SVN_ERR (svn_io_check_special_path (src, &kind, &path_special, pool));
+
+  /* If this is a 'special' file, we may need to create it or
+     detranslate it. */
+  if (special || path_special)
+    {
+      if (expand)
+        SVN_ERR (create_special_file (src, dst, pool));
+      else
+        SVN_ERR (detranslate_special_file (src, dst, pool));
+      
+      return SVN_NO_ERROR;
+    }
 
   /* The easy way out:  no translation needed, just copy. */
   if (! (eol_str || keywords))
     return svn_io_copy_file (src, dst, FALSE, pool);
 
-  /* Else, translate.  For atomicity, we translate to a tmp file and
-     then rename the tmp file over the real destination. */
-
-  SVN_ERR (svn_io_open_unique_file (&d, &dst_tmp, dst,
-                                    ".tmp", FALSE, pool));
+  subpool = svn_pool_create (pool);
 
   /* Open source file. */
-  SVN_ERR (svn_io_file_open (&s, src, APR_READ | APR_BUFFERED,
-                             APR_OS_DEFAULT, pool));
+  err = svn_io_file_open (&s, src, APR_READ | APR_BUFFERED,
+                          APR_OS_DEFAULT, subpool);
+  if (err)
+    goto error;
+
+  /* For atomicity, we translate to a tmp file and
+     then rename the tmp file over the real destination. */
+
+  err = svn_io_open_unique_file (&d, &dst_tmp, dst,
+                                 ".tmp", FALSE, subpool);
+
+  /* Move the file name to a more permanent pool. */
+  if (dst_tmp)
+    dst_tmp = apr_pstrdup(pool, dst_tmp);
+
+  if (err)
+    goto error;
 
   /* Now convert our two open files into streams. */
-  src_stream = svn_stream_from_aprfile (s, pool);
-  dst_stream = svn_stream_from_aprfile (d, pool);
+  src_stream = svn_stream_from_aprfile (s, subpool);
+  dst_stream = svn_stream_from_aprfile (d, subpool);
 
   /* Translate src stream into dst stream. */
   err = svn_subst_translate_stream (src_stream, dst_stream,
                                     eol_str, repair, keywords, expand);
-
   if (err)
     {
-      /* ignore closure errors if we're bailing. */
-      svn_stream_close (src_stream);
-      svn_stream_close (dst_stream);      
-      if (s)
-        apr_file_close (s);
-      if (d)
-        apr_file_close (d);      
-
-      err2 = svn_io_remove_file (dst_tmp, pool);      
-      if (err2)
-        svn_error_clear (err2);
-      return 
-        svn_error_createf (err->apr_err, err,
-                           "file translation failed when copying '%s' to '%s'",
-                           src, dst);
+      if (err->apr_err == SVN_ERR_IO_INCONSISTENT_EOL)
+        err = svn_error_createf 
+          (SVN_ERR_IO_INCONSISTENT_EOL, err,
+           _("File '%s' has inconsistent newlines"), src);
+      goto error;
     }
 
   /* clean up nicely. */
-  SVN_ERR (svn_stream_close (src_stream));
-  SVN_ERR (svn_stream_close (dst_stream));
+  err = svn_stream_close (src_stream);
+  if (err)
+    goto error;
 
-  apr_err = apr_file_close(s);
-  if (apr_err)
-    return svn_error_createf (apr_err, NULL, "error closing '%s'", src);
+  err = svn_stream_close (dst_stream);
+  if (err)
+    goto error;
 
-  apr_err = apr_file_close(d);
-  if (apr_err)
-    return svn_error_createf (apr_err, NULL, "error closing '%s'", dst);
+  err = svn_io_file_close (s, subpool);
+  if (err)
+    goto error;
+
+  err = svn_io_file_close (d, subpool);
+  if (err)
+    goto error;
 
   /* Now that dst_tmp contains the translated data, do the atomic rename. */
-  SVN_ERR (svn_io_file_rename (dst_tmp, dst, pool));
+  err = svn_io_file_rename (dst_tmp, dst, subpool);
+  if (err)
+    goto error;
 
+  svn_pool_destroy (subpool);
   return SVN_NO_ERROR;
+
+ error:
+  svn_pool_destroy (subpool);   /* Make sure all files are closed first. */
+  if (dst_tmp)
+    svn_error_clear (svn_io_remove_file (dst_tmp, pool));
+  return err;
 }
 
 
@@ -909,7 +989,6 @@ svn_subst_translate_string (svn_string_t **new_value,
 {
   const char *val_utf8;
   const char *val_utf8_lf;
-  apr_xlate_t *xlator = NULL;
 
   if (value == NULL)
     {
@@ -919,14 +998,14 @@ svn_subst_translate_string (svn_string_t **new_value,
 
   if (encoding)
     {
-      apr_status_t apr_err =  
-        apr_xlate_open (&xlator, "UTF-8", encoding, pool);
-      if (apr_err != APR_SUCCESS)
-        return svn_error_create (apr_err, NULL,
-                                 "failed to create a converter to UTF-8");
+      SVN_ERR (svn_utf_cstring_to_utf8_ex (&val_utf8, value->data,
+                                           encoding, NULL, pool));
+    }
+  else
+    {
+      SVN_ERR (svn_utf_cstring_to_utf8 (&val_utf8, value->data, pool));
     }
 
-  SVN_ERR (svn_utf_cstring_to_utf8 (&val_utf8, value->data, xlator, pool));
   SVN_ERR (svn_subst_translate_cstring (val_utf8,
                                         &val_utf8_lf,
                                         "\n",  /* translate to LF */
@@ -944,10 +1023,11 @@ svn_subst_translate_string (svn_string_t **new_value,
 svn_error_t *
 svn_subst_detranslate_string (svn_string_t **new_value,
                               const svn_string_t *value,
+                              svn_boolean_t for_output,
                               apr_pool_t *pool)
 {
   svn_error_t *err;
-  const char *val_nlocale;
+  const char *val_neol;
   const char *val_nlocale_neol;
 
   if (value == NULL)
@@ -956,20 +1036,38 @@ svn_subst_detranslate_string (svn_string_t **new_value,
       return SVN_NO_ERROR;
     }
 
-  err = svn_utf_cstring_from_utf8 (&val_nlocale, value->data, pool);
-  if (err && (APR_STATUS_IS_EINVAL (err->apr_err)))
-    val_nlocale = svn_utf_cstring_from_utf8_fuzzy (value->data, pool);
-  else if (err)
-    return err;
-
-  SVN_ERR (svn_subst_translate_cstring (val_nlocale,
-                                        &val_nlocale_neol,
+  SVN_ERR (svn_subst_translate_cstring (value->data,
+                                        &val_neol,
                                         APR_EOL_STR,  /* 'native' eol */
                                         FALSE, /* no repair */
                                         NULL,  /* no keywords */
                                         FALSE, /* no expansion */
                                         pool));
-  
+
+  if (for_output)
+    {
+      err = svn_cmdline_cstring_from_utf8 (&val_nlocale_neol, val_neol, pool);
+      if (err && (APR_STATUS_IS_EINVAL (err->apr_err)))
+        {
+          val_nlocale_neol =
+            svn_cmdline_cstring_from_utf8_fuzzy (val_neol, pool);
+          svn_error_clear (err);
+        }
+      else if (err)
+        return err;
+    }
+  else
+    {
+      err = svn_utf_cstring_from_utf8 (&val_nlocale_neol, val_neol, pool);
+      if (err && (APR_STATUS_IS_EINVAL (err->apr_err)))
+        {
+          val_nlocale_neol = svn_utf_cstring_from_utf8_fuzzy (val_neol, pool);
+          svn_error_clear (err);
+        }
+      else if (err)
+        return err;
+    }
+
   *new_value = svn_string_create (val_nlocale_neol, pool);
 
   return SVN_NO_ERROR;
