@@ -44,6 +44,8 @@
 #include "translate.h"
 #include "entries.h"
 
+#include "svn_private_config.h"
+
 
 /* Helper for report_revisions().
    
@@ -67,6 +69,7 @@ restore_file (const char *file_path,
   apr_time_t tstamp;
   const char *bname;
   apr_uint32_t modify_flags = 0;
+  svn_boolean_t special;
 
   text_base_path = svn_wc__text_base_path (file_path, FALSE, pool);
   tmp_text_base_path = svn_wc__text_base_path (file_path, TRUE, pool);
@@ -78,17 +81,20 @@ restore_file (const char *file_path,
   SVN_ERR (svn_wc__get_eol_style (NULL, &eol, file_path, adm_access, pool));
   SVN_ERR (svn_wc__get_keywords (&keywords,
                                  file_path, adm_access, NULL, pool));
+  SVN_ERR (svn_wc__get_special (&special, file_path, adm_access, pool));
+                                
   
   /* When copying the tmp-text-base out to the working copy, make
      sure to do any eol translations or keyword substitutions,
      as dictated by the property values.  If these properties
      are turned off, then this is just a normal copy. */
-  SVN_ERR (svn_subst_copy_and_translate (tmp_text_base_path,
-                                         file_path,
-                                         eol, FALSE, /* don't repair */
-                                         keywords,
-                                         TRUE, /* expand keywords */
-                                         pool));
+  SVN_ERR (svn_subst_copy_and_translate2 (tmp_text_base_path,
+                                          file_path,
+                                          eol, FALSE, /* don't repair */
+                                          keywords,
+                                          TRUE, /* expand keywords */
+                                          special,
+                                          pool));
   
   SVN_ERR (svn_io_remove_file (tmp_text_base_path, pool));
 
@@ -103,7 +109,7 @@ restore_file (const char *file_path,
   assert(entry != NULL);
 
   /* Possibly set timestamp to last-commit-time. */
-  if (use_commit_times)
+  if (use_commit_times && (! special))
     {
       SVN_ERR (svn_io_set_file_affected_time (entry->cmt_date,
                                               file_path, pool));
@@ -212,7 +218,7 @@ report_revisions (svn_wc_adm_access_t *adm_access,
       apr_ssize_t klen;
       void *val;
       const svn_wc_entry_t *current_entry; 
-      svn_node_kind_t *dirent_kind, dirent_kind_storage;
+      svn_node_kind_t *dirent_kind;
       svn_boolean_t missing = FALSE;
 
       /* Clear the iteration subpool here because the loop has a bunch
@@ -252,12 +258,14 @@ report_revisions (svn_wc_adm_access_t *adm_access,
           /* It is possible on a case insensitive system that the
              entry is not really missing, so we call our trusty but
              expensive friend svn_io_check_path to be sure. */
-          SVN_ERR (svn_io_check_path (this_full_path, &dirent_kind_storage,
+          dirent_kind = apr_palloc (iterpool, sizeof(*dirent_kind)); 
+          SVN_ERR (svn_io_check_path (this_full_path, dirent_kind,
                                       iterpool));
-          if (dirent_kind_storage == svn_node_none)
-            missing = TRUE;
-          else
-            dirent_kind = &dirent_kind_storage;
+          if (*dirent_kind == svn_node_none)
+            {
+              dirent_kind = NULL;
+              missing = TRUE;
+            }
         }
       
       /* From here on out, ignore any entry scheduled for addition */
@@ -363,8 +371,8 @@ report_revisions (svn_wc_adm_access_t *adm_access,
             {
               return svn_error_createf
                 (SVN_ERR_WC_OBSTRUCTED_UPDATE, NULL,
-                 "The entry '%s' is no longer a directory; "
-                 "remove the entry before updating",
+                 _("The entry '%s' is no longer a directory; "
+                   "remove the entry before updating"),
                  this_path);
             }
 
@@ -399,8 +407,9 @@ report_revisions (svn_wc_adm_access_t *adm_access,
                                           subdir_entry->revision,
                                           subdir_entry->incomplete,
                                           iterpool));
-          /* ... or perhaps just a differing revision. */
-          else if (subdir_entry->revision != dir_rev)
+          /* ... or perhaps just a differing revision or incomplete subdir. */
+          else if (subdir_entry->revision != dir_rev
+                   || subdir_entry->incomplete)
             SVN_ERR (reporter->set_path (report_baton,
                                          this_path,
                                          subdir_entry->revision,
@@ -456,6 +465,27 @@ svn_wc_crawl_revisions (const char *path,
      ROOT_DIRECTORY.  This is the first revnum that entries will be
      compared to. */
   SVN_ERR (svn_wc_entry (&entry, path, adm_access, FALSE, pool));
+
+  if ((! entry) || ((entry->schedule == svn_wc_schedule_add)
+                    && (entry->kind == svn_node_dir)))
+    {
+      SVN_ERR (svn_wc_entry (&parent_entry,
+                             svn_path_dirname (path, pool),
+                             adm_access,
+                             FALSE, pool));
+      base_rev = parent_entry->revision;
+      SVN_ERR (reporter->set_path (report_baton, "", base_rev,
+                                   entry ? entry->incomplete : TRUE, 
+                                   pool));
+      SVN_ERR (reporter->delete_path (report_baton, "", pool)); 
+
+      /* Finish the report, which causes the update editor to be
+         driven. */
+      SVN_ERR (reporter->finish_report (report_baton, pool));
+
+      return SVN_NO_ERROR;
+    }
+
   base_rev = entry->revision;
   if (base_rev == SVN_INVALID_REVNUM)
     {
@@ -582,7 +612,7 @@ svn_wc_crawl_revisions (const char *path,
       svn_error_t *fserr;
       if ((fserr = reporter->abort_report (report_baton, pool)))
         {
-          fserr = svn_error_quick_wrap (fserr, "Error aborting report");
+          fserr = svn_error_quick_wrap (fserr, _("Error aborting report"));
           svn_error_compose (err, fserr);
         }
       return err;
@@ -679,8 +709,8 @@ svn_wc_transmit_text_deltas (const char *path,
                   
                   return svn_error_createf
                     (SVN_ERR_WC_CORRUPT_TEXT_BASE, NULL,
-                     "Checksum mismatch for '%s': "
-                     "expected '%s', actual: '%s'",
+                     _("Checksum mismatch for '%s'; "
+                       "expected '%s', actual: '%s'"),
                      tb, ent->checksum, base_digest_hex);
                 }
             }
@@ -704,7 +734,7 @@ svn_wc_transmit_text_deltas (const char *path,
   /* Open a filehandle for tmp text-base. */
   SVN_ERR_W (svn_io_file_open (&localfile, tmp_base,
                                APR_READ, APR_OS_DEFAULT, pool),
-             "Error opening local file");
+             _("Error opening local file"));
 
   /* Create a text-delta stream object that pulls data out of the two
      files. */
