@@ -2,7 +2,7 @@
  * lock.c:  routines for locking working copy subdirectories.
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -26,6 +26,9 @@
 
 #include "wc.h"
 #include "adm_files.h"
+#include "questions.h"
+
+#include "svn_private_config.h"
 
 
 
@@ -64,10 +67,10 @@ struct svn_wc_adm_access_t
   apr_hash_t *set;
 
   /* ENTRIES is the cached entries for PATH, without those in state
-     deleted. ENTRIES_DELETED is the cached entries including those in
-     state deleted. Either may be NULL. */
+     deleted. ENTRIES_HIDDEN is the cached entries including those in
+     state deleted or state absent. Either may be NULL. */
   apr_hash_t *entries;
-  apr_hash_t *entries_deleted;
+  apr_hash_t *entries_hidden;
 
   /* POOL is used to allocate cached items, they need to persist for the
      lifetime of this access baton */
@@ -84,9 +87,47 @@ static svn_wc_adm_access_t missing;
 static svn_error_t *
 do_close (svn_wc_adm_access_t *adm_access, svn_boolean_t preserve_lock);
 
+
+/* Maybe upgrade the working copy directory represented by ADM_ACCESS
+   to the latest 'SVN_WC__VERSION'.  ADM_ACCESS must contain a write
+   lock.  Use POOL for all temporary allocation.
+
+   Not all upgrade paths are necessarily supported.  For example,
+   upgrading a version 1 working copy results in an error.
+
+   Sometimes the format file can contain "0" while the administrative
+   directory is being constructed; calling this on a format 0 working
+   copy has no effect and returns no error. */
+static svn_error_t *
+maybe_upgrade_format (svn_wc_adm_access_t *adm_access, apr_pool_t *pool)
+{
+  SVN_ERR (svn_wc__check_format (adm_access->wc_format,
+                                 adm_access->path,
+                                 pool));
+
+  /* We can upgrade all formats that are accepted by
+     svn_wc__check_format. */
+  if (adm_access->wc_format != SVN_WC__VERSION)
+    {
+      const char *path = svn_wc__adm_path (adm_access->path, FALSE, pool,
+                                           SVN_WC__ADM_FORMAT, NULL);
+
+      SVN_ERR (svn_io_write_version_file (path, SVN_WC__VERSION, pool));
+      adm_access->wc_format = SVN_WC__VERSION;
+    }
+
+  return SVN_NO_ERROR;
+}
+
+
 /* Create a physical lock file in the admin directory for ADM_ACCESS. Wait
    up to WAIT_FOR seconds if the lock already exists retrying every
-   second. */
+   second. 
+
+   Note: most callers of this function determine the wc_format for the
+   lock soon afterwards.  We recommend calling maybe_upgrade_format()
+   as soon as you have the wc_format for a lock, since that's a good
+   opportunity to drag old working directories into the modern era. */
 static svn_error_t *
 create_lock (svn_wc_adm_access_t *adm_access, int wait_for, apr_pool_t *pool)
 {
@@ -114,7 +155,7 @@ create_lock (svn_wc_adm_access_t *adm_access, int wait_for, apr_pool_t *pool)
     }
 
   return svn_error_createf (SVN_ERR_WC_LOCKED, NULL,
-                            "working copy locked: %s",
+                            _("Working copy '%s' locked"),
                             svn_path_local_style (adm_access->path, pool));
 }
 
@@ -126,9 +167,14 @@ create_lock (svn_wc_adm_access_t *adm_access, int wait_for, apr_pool_t *pool)
 static svn_error_t *
 remove_lock (const char *path, apr_pool_t *pool)
 {
-  if (svn_wc__adm_path_exists (path, FALSE, pool, NULL))
-    SVN_ERR (svn_wc__remove_adm_file (path, pool, SVN_WC__ADM_LOCK, NULL));
-
+  svn_error_t *err = svn_wc__remove_adm_file (path, pool, SVN_WC__ADM_LOCK,
+                                              NULL);
+  if (err)
+    {
+      if (svn_wc__adm_path_exists (path, FALSE, pool, NULL))
+        return err;
+      svn_error_clear (err);
+    }
   return SVN_NO_ERROR;
 }
 
@@ -177,7 +223,7 @@ adm_access_alloc (enum svn_wc__adm_access_type type,
   svn_wc_adm_access_t *lock = apr_palloc (pool, sizeof (*lock));
   lock->type = type;
   lock->entries = NULL;
-  lock->entries_deleted = NULL;
+  lock->entries_hidden = NULL;
   lock->wc_format = 0;
   lock->set = NULL;
   lock->lock_exists = FALSE;
@@ -216,7 +262,26 @@ probe (const char **dir,
 
   /* a "version" of 0 means a non-wc directory */
   if (kind != svn_node_dir || *wc_format == 0)
-    *dir = svn_path_dirname (path, pool);
+    {
+      /* Passing a path ending in "." or ".." to svn_path_dirname() is
+         probably always a bad idea; certainly it is in this case.
+         Unfortunately, svn_path_dirname()'s current signature can't
+         return an error, so we have to insert the protection in this
+         caller, as making the larger API change would be very
+         destabilizing right now (just before 1.0).  See issue #1617. */
+      const char *base_name = svn_path_basename (path, pool);
+      if ((strcmp (base_name, "..") == 0)
+          || (strcmp (base_name, ".") == 0))
+        {
+          return svn_error_createf
+            (SVN_ERR_WC_BAD_PATH, NULL,
+             _("Path '%s' ends in '%s', "
+               "which is unsupported for this operation"),
+             svn_path_local_style (path, pool), base_name);
+        }
+
+      *dir = svn_path_dirname (path, pool);
+    }
   else
     *dir = path;
 
@@ -250,12 +315,17 @@ svn_wc__adm_steal_write_lock (svn_wc_adm_access_t **adm_access,
       apr_hash_set (lock->set, lock->path, APR_HASH_KEY_STRING, lock);
     }
 
+  /* We have a write lock.  If the working copy has an old
+     format, this is the time to upgrade it. */
+  SVN_ERR (svn_wc_check_wc (path, &lock->wc_format, pool));
+  SVN_ERR (maybe_upgrade_format (lock, pool));
+  
   lock->lock_exists = TRUE;
   *adm_access = lock;
   return SVN_NO_ERROR;
 }
 
-/* This is essentially the guts of svn_wc_adm_open, with the additional
+/* This is essentially the guts of svn_wc_adm_open2, with the additional
  * parameter UNDER_CONSTRUCTION that gets set TRUE only when locking the
  * admin directory during initial creation.
  */
@@ -264,7 +334,7 @@ do_open (svn_wc_adm_access_t **adm_access,
          svn_wc_adm_access_t *associated,
          const char *path,
          svn_boolean_t write_lock,
-         svn_boolean_t tree_lock,
+         int depth,
          svn_boolean_t under_construction,
          apr_pool_t *pool)
 {
@@ -283,7 +353,7 @@ do_open (svn_wc_adm_access_t **adm_access,
            locked: if it's not locked call svn_wc_adm_open, if it is locked
            call svn_wc_adm_retrieve.  */
         return svn_error_createf (SVN_ERR_WC_LOCKED, NULL,
-                                  "directory already locked (%s)",
+                                  _("Working copy '%s' locked"),
                                   path);
     }
 
@@ -300,12 +370,14 @@ do_open (svn_wc_adm_access_t **adm_access,
         {
           /* Should we attempt to distinguish certain errors? */
           svn_error_clear (err);
-          wc_format = 0;
+          return svn_error_createf (SVN_ERR_WC_NOT_DIRECTORY, NULL,
+                                    _("'%s' is not a working copy"),
+                                    svn_path_local_style (path, pool));
         }
-      if (wc_format == 0 || wc_format > SVN_WC__VERSION)
-        return svn_error_createf (SVN_ERR_WC_NOT_DIRECTORY, NULL,
-                                  "'%s' is not a working copy",
-                                  svn_path_local_style (path, pool));
+
+      SVN_ERR (svn_wc__check_format (wc_format,
+                                     svn_path_local_style (path, pool),
+                                     pool));
     }
 
   /* Need to create a new lock */
@@ -319,15 +391,24 @@ do_open (svn_wc_adm_access_t **adm_access,
     {
       lock = adm_access_alloc (svn_wc__adm_access_unlocked, path, pool);
     }
-  if (! under_construction)
-    lock->wc_format = wc_format;
 
-  if (tree_lock)
+  if (! under_construction)
+    {
+      lock->wc_format = wc_format;
+      if (write_lock)
+        SVN_ERR (maybe_upgrade_format (lock, pool));
+    }
+
+  if (depth != 0)
     {
       apr_hash_t *entries;
       apr_hash_index_t *hi;
       apr_pool_t *subpool = svn_pool_create (pool);
 
+      /* Reduce depth since we are about to recurse */
+      if (depth > 0)
+        depth--;
+      
       /* Ask for the deleted entries because most operations request them
          at some stage, getting them now avoids a second file parse. */
       SVN_ERR (svn_wc_entries_read (&entries, lock, TRUE, subpool));
@@ -346,26 +427,28 @@ do_open (svn_wc_adm_access_t **adm_access,
 
           apr_hash_this (hi, NULL, NULL, &val);
           entry = val;
-          if ((entry->deleted && entry->schedule != svn_wc_schedule_add)
+          if ((entry->deleted
+               && (entry->schedule != svn_wc_schedule_add)
+               && (entry->schedule != svn_wc_schedule_replace))
               || entry->kind != svn_node_dir
               || ! strcmp (entry->name, SVN_WC_ENTRY_THIS_DIR))
             continue;
           entry_path = svn_path_join (lock->path, entry->name, subpool);
 
           /* Don't use the subpool pool here, the lock needs to persist */
-          err = do_open (&entry_access, lock, entry_path, write_lock, tree_lock,
+          err = do_open (&entry_access, lock, entry_path, write_lock, depth,
                          FALSE, lock->pool);
           if (err)
             {
               if (err->apr_err != SVN_ERR_WC_NOT_DIRECTORY)
                 {
                   /* This closes all the children in temporary hash as well */
-                  svn_wc_adm_close (lock);
+                  svn_error_clear (svn_wc_adm_close (lock));
                   svn_pool_destroy (subpool);
                   return err;
                 }
 
-              /* It's a missing, or obstructed, so store a placeholder */
+              /* It's missing or obstructed, so store a placeholder */
               svn_error_clear (err);
               adm_ensure_set (lock);
               apr_hash_set (lock->set, apr_pstrdup (lock->pool, entry_path),
@@ -420,6 +503,7 @@ do_open (svn_wc_adm_access_t **adm_access,
   return SVN_NO_ERROR;
 }
 
+/* To preserve API compatibility with Subversion 1.0.0 */
 svn_error_t *
 svn_wc_adm_open (svn_wc_adm_access_t **adm_access,
                  svn_wc_adm_access_t *associated,
@@ -428,7 +512,19 @@ svn_wc_adm_open (svn_wc_adm_access_t **adm_access,
                  svn_boolean_t tree_lock,
                  apr_pool_t *pool)
 {
-  return do_open (adm_access, associated, path, write_lock, tree_lock, FALSE,
+  return svn_wc_adm_open2 (adm_access, associated, path, write_lock,
+                           (tree_lock ? -1 : 0), pool);
+}
+
+svn_error_t *
+svn_wc_adm_open2 (svn_wc_adm_access_t **adm_access,
+                  svn_wc_adm_access_t *associated,
+                  const char *path,
+                  svn_boolean_t write_lock,
+                  int depth,
+                  apr_pool_t *pool)
+{
+  return do_open (adm_access, associated, path, write_lock, depth, FALSE,
                   pool);
 }
 
@@ -437,10 +533,11 @@ svn_wc__adm_pre_open (svn_wc_adm_access_t **adm_access,
                       const char *path,
                       apr_pool_t *pool)
 {
-  return do_open (adm_access, NULL, path, TRUE, FALSE, TRUE, pool);
+  return do_open (adm_access, NULL, path, TRUE, 0, TRUE, pool);
 }
-     
 
+
+/* To preserve API compatibility with Subversion 1.0.0 */
 svn_error_t *
 svn_wc_adm_probe_open (svn_wc_adm_access_t **adm_access,
                        svn_wc_adm_access_t *associated,
@@ -448,6 +545,19 @@ svn_wc_adm_probe_open (svn_wc_adm_access_t **adm_access,
                        svn_boolean_t write_lock,
                        svn_boolean_t tree_lock,
                        apr_pool_t *pool)
+{
+  return svn_wc_adm_probe_open2 (adm_access, associated, path,
+                                 write_lock, (tree_lock ? -1 : 0), pool);
+}
+
+
+svn_error_t *
+svn_wc_adm_probe_open2 (svn_wc_adm_access_t **adm_access,
+                        svn_wc_adm_access_t *associated,
+                        const char *path,
+                        svn_boolean_t write_lock,
+                        int depth,
+                        apr_pool_t *pool)
 {
   svn_error_t *err;
   const char *dir;
@@ -456,30 +566,37 @@ svn_wc_adm_probe_open (svn_wc_adm_access_t **adm_access,
   SVN_ERR (probe (&dir, path, &wc_format, pool));
 
   /* If we moved up a directory, then the path is not a directory, or it
-     is not under version control. In either case, the notion of a tree_lock
+     is not under version control. In either case, the notion of a depth
      does not apply to the provided path. Disable it so that we don't end
      up trying to lock more than we need.  */
   if (dir != path)
-    tree_lock = FALSE;
+    depth = 0;
 
-  err = svn_wc_adm_open (adm_access, associated, dir, write_lock, tree_lock,
-                         pool);
+  err = svn_wc_adm_open2 (adm_access, associated, dir, write_lock, 
+                          depth, pool);
   if (err)
     {
+      svn_error_t *err2;
+
       /* If we got an error on the parent dir, that means we failed to
          get an access baton for the child in the first place.  And if
          the reason we couldn't get the child access baton is that the
          child is not a versioned directory, then return an error
          about the child, not the parent. */ 
       svn_node_kind_t child_kind;
-      SVN_ERR (svn_io_check_path (path, &child_kind, pool));
-
+      if ((err2 = svn_io_check_path (path, &child_kind, pool)))
+        {
+          svn_error_compose (err, err2);
+          return err;
+        }
+  
       if ((dir != path)
           && (child_kind == svn_node_dir)
           && (err->apr_err == SVN_ERR_WC_NOT_DIRECTORY))
         {
+          svn_error_clear (err);
           return svn_error_createf (SVN_ERR_WC_NOT_DIRECTORY, NULL,
-                                    "'%s' is not a working copy",
+                                    _("'%s' is not a working copy"),
                                     svn_path_local_style (path, pool));
         }
       else
@@ -496,10 +613,10 @@ svn_wc_adm_probe_open (svn_wc_adm_access_t **adm_access,
 
 
 svn_error_t *
-svn_wc_adm_retrieve (svn_wc_adm_access_t **adm_access,
-                     svn_wc_adm_access_t *associated,
-                     const char *path,
-                     apr_pool_t *pool)
+svn_wc__adm_retrieve_internal (svn_wc_adm_access_t **adm_access,
+                               svn_wc_adm_access_t *associated,
+                               const char *path,
+                               apr_pool_t *pool)
 {
   if (associated->set)
     *adm_access = apr_hash_get (associated->set, path, APR_HASH_KEY_STRING);
@@ -507,10 +624,29 @@ svn_wc_adm_retrieve (svn_wc_adm_access_t **adm_access,
     *adm_access = associated;
   else
     *adm_access = NULL;
-  if (! *adm_access || *adm_access == &missing)
+
+  if (*adm_access == &missing)
+    *adm_access = NULL;
+
+  return SVN_NO_ERROR;
+}
+
+svn_error_t *
+svn_wc_adm_retrieve (svn_wc_adm_access_t **adm_access,
+                     svn_wc_adm_access_t *associated,
+                     const char *path,
+                     apr_pool_t *pool)
+{
+  SVN_ERR (svn_wc__adm_retrieve_internal (adm_access, associated, path, pool));
+
+  /* Most of the code expects access batons to exist, so returning an error
+     generally makes the calling code simpler as it doesn't need to check
+     for NULL batons. */
+  if (! *adm_access)
     return svn_error_createf (SVN_ERR_WC_NOT_LOCKED, NULL,
-                              "directory not locked (%s)",
+                              _("Working copy '%s' not locked"),
                               path);
+
   return SVN_NO_ERROR;
 }
 
@@ -534,6 +670,7 @@ svn_wc_adm_probe_retrieve (svn_wc_adm_access_t **adm_access,
 }
 
 
+/* To preserve API compatibility with Subversion 1.0.0 */
 svn_error_t *
 svn_wc_adm_probe_try (svn_wc_adm_access_t **adm_access,
                       svn_wc_adm_access_t *associated,
@@ -541,6 +678,18 @@ svn_wc_adm_probe_try (svn_wc_adm_access_t **adm_access,
                       svn_boolean_t write_lock,
                       svn_boolean_t tree_lock,
                       apr_pool_t *pool)
+{
+  return svn_wc_adm_probe_try2 (adm_access, associated, path, write_lock,
+                                (tree_lock ? -1 : 0), pool);
+}
+
+svn_error_t *
+svn_wc_adm_probe_try2 (svn_wc_adm_access_t **adm_access,
+                       svn_wc_adm_access_t *associated,
+                       const char *path,
+                       svn_boolean_t write_lock,
+                       int depth,
+                       apr_pool_t *pool)
 {
   svn_error_t *err;
 
@@ -552,9 +701,9 @@ svn_wc_adm_probe_try (svn_wc_adm_access_t **adm_access,
   if (err && (err->apr_err == SVN_ERR_WC_NOT_LOCKED))
     {
       svn_error_clear (err);
-      err = svn_wc_adm_probe_open (adm_access, associated,
-                                   path, write_lock, tree_lock,
-                                   svn_wc_adm_access_pool (associated));
+      err = svn_wc_adm_probe_open2 (adm_access, associated,
+                                    path, write_lock, depth,
+                                    svn_wc_adm_access_pool (associated));
 
       /* If the path is not a versioned directory, we just return a
          null access baton with no error.  Note that of the errors we
@@ -663,27 +812,40 @@ svn_wc_adm_close (svn_wc_adm_access_t *adm_access)
   return do_close (adm_access, FALSE);
 }
 
+svn_boolean_t
+svn_wc_adm_locked (svn_wc_adm_access_t *adm_access)
+{
+  return adm_access->type == svn_wc__adm_access_write_lock;
+}
+
 svn_error_t *
-svn_wc_adm_write_check (svn_wc_adm_access_t *adm_access)
+svn_wc__adm_write_check (svn_wc_adm_access_t *adm_access)
 {
   if (adm_access->type == svn_wc__adm_access_write_lock)
     {
       if (adm_access->lock_exists)
         {
+          /* Check physical lock still exists and hasn't been stolen.  This
+             really is paranoia, I have only ever seen one report of this
+             triggering (from someone using the 0.25 release) and that was
+             never reproduced.  The check accesses the physical filesystem
+             so it is expensive, but it only runs when we are going to
+             modify the admin area.  If it ever proves to be a bottleneck
+             the physical check could be removed, just leaving the logical
+             check. */
           svn_boolean_t locked;
 
-          /* Check physical lock still exists and hasn't been stolen */
           SVN_ERR (svn_wc_locked (&locked, adm_access->path, adm_access->pool));
           if (! locked)
             return svn_error_createf (SVN_ERR_WC_NOT_LOCKED, NULL, 
-                                      "write-lock stolen in: %s",
+                                      _("Write-lock stolen in '%s'"),
                                       adm_access->path); 
         }
     }
   else
     {
       return svn_error_createf (SVN_ERR_WC_NOT_LOCKED, NULL, 
-                                "no write-lock in: %s", adm_access->path); 
+                                _("No write-lock in '%s'"), adm_access->path);
     }
 
   return SVN_NO_ERROR;
@@ -703,8 +865,7 @@ svn_wc_locked (svn_boolean_t *locked, const char *path, apr_pool_t *pool)
     *locked = FALSE;
   else
     return svn_error_createf (SVN_ERR_WC_LOCKED, NULL,
-                              "svn_wc_locked: "
-                              "lock file is not a regular file (%s)",
+                              _("Lock file '%s' is not a regular file"),
                               lockfile);
     
   return SVN_NO_ERROR;
@@ -750,13 +911,13 @@ static void
 prune_deleted (svn_wc_adm_access_t *adm_access,
                apr_pool_t *pool)
 {
-  if (! adm_access->entries && adm_access->entries_deleted)
+  if (! adm_access->entries && adm_access->entries_hidden)
     {
       apr_hash_index_t *hi;
 
       /* I think it will be common for there to be no deleted entries, so
          it is worth checking for that case as we can optimise it. */
-      for (hi = apr_hash_first (pool, adm_access->entries_deleted);
+      for (hi = apr_hash_first (pool, adm_access->entries_hidden);
            hi;
            hi = apr_hash_next (hi))
         {
@@ -764,20 +925,23 @@ prune_deleted (svn_wc_adm_access_t *adm_access,
           const svn_wc_entry_t *entry;
           apr_hash_this (hi, NULL, NULL, &val);
           entry = val;
-          if (entry->deleted && entry->schedule != svn_wc_schedule_add)
+          if ((entry->deleted
+               && (entry->schedule != svn_wc_schedule_add)
+               && (entry->schedule != svn_wc_schedule_replace))
+              || entry->absent)
             break;
         }
 
       if (! hi)
         {
           /* There are no deleted entries, so we can use the full hash */
-          adm_access->entries = adm_access->entries_deleted;
+          adm_access->entries = adm_access->entries_hidden;
           return;
         }
 
       /* Construct pruned hash without deleted entries */
       adm_access->entries = apr_hash_make (adm_access->pool);
-      for (hi = apr_hash_first (pool, adm_access->entries_deleted);
+      for (hi = apr_hash_first (pool, adm_access->entries_hidden);
            hi;
            hi = apr_hash_next (hi))
         {
@@ -787,8 +951,13 @@ prune_deleted (svn_wc_adm_access_t *adm_access,
 
           apr_hash_this (hi, &key, NULL, &val);
           entry = val;
-          if (! entry->deleted || entry->schedule == svn_wc_schedule_add)
-            apr_hash_set (adm_access->entries, key, APR_HASH_KEY_STRING, entry);
+          if (((entry->deleted == FALSE) && (entry->absent == FALSE))
+              || (entry->schedule == svn_wc_schedule_add)
+              || (entry->schedule == svn_wc_schedule_replace))
+            {
+              apr_hash_set (adm_access->entries, key,
+                            APR_HASH_KEY_STRING, entry);
+            }
         }
     }
 }
@@ -796,11 +965,11 @@ prune_deleted (svn_wc_adm_access_t *adm_access,
 
 void
 svn_wc__adm_access_set_entries (svn_wc_adm_access_t *adm_access,
-                                svn_boolean_t show_deleted,
+                                svn_boolean_t show_hidden,
                                 apr_hash_t *entries)
 {
-  if (show_deleted)
-    adm_access->entries_deleted = entries;
+  if (show_hidden)
+    adm_access->entries_hidden = entries;
   else
     adm_access->entries = entries;
 }
@@ -808,16 +977,16 @@ svn_wc__adm_access_set_entries (svn_wc_adm_access_t *adm_access,
 
 apr_hash_t *
 svn_wc__adm_access_entries (svn_wc_adm_access_t *adm_access,
-                            svn_boolean_t show_deleted,
+                            svn_boolean_t show_hidden,
                             apr_pool_t *pool)
 {
-  if (! show_deleted)
+  if (! show_hidden)
     {
       prune_deleted (adm_access, pool);
       return adm_access->entries;
     }
   else
-    return adm_access->entries_deleted;
+    return adm_access->entries_hidden;
 }
 
 
