@@ -2,7 +2,7 @@
  * props.c :  routines dealing with properties in the working copy
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -51,87 +51,12 @@
 #include "translate.h"
 #include "questions.h"
 
+#include "svn_private_config.h"
 
 /*---------------------------------------------------------------------*/
 
 /*** Deducing local changes to properties ***/
 
-svn_error_t *
-svn_wc_get_local_propchanges (apr_array_header_t **local_propchanges,
-                              apr_hash_t *localprops,
-                              apr_hash_t *baseprops,
-                              apr_pool_t *pool)
-{
-  apr_hash_index_t *hi;
-  apr_array_header_t *ary = apr_array_make (pool, 1, sizeof(svn_prop_t));
-
-  /* Note: we will be storing the pointers to the keys (from the hashes)
-     into the local_propchanges array. It is acceptable for us to
-     reference the same memory as the base/localprops hash. */
-
-  /* Loop over baseprops and examine each key.  This will allow us to
-     detect any `deletion' events or `set-modification' events.  */
-  for (hi = apr_hash_first (pool, baseprops); hi; hi = apr_hash_next (hi))
-    {
-      const void *key;
-      apr_ssize_t klen;
-      void *val;
-      const svn_string_t *propval1, *propval2;
-
-      /* Get next property */
-      apr_hash_this (hi, &key, &klen, &val);
-      propval1 = val;
-
-      /* Does property name exist in localprops? */
-      propval2 = apr_hash_get (localprops, key, klen);
-
-      if (propval2 == NULL)
-        {
-          /* Add a delete event to the array */
-          svn_prop_t *p = apr_array_push (ary);
-          p->name = key;
-          p->value = NULL;
-        }
-      else if (! svn_string_compare (propval1, propval2))
-        {
-          /* Add a set (modification) event to the array */
-          svn_prop_t *p = apr_array_push (ary);
-          p->name = key;
-          p->value = svn_string_dup (propval2, pool);
-        } 
-    }
-
-  /* Loop over localprops and examine each key.  This allows us to
-     detect `set-creation' events */
-  for (hi = apr_hash_first (pool, localprops); hi; hi = apr_hash_next (hi))
-    {
-      const void *key;
-      apr_ssize_t klen;
-      void *val;
-      const svn_string_t *propval;
-
-      /* Get next property */
-      apr_hash_this (hi, &key, &klen, &val);
-      propval = val;
-
-      /* Does property name exist in baseprops? */
-      if (NULL == apr_hash_get (baseprops, key, klen))
-        {
-          /* Add a set (creation) event to the array */
-          svn_prop_t *p = apr_array_push (ary);
-          p->name = key;
-          p->value = svn_string_dup (propval, pool);
-        }
-    }
-
-
-  /* Done building our array of user events. */
-  *local_propchanges = ary;
-
-  return SVN_NO_ERROR;
-}
-
-
 /*---------------------------------------------------------------------*/
 
 /*** Detecting a property conflict ***/
@@ -174,7 +99,8 @@ svn_wc__conflicting_propchanges_p (const svn_string_t **description,
     {
       *description =
         svn_string_createf
-        (pool, "prop `%s': user set value to '%s', but update deletes it.\n",
+        (pool,
+         _("Property '%s' locally changed to '%s', but update deletes it\n"),
          local->name, local->value->data);
       return TRUE;  /* conflict */
     }
@@ -182,7 +108,8 @@ svn_wc__conflicting_propchanges_p (const svn_string_t **description,
     {
       *description =
         svn_string_createf
-        (pool, "prop `%s': user deleted, but update sets it to '%s'.\n",
+        (pool,
+         _("Property '%s' locally deleted, but update sets it to '%s'\n"),
          local->name, update->value->data);
       return TRUE;  /* conflict */
     }
@@ -198,7 +125,8 @@ svn_wc__conflicting_propchanges_p (const svn_string_t **description,
     {
       *description =
         svn_string_createf
-        (pool, "prop `%s': user set to '%s', but update set to '%s'.\n",
+        (pool, _("Property '%s' locally changed to '%s', "
+                 "but update sets it to '%s'\n"),
          local->name, local->value->data, update->value->data);
       return TRUE;  /* conflict */
     }
@@ -233,25 +161,16 @@ svn_wc__load_prop_file (const char *propfile_path,
   if (kind == svn_node_file)
     {
       /* Ah, this file already has on-disk properties.  Load 'em. */
-      apr_status_t status;
       apr_file_t *propfile = NULL;
 
-      SVN_ERR_W (svn_io_file_open (&propfile, propfile_path,
-                                   APR_READ | APR_BUFFERED, APR_OS_DEFAULT,
-                                   pool),
-                 "load_prop_file: can't open propfile");
+      SVN_ERR (svn_io_file_open (&propfile, propfile_path,
+                                 APR_READ | APR_BUFFERED, APR_OS_DEFAULT,
+                                 pool));
 
-      status = svn_hash_read (hash, propfile, pool);
-      if (status)
-        return svn_error_createf (status, NULL,
-                                  "load_prop_file:  can't parse `%s'",
-                                  propfile_path);
+      SVN_ERR_W (svn_hash_read (hash, propfile, pool),
+                 apr_psprintf (pool, _("Can't parse '%s'"), propfile_path));
 
-      status = apr_file_close (propfile);
-      if (status)
-        return svn_error_createf (status, NULL,
-                                  "load_prop_file: can't close `%s'",
-                                  propfile_path);
+      SVN_ERR (svn_io_file_close (propfile, pool));
     }
 
   return SVN_NO_ERROR;
@@ -266,26 +185,19 @@ svn_wc__save_prop_file (const char *propfile_path,
                         apr_hash_t *hash,
                         apr_pool_t *pool)
 {
-  apr_status_t apr_err;
   apr_file_t *prop_tmp;
 
-  SVN_ERR_W (svn_io_file_open (&prop_tmp, propfile_path,
-                               (APR_WRITE | APR_CREATE | APR_TRUNCATE
-                                | APR_BUFFERED),
-                               APR_OS_DEFAULT, pool),
-             "save_prop_file: can't open propfile");
+  SVN_ERR (svn_io_file_open (&prop_tmp, propfile_path,
+                             (APR_WRITE | APR_CREATE | APR_TRUNCATE
+                              | APR_BUFFERED), 
+                             APR_OS_DEFAULT, pool));
 
-  apr_err = svn_hash_write (hash, prop_tmp, pool);
-  if (apr_err)
-    return svn_error_createf (apr_err, NULL,
-                              "save_prop_file: can't write prop hash to `%s'",
-                              propfile_path);
+  SVN_ERR_W (svn_hash_write (hash, prop_tmp, pool),
+             apr_psprintf (pool, 
+                           _("Can't write property hash to '%s'"),
+                           propfile_path));
 
-  apr_err = apr_file_close (prop_tmp);
-  if (apr_err)
-    return svn_error_createf (apr_err, NULL,
-                              "save_prop_file: can't close `%s'",
-                              propfile_path);
+  SVN_ERR (svn_io_file_close (prop_tmp, pool));
 
   return SVN_NO_ERROR;
 }
@@ -305,19 +217,16 @@ append_prop_conflict (apr_file_t *fp,
   /* TODO:  someday, perhaps prefix each conflict_description with a
      timestamp or something? */
   apr_size_t written;
-  apr_status_t status;
   const svn_string_t *conflict_description_native;
 
   SVN_ERR (svn_utf_string_from_utf8 (&conflict_description_native,
                                      conflict_description,
                                      pool));
 
-  status = apr_file_write_full (fp, conflict_description_native->data,
-                                conflict_description_native->len, &written);
-  if (status)
-    return svn_error_create (status, NULL,
-                             "append_prop_conflict: "
-                             "apr_file_write_full failed.");
+  SVN_ERR (svn_io_file_write_full (fp, conflict_description_native->data,
+                                   conflict_description_native->len, &written,
+                                   pool));
+
   return SVN_NO_ERROR;
 }
 
@@ -338,7 +247,7 @@ svn_wc__get_existing_prop_reject_file (const char **reject_file,
   if (! the_entry)
     return svn_error_createf
       (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-       "get_existing_reject_prop_reject_file: can't find entry '%s' in '%s'",
+       _("Can't find entry '%s' in '%s'"),
        name, svn_wc_adm_access_path (adm_access));
 
   *reject_file = the_entry->prejfile 
@@ -362,7 +271,6 @@ svn_wc_merge_prop_diffs (svn_wc_notify_state_t *state,
                          svn_boolean_t dry_run,
                          apr_pool_t *pool)
 {
-  apr_status_t apr_err;
   const svn_wc_entry_t *entry;
   const char *parent, *base_name;
   svn_stringbuf_t *log_accum;
@@ -370,9 +278,8 @@ svn_wc_merge_prop_diffs (svn_wc_notify_state_t *state,
 
   SVN_ERR (svn_wc_entry (&entry, path, adm_access, FALSE, pool));
   if (entry == NULL)
-    return svn_error_createf (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-                              "Can't merge props into '%s':"
-                              "it's not under revision control.", path);
+    return svn_error_createf (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
+                              _("'%s' is not under version control"), path);
 
   /* Notice that we're not using svn_path_split_if_file(), because
      that looks at the actual working file.  It's existence shouldn't
@@ -406,15 +313,9 @@ svn_wc_merge_prop_diffs (svn_wc_notify_state_t *state,
 
   if (! dry_run)
     {
-      apr_err = apr_file_write_full (log_fp, log_accum->data, 
-                                     log_accum->len, NULL);
-      if (apr_err)
-        {
-          apr_file_close (log_fp);
-          return svn_error_createf (apr_err, NULL,
-                                    "svn_wc_merge_prop_diffs:"
-                                    "error writing log for '%s'", path);
-        }
+      SVN_ERR_W (svn_io_file_write_full (log_fp, log_accum->data, 
+                                         log_accum->len, NULL, pool),
+                 apr_psprintf (pool, _("Error writing log for '%s'"), path));
 
       SVN_ERR (svn_wc__close_adm_file (log_fp, parent, SVN_WC__ADM_LOG,
                                        1, /* sync */ pool));
@@ -446,7 +347,7 @@ svn_wc__merge_prop_diffs (svn_wc_notify_state_t *state,
 
   const char *access_path = svn_wc_adm_access_path (adm_access);
   int access_len = strlen (access_path);
-  int slash = access_len ? 1 : 0;   /* "foo/.svn/..." versus ".svn/..." */
+  int slash;
 
   const char *entryname;
   const char *full_path;
@@ -463,6 +364,12 @@ svn_wc__merge_prop_diffs (svn_wc_notify_state_t *state,
   apr_file_t *reject_tmp_fp = NULL;       /* the temporary conflicts file */
   const char *reject_tmp_path = NULL;
 
+  /* Empty path and paths ending in / don't need an extra slash removed */
+  if (access_len == 0 || access_path[access_len - 1] == '/')
+    slash = 0;
+  else
+    slash = 1;
+
   if (name == NULL)
     {
       /* We must be merging props on the directory PATH  */
@@ -495,8 +402,7 @@ svn_wc__merge_prop_diffs (svn_wc_notify_state_t *state,
   
   /* Deduce any local propchanges the user has made since the last
      update.  */
-  SVN_ERR (svn_wc_get_local_propchanges (&local_propchanges,
-                                         localhash, basehash, pool));
+  SVN_ERR (svn_prop_diffs (&local_propchanges, localhash, basehash, pool));
   
   if (state)
     {
@@ -721,12 +627,7 @@ svn_wc__merge_prop_diffs (svn_wc_notify_state_t *state,
 
       /* First, _close_ this temporary conflicts file.  We've been
          appending to it all along. */
-      apr_status_t status;
-      status = apr_file_close (reject_tmp_fp);
-      if (status)
-        return svn_error_createf (status, NULL,
-                                  "do_property_merge: can't close '%s'",
-                                  reject_tmp_path);
+      SVN_ERR (svn_io_file_close (reject_tmp_fp, pool));
                                   
       /* Now try to get the name of a pre-existing .prej file from the
          entries file */
@@ -754,11 +655,7 @@ svn_wc__merge_prop_diffs (svn_wc_notify_state_t *state,
                                             FALSE,
                                             pool));
 
-          status = apr_file_close (reject_fp);
-          if (status)
-            return svn_error_createf (status, NULL,
-                                      "do_property_merge: can't close '%s'",
-                                      full_reject_path);
+          SVN_ERR (svn_io_file_close (reject_fp, pool));
           
           /* This file will be overwritten when the log is run; that's
              ok, because at least now we have a reservation on
@@ -836,12 +733,12 @@ wcprop_list (apr_hash_t **props,
 #if 0
   if (kind == svn_node_none)
     return svn_error_createf (SVN_ERR_BAD_FILENAME, NULL,
-                              "wcprop_list: non-existent path '%s'.",
+                              _("'%s' does not exist"),
                               path);
   
   if (kind == svn_node_unknown)
     return svn_error_createf (SVN_ERR_NODE_UNKNOWN_KIND, NULL,
-                              "wcprop_list: unknown node kind: '%s'.",
+                              _("Unknown node kind: '%s'"),
                               path);
 #endif
 
@@ -877,7 +774,7 @@ svn_wc__wcprop_get (const svn_string_t **value,
   if (err)
     return
       svn_error_quick_wrap
-      (err, "svn_wc__wcprop_get: failed to load props from disk.");
+      (err, _("Failed to load properties from disk"));
 
   *value = apr_hash_get (prophash, name, APR_HASH_KEY_STRING);
   return SVN_NO_ERROR;
@@ -892,7 +789,6 @@ svn_wc__wcprop_set (const char *name,
                     apr_pool_t *pool)
 {
   svn_error_t *err;
-  apr_status_t apr_err;
   apr_hash_t *prophash;
   apr_file_t *fp = NULL;
 
@@ -900,7 +796,7 @@ svn_wc__wcprop_set (const char *name,
   if (err)
     return
       svn_error_quick_wrap
-      (err, "svn_wc__wcprop_set: failed to load props from disk.");
+      (err, _("Failed to load properties from disk"));
 
   /* Now we have all the properties in our hash.  Simply merge the new
      property into it. */
@@ -914,10 +810,9 @@ svn_wc__wcprop_set (const char *name,
                                1, /* we DO want wcprops */
                                pool));
   /* Write. */
-  apr_err = svn_hash_write (prophash, fp, pool);
-  if (apr_err)
-    return svn_error_createf (apr_err, NULL,
-                              "can't write prop hash for '%s'", path);
+  SVN_ERR_W (svn_hash_write (prophash, fp, pool),
+             apr_psprintf (pool,
+                           _("Cannot write property hash for '%s'"), path));
   
   /* Close file, and doing an atomic "move". */
   SVN_ERR (svn_wc__close_props (fp, path, 0, 1,
@@ -987,7 +882,7 @@ svn_wc_prop_get (const svn_string_t **value,
     {
       return svn_error_createf   /* we don't do entry properties here */
         (SVN_ERR_BAD_PROP_KIND, NULL,
-         "property '%s' is an entry property", name);
+         _("Property '%s' is an entry property"), name);
     }
   else  /* regular prop */
     {
@@ -995,7 +890,7 @@ svn_wc_prop_get (const svn_string_t **value,
       if (err)
         return
           svn_error_quick_wrap
-          (err, "svn_wc_prop_get: failed to load props from disk.");
+          (err, _("Failed to load properties from disk"));
       
       *value = apr_hash_get (prophash, name, APR_HASH_KEY_STRING);
 
@@ -1023,29 +918,30 @@ validate_prop_against_node_kind (const char *name,
                                  SVN_PROP_MIME_TYPE,
                                  NULL };
   const char **node_kind_prohibit;
-  const char *node_kind_text;
 
   switch (node_kind)
     {
     case svn_node_dir:
       node_kind_prohibit = dir_prohibit;
-      node_kind_text = "directory";
+      while (*node_kind_prohibit)
+        if (strcmp (name, *node_kind_prohibit++) == 0)
+          return svn_error_createf (SVN_ERR_ILLEGAL_TARGET, NULL,
+                                    _("Cannot set '%s' on a directory (%s)"),
+                                    name, path);
       break;
     case svn_node_file:
       node_kind_prohibit = file_prohibit;
-      node_kind_text = "file";
+      while (*node_kind_prohibit)
+        if (strcmp (name, *node_kind_prohibit++) == 0)
+          return svn_error_createf (SVN_ERR_ILLEGAL_TARGET, NULL,
+                                    _("Cannot set '%s' on a file (%s)"),
+                                    name, path);
       break;
     default:
       return svn_error_createf (SVN_ERR_NODE_UNEXPECTED_KIND, NULL,
-                                "'%s' is not a file or directory", path);
+                                _("'%s' is not a file or directory"), path);
     }
 
-  while (*node_kind_prohibit)
-    if (strcmp (name, *node_kind_prohibit++) == 0)
-      return svn_error_createf (SVN_ERR_ILLEGAL_TARGET, NULL,
-                                "Cannot set '%s' on a %s (%s)",
-                                name, node_kind_text, path);
-
   return SVN_NO_ERROR;
 }                             
 
@@ -1064,8 +960,9 @@ validate_eol_prop_against_file (const char *path,
   SVN_ERR (svn_wc_prop_get (&mime_type, SVN_PROP_MIME_TYPE, path, adm_access,
                             pool));
   if (mime_type && svn_mime_type_is_binary (mime_type->data))
-    return svn_error_createf (SVN_ERR_ILLEGAL_TARGET, NULL,
-                              "File '%s' has binary mimetype property", path);
+    return svn_error_createf 
+      (SVN_ERR_ILLEGAL_TARGET, NULL,
+       _("File '%s' has binary mime type property"), path);
 
   /* Open PATH. */
   SVN_ERR (svn_io_file_open (&fp, path, 
@@ -1086,8 +983,12 @@ validate_eol_prop_against_file (const char *path,
                                     "", FALSE, NULL, FALSE);
   if (err && err->apr_err == SVN_ERR_IO_INCONSISTENT_EOL)
     return svn_error_createf (SVN_ERR_ILLEGAL_TARGET, err,
-                              "File '%s' has inconsistent newlines", path);
-  return err;
+                              _("File '%s' has inconsistent newlines"), path);
+  else if (err)
+    return err;
+
+  SVN_ERR (svn_io_file_close (fp, pool));
+  return SVN_NO_ERROR;
 }
 
 
@@ -1099,7 +1000,6 @@ svn_wc_prop_set (const char *name,
                  apr_pool_t *pool)
 {
   svn_error_t *err;
-  apr_status_t apr_err;
   apr_hash_t *prophash;
   apr_file_t *fp = NULL;
   svn_subst_keywords_t *old_keywords;
@@ -1114,7 +1014,7 @@ svn_wc_prop_set (const char *name,
   else if (prop_kind == svn_prop_entry_kind)
     return svn_error_createf   /* we don't do entry properties here */
       (SVN_ERR_BAD_PROP_KIND, NULL,
-       "property '%s' is an entry property", name);
+       _("Property '%s' is an entry property"), name);
 
   /* Else, handle a regular property: */
 
@@ -1145,6 +1045,18 @@ svn_wc_prop_set (const char *name,
               new_value = svn_stringbuf_create_from_string (value, pool);
               svn_stringbuf_appendbytes (new_value, "\n", 1);
             }
+
+          /* Make sure this is a valid externals property. */
+          if (strcmp (name, SVN_PROP_EXTERNALS) == 0)
+            {
+              /* We don't allow "." nor ".." as target directories in
+                 an svn:externals line.  As it happens, our parse code
+                 checks for this, so all we have to is invoke it --
+                 we're not interested in the parsed result, only in
+                 whether or the parsing errored. */
+              SVN_ERR (svn_wc_parse_externals_description2
+                       (NULL, path, value->data, pool));
+            }
         }
       else if (strcmp (name, SVN_PROP_KEYWORDS) == 0)
         {
@@ -1181,7 +1093,7 @@ svn_wc_prop_set (const char *name,
   if (err)
     return
       svn_error_quick_wrap
-      (err, "svn_wc_prop_set: failed to load props from disk.");
+      (err, _("Failed to load properties from disk"));
 
   /* If we're changing this file's list of expanded keywords, then
    * we'll need to invalidate its text timestamp, since keyword
@@ -1209,10 +1121,9 @@ svn_wc_prop_set (const char *name,
                                0, /* not wcprops */
                                pool));
   /* Write. */
-  apr_err = svn_hash_write (prophash, fp, pool);
-  if (apr_err)
-    return svn_error_createf (apr_err, NULL,
-                              "can't write prop hash for '%s'", path);
+  SVN_ERR_W (svn_hash_write (prophash, fp, pool),
+             apr_psprintf (pool, 
+                           _("Cannot write property hash for '%s'"), path));
   
   /* Close file, and doing an atomic "move". */
   SVN_ERR (svn_wc__close_props (fp, path, 0, 0,
@@ -1280,33 +1191,39 @@ empty_props_p (svn_boolean_t *empty_p,
                const char *path_to_prop_file,
                apr_pool_t *pool)
 {
-  svn_node_kind_t kind;
+  svn_error_t *err;
+  apr_finfo_t finfo;
 
-  SVN_ERR (svn_io_check_path (path_to_prop_file, &kind, pool));
+  err = svn_io_stat (&finfo, path_to_prop_file, APR_FINFO_MIN | APR_FINFO_TYPE,
+                     pool);
+  if (err)
+    {
+      if (! APR_STATUS_IS_ENOENT (err->apr_err)
+          && ! APR_STATUS_IS_ENOTDIR (err->apr_err))
+        return err;
 
-  if (kind == svn_node_none)
-    *empty_p = TRUE;
-
-  else 
+      /* nonexistent */
+      svn_error_clear (err);
+      *empty_p = TRUE;
+    }
+  else
     {
-      apr_finfo_t finfo;
 
-      SVN_ERR (svn_io_stat (&finfo, path_to_prop_file, APR_FINFO_MIN, pool));
 
       /* If we remove props from a propfile, eventually the file will
          contain nothing but "END\n" */
-      if (finfo.size == 4)  
+      if (finfo.filetype == APR_REG && finfo.size == 4)  
         *empty_p = TRUE;
 
       else
         *empty_p = FALSE;
 
-      /* ### really, if the size is < 4, then something is corrupt.
+      /* If the size is < 4, then something is corrupt.
          If the size is between 4 and 16, then something is corrupt,
          because 16 is the -smallest- the file can possibly be if it
-         contained only one property.  someday we should check for
-         this. */
-
+         contained only one property.  So long as we say it is "not
+         empty", we will discover such corruption later when we try
+         to read the properties from the file. */
     }
 
   return SVN_NO_ERROR;
@@ -1361,14 +1278,8 @@ svn_wc_props_modified_p (svn_boolean_t *modified_p,
      pay attention to any base-props;  they might be residual from the
      old deleted file. */
   SVN_ERR (svn_wc_entry (&entry, path, adm_access, TRUE, subpool));  
-  if (entry 
-      && ((entry->schedule == svn_wc_schedule_replace)
-          || (entry->schedule == svn_wc_schedule_add)))
+  if (entry && (entry->schedule == svn_wc_schedule_replace))
     {
-      /* svn_wc_add() guarantees that a newly added file has no
-         working props at all; thus if this file is non-empty, the
-         user must have modified them.  Hopefully the caller will know
-         to ignore the baseprops as well!  */
       *modified_p = wempty ? FALSE : TRUE;
       goto cleanup;        
     }
@@ -1404,7 +1315,7 @@ svn_wc_props_modified_p (svn_boolean_t *modified_p,
   
   /* There are at least three tests we can try in succession. */
   
-  /* Easy-answer attempt #1:  */
+  /* Easy-answer attempt #1:  (### this stat's the files again) */
   
   /* Check if the local and prop-base file have *definitely* different
      filesizes. */
@@ -1418,7 +1329,7 @@ svn_wc_props_modified_p (svn_boolean_t *modified_p,
       goto cleanup;
     }
   
-  /* Easy-answer attempt #2:  */
+  /* Easy-answer attempt #2:  (### this stat's the files again) */
       
   /* See if the local file's prop timestamp is the same as the one
      recorded in the administrative directory.  */
@@ -1443,25 +1354,48 @@ svn_wc_props_modified_p (svn_boolean_t *modified_p,
      guarantees about ordering.)
 
      Therefore, rather than use contents_identical_p(), we use
-     svn_wc_get_local_propchanges(). */
+     svn_prop_diffs(). */
   {
     apr_array_header_t *local_propchanges;
     apr_hash_t *localprops = apr_hash_make (subpool);
     apr_hash_t *baseprops = apr_hash_make (subpool);
 
+    /* ### Amazingly, this stats the files again! */
     SVN_ERR (svn_wc__load_prop_file (prop_path, localprops, subpool));
     SVN_ERR (svn_wc__load_prop_file (prop_base_path,
                                      baseprops,
                                      subpool));
-    SVN_ERR (svn_wc_get_local_propchanges (&local_propchanges,
-                                           localprops,
-                                           baseprops,
-                                           subpool));
+    SVN_ERR (svn_prop_diffs (&local_propchanges, localprops, 
+                             baseprops, subpool));
                                          
     if (local_propchanges->nelts > 0)
       *modified_p = TRUE;
     else
       *modified_p = FALSE;
+
+    /* If it turns out that there are no differences then we might be able
+       to "repair" the prop-time in the entries file and avoid the
+       expensive file contents comparison next time.
+
+       ### Unlike the text-time in svn_wc_text_modified_p the only
+       ### "legitimate" way to produce a prop-time variation with no
+       ### corresponding property variation, is by using the Subversion
+       ### property interface.  Perhaps those functions should detect the
+       ### change that restores the pristine values and reset the
+       ### prop-time?  This code would still be needed, to handle someone
+       ### or something manually changing the timestamp on the
+       ### prop-base. */
+    if (! *modified_p && svn_wc_adm_locked (adm_access))
+      {
+        svn_wc_entry_t tmp;
+        SVN_ERR (svn_io_file_affected_time (&tmp.prop_time, prop_path, pool));
+        SVN_ERR (svn_wc__entry_modify (adm_access,
+                                       (entry->kind == svn_node_dir
+                                        ? SVN_WC_ENTRY_THIS_DIR
+                                        : svn_path_basename (path, pool)),
+                                       &tmp, SVN_WC__ENTRY_MODIFY_PROP_TIME,
+                                       TRUE, pool));
+      }
   }
  
  cleanup:
@@ -1500,12 +1434,158 @@ svn_wc_get_prop_diffs (apr_array_header_t **propchanges,
   
   if (propchanges != NULL)
     {
-      SVN_ERR (svn_wc_get_local_propchanges (&local_propchanges,
-                                             localprops,
-                                             baseprops,
-                                             pool));      
+      SVN_ERR (svn_prop_diffs (&local_propchanges, localprops, 
+                               baseprops, pool));      
       *propchanges = local_propchanges;
     }
 
   return SVN_NO_ERROR;
 }
+
+
+
+/** Externals **/
+
+svn_error_t *
+svn_wc_parse_externals_description2 (apr_array_header_t **externals_p,
+                                     const char *parent_directory,
+                                     const char *desc,
+                                     apr_pool_t *pool)
+{
+  apr_array_header_t *lines = svn_cstring_split (desc, "\n\r", TRUE, pool);
+  int i;
+  
+  if (externals_p)
+    *externals_p = apr_array_make (pool, 1, sizeof (svn_wc_external_item_t *));
+
+  for (i = 0; i < lines->nelts; i++)
+    {
+      const char *line = APR_ARRAY_IDX (lines, i, const char *);
+      apr_array_header_t *line_parts;
+      svn_wc_external_item_t *item;
+
+      if ((! line) || (line[0] == '#'))
+        continue;
+
+      /* else proceed */
+
+      line_parts = svn_cstring_split (line, " \t", TRUE, pool);
+
+      item = apr_palloc (pool, sizeof (*item));
+
+      if (line_parts->nelts < 2)
+        goto parse_error;
+
+      else if (line_parts->nelts == 2)
+        {
+          /* No "-r REV" given. */
+          item->target_dir = APR_ARRAY_IDX (line_parts, 0, const char *);
+          item->url = APR_ARRAY_IDX (line_parts, 1, const char *);
+          item->revision.kind = svn_opt_revision_head;
+        }
+      else if ((line_parts->nelts == 3) || (line_parts->nelts == 4))
+        {
+          /* We're dealing with one of these two forms:
+           * 
+           *    TARGET_DIR  -rN  URL
+           *    TARGET_DIR  -r N  URL
+           * 
+           * Handle either way.
+           */
+
+          const char *r_part_1 = NULL, *r_part_2 = NULL;
+
+          item->target_dir = APR_ARRAY_IDX (line_parts, 0, const char *);
+          item->revision.kind = svn_opt_revision_number;
+
+          if (line_parts->nelts == 3)
+            {
+              r_part_1 = APR_ARRAY_IDX (line_parts, 1, const char *);
+              item->url = APR_ARRAY_IDX (line_parts, 2, const char *);
+            }
+          else  /* nelts == 4 */
+            {
+              r_part_1 = APR_ARRAY_IDX (line_parts, 1, const char *);
+              r_part_2 = APR_ARRAY_IDX (line_parts, 2, const char *);
+              item->url = APR_ARRAY_IDX (line_parts, 3, const char *);
+            }
+
+          if ((! r_part_1) || (r_part_1[0] != '-') || (r_part_1[1] != 'r'))
+            goto parse_error;
+
+          if (! r_part_2)  /* "-rN" */
+            {
+              if (strlen (r_part_1) < 3)
+                goto parse_error;
+              else
+                item->revision.value.number = SVN_STR_TO_REV (r_part_1 + 2);
+            }
+          else             /* "-r N" */
+            {
+              if (strlen (r_part_2) < 1)
+                goto parse_error;
+              else
+                item->revision.value.number = SVN_STR_TO_REV (r_part_2);
+            }
+        }
+      else    /* too many items on line */
+        goto parse_error;
+
+      if (0)
+        {
+        parse_error:
+          return svn_error_createf
+            (SVN_ERR_CLIENT_INVALID_EXTERNALS_DESCRIPTION, NULL,
+             _("Error parsing %s property on '%s': '%s'"),
+             SVN_PROP_EXTERNALS, parent_directory, line);
+        }
+
+      item->target_dir = svn_path_canonicalize
+        (svn_path_internal_style (item->target_dir, pool), pool);
+      {
+        if (item->target_dir[0] == '\0' || item->target_dir[0] == '/'
+            || svn_path_is_backpath_present (item->target_dir))
+          return svn_error_createf
+            (SVN_ERR_CLIENT_INVALID_EXTERNALS_DESCRIPTION, NULL,
+             _("Invalid %s property on '%s': "
+               "target involves '.' or '..' or is an absolute path"),
+             SVN_PROP_EXTERNALS, parent_directory);
+      }
+
+      item->url = svn_path_canonicalize (item->url, pool);
+
+      if (externals_p)
+        APR_ARRAY_PUSH (*externals_p, svn_wc_external_item_t *) = item;
+    }
+
+  return SVN_NO_ERROR;
+}
+
+
+svn_error_t *
+svn_wc_parse_externals_description (apr_hash_t **externals_p,
+                                    const char *parent_directory,
+                                    const char *desc,
+                                    apr_pool_t *pool)
+{
+  apr_array_header_t *list;
+  int i;
+
+  SVN_ERR (svn_wc_parse_externals_description2 (externals_p ? &list : NULL,
+                                                parent_directory, desc, pool));
+
+  /* Store all of the items into the hash if that was requested. */
+  if (externals_p)
+    {
+      *externals_p = apr_hash_make (pool);
+      for (i = 0; i < list->nelts; i++)
+        {
+          svn_wc_external_item_t *item;
+          item = APR_ARRAY_IDX (list, i, svn_wc_external_item_t *);
+
+          apr_hash_set (*externals_p, item->target_dir,
+                        APR_HASH_KEY_STRING, item);
+        }
+    }
+  return SVN_NO_ERROR;
+}
