@@ -88,6 +88,15 @@ svn_error_t *svn_wc__get_keywords (svn_subst_keywords_t **keywords,
                                    apr_pool_t *pool);
 
 
+/* Determine if the svn:special flag is set on PATH.  If so, set
+   SPECIAL to TRUE, if not, set it to FALSE.  ADM_ACCESS must be an
+   access baton for PATH.  Perform any temporary allocations in
+   POOL. */
+svn_error_t *svn_wc__get_special (svn_boolean_t *special,
+                                  const char *path,
+                                  svn_wc_adm_access_t *adm_access,
+                                  apr_pool_t *pool);
+
 /* If the SVN_PROP_EXECUTABLE property is present at all, then set
    PATH executable.  If DID_SET is non-null, then set *DID_SET to
    TRUE if did set PATH executable, or to FALSE if not.  ADM_ACCESS
