@@ -102,29 +102,26 @@ typedef struct {
 #define DIR_OR_FILE(is_dir) ((is_dir) ? "directory" : "file")
 
 
-struct authz_read_baton
+svn_error_t *dav_svn_authz_read(svn_boolean_t *allowed,
+                                svn_fs_root_t *root,
+                                const char *path,
+                                void *baton,
+                                apr_pool_t *pool)
 {
-  /* The original request, needed to generate a subrequest. */
-  request_rec *r;
-
-  /* We need this to construct a URI based on a repository abs path. */
-  const dav_svn_repos *repos;
-};
-
-
-/* This implements 'svn_repos_authz_func_t'. */
-static svn_error_t *authz_read(svn_boolean_t *allowed,
-                               svn_fs_root_t *root,
-                               const char *path,
-                               void *baton,
-                               apr_pool_t *pool)
-{
-  struct authz_read_baton *arb = baton;
+  dav_svn_authz_read_baton *arb = baton;
   request_rec *subreq = NULL;
   const char *uri;
   svn_revnum_t rev = SVN_INVALID_REVNUM;
   const char *revpath = NULL;
 
+  /* Easy out:  if the admin has explicitly set 'SVNPathAuthz Off',
+     then this whole callback does nothing. */
+  if (! dav_svn_get_pathauthz_flag(arb->r))
+    {
+      *allowed = TRUE;
+      return SVN_NO_ERROR;
+    }
+
   /* Our ultimate goal here is to create a Version Resource (VR) url,
      which is a url that represents a path within a revision.  We then
      send a subrequest to apache, so that any installed authz modules
@@ -321,9 +318,9 @@ static svn_error_t * brigade_write_fn(void *baton,
 }
 
 
-static svn_stream_t * make_base64_output_stream(apr_bucket_brigade *bb,
-                                                ap_filter_t *output,
-                                                apr_pool_t *pool)
+svn_stream_t * dav_svn_make_base64_output_stream(apr_bucket_brigade *bb,
+                                                 ap_filter_t *output,
+                                                 apr_pool_t *pool)
 {
   struct brigade_write_baton *wb = apr_palloc(pool, sizeof(*wb));
   svn_stream_t *stream = svn_stream_create(wb, pool);
@@ -493,16 +490,15 @@ static svn_error_t * add_helper(svn_boolean_t is_dir,
 
           if (bc_url)
             elt = apr_psprintf(pool, "<S:add-%s name=\"%s\" "
-                               "copyfrom-path=\"%s\" copyfrom-rev=\"%"
-                               SVN_REVNUM_T_FMT "\" "
+                               "copyfrom-path=\"%s\" copyfrom-rev=\"%ld\" "
                                "bc-url=\"%s\">" DEBUG_CR,
                                DIR_OR_FILE(is_dir),
                                qname, qcopy, copyfrom_revision,
                                bc_url);
           else
             elt = apr_psprintf(pool, "<S:add-%s name=\"%s\" "
-                               "copyfrom-path=\"%s\" copyfrom-rev=\"%"
-                               SVN_REVNUM_T_FMT "\">" DEBUG_CR,
+                               "copyfrom-path=\"%s\""
+                               " copyfrom-rev=\"%ld\">" DEBUG_CR,
                                DIR_OR_FILE(is_dir),
                                qname, qcopy, copyfrom_revision);
         }
@@ -535,8 +531,8 @@ static svn_error_t * open_helper(svn_boolean_t is_dir,
   item_baton_t *child = make_child_baton(parent, path, pool);
   const char *qname = apr_xml_quote_string(pool, child->name, 1);
 
-  SVN_ERR( send_xml(child->uc, "<S:open-%s name=\"%s\" rev=\"%"
-                    SVN_REVNUM_T_FMT "\">" DEBUG_CR,
+  SVN_ERR( send_xml(child->uc, "<S:open-%s name=\"%s\""
+                    " rev=\"%ld\">" DEBUG_CR,
                     DIR_OR_FILE(is_dir), qname, base_revision));
   SVN_ERR( send_vsn_url(child, pool) );
   *child_baton = child;
@@ -654,7 +650,7 @@ static svn_error_t * upd_set_target_revision(void *edit_baton,
   SVN_ERR( maybe_start_update_report(uc) );
 
   if (! uc->resource_walk)
-    SVN_ERR( send_xml(uc, "<S:target-revision rev=\"%" SVN_REVNUM_T_FMT 
+    SVN_ERR( send_xml(uc, "<S:target-revision rev=\"%ld"
                       "\"/>" DEBUG_CR, target_revision) );
 
   return SVN_NO_ERROR;
@@ -688,7 +684,7 @@ static svn_error_t * upd_open_root(void *edit_baton,
     }
   else    
     {
-      SVN_ERR( send_xml(uc, "<S:open-directory rev=\"%" SVN_REVNUM_T_FMT "\">"
+      SVN_ERR( send_xml(uc, "<S:open-directory rev=\"%ld\">"
                         DEBUG_CR, base_revision) );
     }
 
@@ -933,8 +929,8 @@ static svn_error_t * upd_apply_textdelta(void *file_baton,
 
   wb->seen_first_window = FALSE;
   wb->uc = file->uc;
-  base64_stream = make_base64_output_stream(wb->uc->bb, wb->uc->output,
-                                            file->pool);
+  base64_stream = dav_svn_make_base64_output_stream(wb->uc->bb, wb->uc->output,
+                                                    file->pool);
 
   svn_txdelta_to_svndiff(base64_stream, file->pool,
                          &(wb->handler), &(wb->handler_baton));
@@ -1002,7 +998,7 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
   svn_boolean_t recurse = TRUE;
   svn_boolean_t resource_walk = FALSE;
   svn_boolean_t ignore_ancestry = FALSE;
-  struct authz_read_baton arb;
+  dav_svn_authz_read_baton arb;
   apr_pool_t *subpool = svn_pool_create(resource->pool);
 
   /* Construct the authz read check baton. */
@@ -1243,7 +1239,7 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
                                      recurse,
                                      ignore_ancestry,
                                      editor, &uc,
-                                     authz_read,
+                                     dav_svn_authz_read,
                                      &arb,
                                      resource->pool)))
     {
@@ -1293,7 +1289,7 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
                 goto cleanup;
               }
 
-            /* get cdata, stipping whitespace */
+            /* get cdata, stripping whitespace */
             path = dav_xml_get_cdata(child, subpool, 1);
             
             if (! linkpath)
@@ -1326,7 +1322,7 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
           }
         else if (strcmp(child->name, "missing") == 0)
           {
-            /* get cdata, stipping whitespace */
+            /* get cdata, stripping whitespace */
             const char *path = dav_xml_get_cdata(child, subpool, 1);
             serr = svn_repos_delete_path(rbaton, path, subpool);
             if (serr != NULL)
@@ -1412,7 +1408,7 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
                                      uc.rev_root, dst_path,
                                      /* re-use the editor */
                                      editor, &uc,
-                                     authz_read,
+                                     dav_svn_authz_read,
                                      &arb,
                                      FALSE, /* no text deltas */
                                      recurse,
