@@ -37,6 +37,8 @@
 #include "svn_utf.h"
 #include "svn_path.h"
 #include "svn_opt.h"
+#include "svn_repos.h"
+#include "svn_version.h"
 
 #include "server.h"
 
@@ -54,7 +56,7 @@ enum run_mode {
   run_mode_inetd,
   run_mode_daemon,
   run_mode_tunnel,
-  run_mode_listen_once,
+  run_mode_listen_once
 };
 
 #if APR_HAS_FORK
@@ -91,6 +93,8 @@ enum run_mode {
 #define SVNSERVE_OPT_LISTEN_PORT 256
 #define SVNSERVE_OPT_LISTEN_HOST 257
 #define SVNSERVE_OPT_FOREGROUND  258
+#define SVNSERVE_OPT_TUNNEL_USER 259
+#define SVNSERVE_OPT_VERSION     260
 
 static const apr_getopt_option_t svnserve__options[] =
   {
@@ -102,10 +106,14 @@ static const apr_getopt_option_t svnserve__options[] =
     {"foreground",        SVNSERVE_OPT_FOREGROUND, 0,
      "run in foreground (useful for debugging)"},
     {"help",             'h', 0, "display this help"},
+    {"version",           SVNSERVE_OPT_VERSION, 0,
+     "show version information"},
     {"inetd",            'i', 0, "inetd mode"},
     {"root",             'r', 1, "root of directory to serve"},
     {"read-only",        'R', 0, "deprecated; use repository config file"},
     {"tunnel",           't', 0, "tunnel mode"},
+    {"tunnel-user",      SVNSERVE_OPT_TUNNEL_USER, 1,
+     "tunnel username (default is current uid's name)"},
 #ifdef CONNECTION_HAVE_THREAD_OPTION
     {"threads",          'T', 0, "use threads instead of fork"},
 #endif
@@ -140,6 +148,13 @@ static void help(apr_pool_t *pool)
   exit(1);
 }
 
+static svn_error_t * version(apr_getopt_t *os, apr_pool_t *pool)
+{
+  return svn_opt_print_help(os, "svnserve", TRUE, FALSE, NULL, NULL,
+                            NULL, NULL, NULL, pool);
+}
+  
+
 #if APR_HAS_FORK
 static void sigchld_handler(int signo)
 {
@@ -165,9 +180,8 @@ static apr_status_t redirect_stdout(void *arg)
 
 /* "Arguments" passed from the main thread to the connection thread */
 struct serve_thread_t {
-  const char *root;
   svn_ra_svn_conn_t *conn;
-  svn_boolean_t read_only;
+  serve_params_t *params;
   apr_pool_t *pool;
 };
 
@@ -176,17 +190,36 @@ static void * APR_THREAD_FUNC serve_thread(apr_thread_t *tid, void *data)
 {
   struct serve_thread_t *d = data;
 
-  svn_error_clear(serve(d->conn, d->root, FALSE, d->read_only, d->pool));
+  svn_error_clear(serve(d->conn, d->params, d->pool));
   svn_pool_destroy(d->pool);
 
   return NULL;
 }
 #endif
 
+/* Version compatibility check */
+static svn_error_t *
+check_lib_versions (void)
+{
+  static const svn_version_checklist_t checklist[] =
+    {
+      { "svn_subr",  svn_subr_version },
+      { "svn_repos", svn_repos_version },
+      { "svn_fs",    svn_fs_version },
+      { "svn_delta", svn_delta_version },
+      { "svn_ra_svn", svn_ra_svn_version },
+      { NULL, NULL }
+    };
+
+   SVN_VERSION_DEFINE (my_version);
+   return svn_ver_check_list (&my_version, checklist);
+}
+
+
 int main(int argc, const char *const *argv)
 {
   enum run_mode run_mode = run_mode_none;
-  svn_boolean_t read_only = FALSE, foreground = FALSE;
+  svn_boolean_t foreground = FALSE;
   apr_socket_t *sock, *usock;
   apr_file_t *in_file, *out_file;
   apr_sockaddr_t *sa;
@@ -196,7 +229,8 @@ int main(int argc, const char *const *argv)
   apr_getopt_t *os;
   char errbuf[256];
   int opt;
-  const char *arg, *root = "/";
+  serve_params_t params;
+  const char *arg;
   apr_status_t status;
   svn_ra_svn_conn_t *conn;
   apr_proc_t proc;
@@ -216,8 +250,22 @@ int main(int argc, const char *const *argv)
   /* Create our top-level pool. */
   pool = svn_pool_create(NULL);
 
+  /* Check library versions */
+  err = check_lib_versions ();
+  if (err)
+    {
+      svn_handle_error (err, stderr, FALSE);
+      svn_error_clear (err);
+      svn_pool_destroy (pool);
+      return EXIT_FAILURE;
+    }
+
   apr_getopt_init(&os, pool, argc, argv);
 
+  params.root = "/";
+  params.tunnel = FALSE;
+  params.tunnel_user = NULL;
+  params.read_only = FALSE;
   while (1)
     {
       status = apr_getopt_long(os, svnserve__options, &opt, &arg);
@@ -231,6 +279,11 @@ int main(int argc, const char *const *argv)
           help(pool);
           break;
 
+        case SVNSERVE_OPT_VERSION:
+          SVN_INT_ERR(version(os, pool));
+          exit(0);
+          break;
+          
         case 'd':
           run_mode = run_mode_daemon;
           break;
@@ -255,18 +308,22 @@ int main(int argc, const char *const *argv)
           run_mode = run_mode_tunnel;
           break;
 
+        case SVNSERVE_OPT_TUNNEL_USER:
+          params.tunnel_user = arg;
+          break;
+
         case 'X':
           run_mode = run_mode_listen_once;
           break;
 
         case 'r':
-          SVN_INT_ERR(svn_utf_cstring_to_utf8(&root, arg, pool));
-          root = svn_path_internal_style(root, pool);
-          SVN_INT_ERR(svn_path_get_absolute(&root, root, pool));
+          SVN_INT_ERR(svn_utf_cstring_to_utf8(&params.root, arg, pool));
+          params.root = svn_path_internal_style(params.root, pool);
+          SVN_INT_ERR(svn_path_get_absolute(&params.root, params.root, pool));
           break;
 
         case 'R':
-          read_only = TRUE;
+          params.read_only = TRUE;
           fprintf(stderr, "Warning: -R is deprecated.\n");
           fprintf(stderr, "Anonymous access is now read-only by default.\n");
           fprintf(stderr, "To change, use conf/svnserve.conf in repos:\n");
@@ -284,21 +341,27 @@ int main(int argc, const char *const *argv)
   if (os->ind != argc)
     usage(argv[0]);
 
+  if (params.tunnel_user && run_mode != run_mode_tunnel)
+    {
+      fprintf(stderr, "Option --tunnel-user is only valid in tunnel mode.\n");
+      exit(1);
+    }
+
   if (run_mode == run_mode_none)
     {
-      fprintf(stdout, "You must specify one of -d, -i, -t or -X.\n");
+      fprintf(stderr, "You must specify one of -d, -i, -t or -X.\n");
       usage(argv[0]);
     }
 
   if (run_mode == run_mode_inetd || run_mode == run_mode_tunnel)
     {
+      params.tunnel = (run_mode == run_mode_tunnel);
       apr_pool_cleanup_register(pool, pool, apr_pool_cleanup_null,
                                 redirect_stdout);
       apr_file_open_stdin(&in_file, pool);
       apr_file_open_stdout(&out_file, pool);
       conn = svn_ra_svn_create_conn(NULL, in_file, out_file, pool);
-      svn_error_clear(serve(conn, root, run_mode == run_mode_tunnel,
-                            read_only, pool));
+      svn_error_clear(serve(conn, &params, pool));
       exit(0);
     }
 
@@ -381,7 +444,7 @@ int main(int argc, const char *const *argv)
 
       if (run_mode == run_mode_listen_once)
         {
-          err = serve(conn, root, FALSE, read_only, connection_pool);
+          err = serve(conn, &params, connection_pool);
 
           if (err && err->apr_err != SVN_ERR_RA_SVN_CONNECTION_CLOSED)
             svn_handle_error(err, stdout, FALSE);
@@ -399,8 +462,8 @@ int main(int argc, const char *const *argv)
           status = apr_proc_fork(&proc, connection_pool);
           if (status == APR_INCHILD)
             {
-              svn_error_clear(serve(conn, root, FALSE, read_only,
-                                    connection_pool));
+              apr_socket_close(sock);
+              svn_error_clear(serve(conn, &params, connection_pool));
               apr_socket_close(usock);
               exit(0);
             }
@@ -438,8 +501,7 @@ int main(int argc, const char *const *argv)
             }
           thread_data = apr_palloc(connection_pool, sizeof(*thread_data));
           thread_data->conn = conn;
-          thread_data->root = root;
-          thread_data->read_only = read_only;
+          thread_data->params = &params;
           thread_data->pool = connection_pool;
           status = apr_thread_create(&tid, tattr, serve_thread, thread_data,
                                      connection_pool);
@@ -454,8 +516,7 @@ int main(int argc, const char *const *argv)
 
         case connection_mode_single:
           /* Serve one connection at a time. */
-          svn_error_clear(serve(conn, root, FALSE, read_only,
-                                connection_pool));
+          svn_error_clear(serve(conn, &params, connection_pool));
           svn_pool_destroy(connection_pool);
         }
     }
