@@ -19,8 +19,12 @@
 #include "svn_client.h"
 #include "svn_utf.h"
 #include "svn_path.h"
+#include "svn_opt.h"
 #include <apr_tables.h>
 
+#include "svn_private_config.h"
+
+#define SVNVERSION_OPT_VERSION SVN_OPT_FIRST_LONGOPT_ID
 
 struct status_baton
 {
@@ -107,44 +111,69 @@ cancel (void *baton)
     return SVN_NO_ERROR;
 }
 
+static svn_error_t * version(apr_getopt_t *os, apr_pool_t *pool)
+{
+  return svn_opt_print_help(os, "svnversion", TRUE, FALSE, NULL, NULL,
+                            NULL, NULL, NULL, pool);
+}
 
 static void
-usage(const apr_getopt_option_t *options)
+usage(const apr_getopt_option_t *options, apr_pool_t *pool)
 {
-  fprintf(stderr, 
-          "usage: svnversion [options] wc_path [trail_url]\n\n"
-          "  Produce a compact \"version number\" for the working copy path\n"
-          "  WC_PATH.  TRAIL_URL is the trailing portion of the URL used to\n"
-          "  determine if WC_PATH itself is switched (detection of switches\n"
-          "  within WC_PATH does not rely on TRAIL_URL).  The version number\n"
-          "  is written to standard output.  For example:\n"
-          "\n"
-          "    $ svnversion . /repos/svn/trunk \n"
-          "    4168\n"
-          "\n"
-          "  The version number will be a single number if the working\n"
-          "  copy is single revision, unmodified, not switched and with\n"
-          "  an URL that matches the TRAIL_URL argument.  If the working\n"
-          "  copy is unusual the version number will be more complex:\n"
-          "\n"
-          "   4123:4168     mixed revision working copy\n"
-          "   4168M         modified working copy\n"
-          "   4123S         switched working copy\n"
-          "   4123:4168MS   mixed revision, modified, switched working copy\n"
-          "\n"
-          "  If invoked on a directory that is not a working copy, an\n"
-          "  exported directory say, the program will output \"exported\".\n"
-          "\n"
-          "options:\n");
+  svn_error_clear
+    (svn_cmdline_fprintf
+     (stderr, pool,
+      _("usage: svnversion [OPTIONS] WC_PATH [TRAIL_URL]\n\n"
+        "  Produce a compact \"version number\" for the working copy path\n"
+        "  WC_PATH.  TRAIL_URL is the trailing portion of the URL used to\n"
+        "  determine if WC_PATH itself is switched (detection of switches\n"
+        "  within WC_PATH does not rely on TRAIL_URL).  The version number\n"
+        "  is written to standard output.  For example:\n"
+        "\n"
+        "    $ svnversion . /repos/svn/trunk \n"
+        "    4168\n"
+        "\n"
+        "  The version number will be a single number if the working\n"
+        "  copy is single revision, unmodified, not switched and with\n"
+        "  an URL that matches the TRAIL_URL argument.  If the working\n"
+        "  copy is unusual the version number will be more complex:\n"
+        "\n"
+        "   4123:4168     mixed revision working copy\n"
+        "   4168M         modified working copy\n"
+        "   4123S         switched working copy\n"
+        "   4123:4168MS   mixed revision, modified, switched working copy\n"
+        "\n"
+        "  If invoked on a directory that is not a working copy, an\n"
+        "  exported directory say, the program will output \"exported\".\n"
+        "\n"
+        "Valid options:\n")));
   while (options->description)
     {
-      fprintf(stderr, "  -%c  %s\n",
-              options->optch, options->description);
+      const char *optstr;
+      svn_opt_format_option(&optstr, options, TRUE, pool);
+      svn_error_clear (svn_cmdline_fprintf(stderr, pool, "  %s\n", optstr));
       ++options;
     }
 }
 
 
+/* Version compatibility check */
+static svn_error_t *
+check_lib_versions (void)
+{
+  static const svn_version_checklist_t checklist[] =
+    {
+      { "svn_subr",   svn_subr_version },
+      { "svn_client", svn_client_version },
+      { "svn_wc",     svn_wc_version },
+      { NULL, NULL }
+    };
+
+   SVN_VERSION_DEFINE (my_version);
+   return svn_ver_check_list (&my_version, checklist);
+}
+
+
 /*
  * Why is this not an svn subcommand?  I have this vague idea that it could
  * be run as part of the build process, with the output embedded in the svn
@@ -167,8 +196,9 @@ main(int argc, const char *argv[])
   apr_getopt_t *os;
   const apr_getopt_option_t options[] =
     {
-      {"no-newline", 'n', 0, "do not output the trailing newline"},
-      {"committed",  'c', 0, "last changed rather than current revisions"},
+      {"no-newline", 'n', 0, N_("do not output the trailing newline")},
+      {"committed",  'c', 0, N_("last changed rather than current revisions")},
+      {"version", SVNVERSION_OPT_VERSION, 0, N_("show version information")},
       {0,             0,  0,  0}
     };
 
@@ -187,6 +217,16 @@ main(int argc, const char *argv[])
   pool = svn_pool_create_ex (NULL, allocator);
   apr_allocator_owner_set (allocator, pool);
 
+  /* Check library versions */
+  err = check_lib_versions ();
+  if (err)
+    {
+      svn_handle_error (err, stderr, FALSE);
+      svn_error_clear (err);
+      svn_pool_destroy (pool);
+      return EXIT_FAILURE;
+    }
+
   sb.switched = FALSE;
   sb.modified = FALSE;
   sb.committed = FALSE;
@@ -208,7 +248,7 @@ main(int argc, const char *argv[])
         break;
       if (status != APR_SUCCESS)
         {
-          usage(options);
+          usage(options, pool);
           return EXIT_FAILURE;
         }
       switch (opt)
@@ -219,15 +259,19 @@ main(int argc, const char *argv[])
         case 'c':
           sb.committed = TRUE;
           break;
+        case SVNVERSION_OPT_VERSION:
+          SVN_INT_ERR(version(os, pool));
+          exit(0);
+          break;
         default:
-          usage(options);
+          usage(options, pool);
           return EXIT_FAILURE;
         }
     }
 
   if (os->ind >= argc || os->ind < argc - 2)
     {
-      usage(options);
+      usage(options, pool);
       return EXIT_FAILURE;
     }
 
@@ -240,13 +284,16 @@ main(int argc, const char *argv[])
       SVN_INT_ERR(svn_io_check_path (wc_path, &kind, pool));
       if (kind == svn_node_dir)
         {
-          printf ("exported\n");
+          SVN_INT_ERR (svn_cmdline_printf (pool, _("exported\n")));
           svn_pool_destroy (pool);
           return EXIT_SUCCESS;
         }
       else
         {
-          fprintf (stderr, "'%s' not versioned, and not exported\n", wc_path);
+          svn_error_clear
+            (svn_cmdline_fprintf (stderr, pool,
+                                  _("'%s' not versioned, and not exported\n"),
+                                  wc_path));
           svn_pool_destroy (pool);
           return EXIT_FAILURE;
         }
@@ -291,17 +338,20 @@ main(int argc, const char *argv[])
         }
     }
 
-  printf ("%" SVN_REVNUM_T_FMT, sb.min_rev);
+  SVN_INT_ERR (svn_cmdline_printf (pool, "%ld", sb.min_rev));
   if (sb.min_rev != sb.max_rev)
-    printf (":%" SVN_REVNUM_T_FMT, sb.max_rev);
+    SVN_INT_ERR (svn_cmdline_printf (pool, ":%ld", sb.max_rev));
   if (sb.modified)
-    fputs ("M", stdout);
+    SVN_INT_ERR (svn_cmdline_fputs ("M", stdout, pool));
   if (sb.switched)
-    fputs ("S", stdout);
+    SVN_INT_ERR (svn_cmdline_fputs ("S", stdout, pool));
   if (! no_newline)
-    fputs ("\n", stdout);
+    SVN_INT_ERR (svn_cmdline_fputs ("\n", stdout, pool));
 
   svn_pool_destroy (pool);
 
+  /* Flush stdout to make sure that the user will see any printing errors. */
+  SVN_INT_ERR (svn_cmdline_fflush (stdout));
+
   return EXIT_SUCCESS;
 }
