@@ -6,7 +6,7 @@
 #  See http://subversion.tigris.org for more information.
 #    
 # ====================================================================
-# Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2004 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -21,6 +21,7 @@ import string, sys, re, os.path
 
 # Our testing module
 import svntest
+from svntest import SVNAnyOutput
 
 # (abbreviation)
 Skip = svntest.testcase.Skip
@@ -37,6 +38,9 @@ Item = svntest.wc.StateItem
 def check_diff_output(diff_output, name, diff_type):
   "check diff output"
 
+# On Windows, diffs still display / rather than \ in paths
+  if svntest.main.windows == 1:
+    name = string.replace(name, '\\', '/')
   i_re = re.compile('^Index:')
   d_re = re.compile('^Index: (\\./)?' + name)
   p_re = re.compile('^--- (\\./)?' + name)
@@ -92,6 +96,20 @@ def count_diff_output(diff_output):
 
   return diff_count
 
+def verify_expected_output(diff_output, expected):
+  "verify given line exists in diff output"
+  for line in diff_output:
+    if line.find(expected) != -1:
+      break
+  else:
+    raise svntest.Failure
+
+
+def extract_diff_path(line):
+  l2 = line[(line.find("(")+1):]
+  l3 = l2[0:(l2.find(")"))]
+  return l3
+
 ######################################################################
 # diff on a repository subset and check the output
 
@@ -235,15 +253,9 @@ def replace_a_file():
 
 def check_replace_a_file(diff_output):
   "check diff for replace a file"
-  if check_diff_output(diff_output,
+  return check_diff_output(diff_output,
                        os.path.join('A', 'D', 'G', 'rho'),
-                       'D'):
-    return 1
-  if check_diff_output(diff_output,
-                       os.path.join('A', 'D', 'G', 'rho'),
-                       'A'):
-    return 1
-  return 0
+                       'M')
     
 #----------------------------------------------------------------------
 
@@ -396,7 +408,6 @@ def diff_multiple_reverse(sbox):
   "multiple revisions diff'd forwards and backwards"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # rev 2
@@ -436,7 +447,6 @@ def diff_non_recursive(sbox):
   "non-recursive behaviour"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   change_diff_commit_diff(wc_dir, 1,
@@ -475,7 +485,6 @@ def diff_repo_subset(sbox):
   "diff only part of the repository"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   was_cwd = os.getcwd()
@@ -502,7 +511,6 @@ def diff_non_version_controlled_file(sbox):
   "non version controlled files"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   svntest.main.file_append(os.path.join(wc_dir, 'A', 'D', 'foo'), "a new file")
@@ -517,16 +525,19 @@ def diff_non_version_controlled_file(sbox):
   # error message.  The appropriate response is a few lines of errors.  I wish 
   # there was a way to figure out if svn crashed, but all run_svn gives us is 
   # the output, so here we are...
-  if len(err_output) <= 1: raise svntest.Failure
-
+  for line in err_output:
+    if re.search("foo' is not under version control$", line):
+      break
+  else:
+    raise svntest.Failure
   
 # test 9
 def diff_pure_repository_update_a_file(sbox):
   "pure repository diff update a file"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
+  
   was_cwd = os.getcwd()
   os.chdir(wc_dir)
 
@@ -617,27 +628,26 @@ def diff_only_property_change(sbox):
   ### really ought to check that the property diff gets output.
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
-  current_dir = os.getcwd();
-  os.chdir(sbox.wc_dir);
+  current_dir = os.getcwd()
+  os.chdir(sbox.wc_dir)
   try:
+    svntest.actions.run_and_verify_svn(None, None, [],
+                                       'propset',
+                                       'svn:eol-style', 'native', 'iota')
 
-    output, errput = svntest.main.run_svn(None, 'propset',
-                                          'svn:eol-style', 'native', 'iota')
-    if errput: raise svntest.Failure
-    output, errput = svntest.main.run_svn(None, 'ci', '-m', 'empty-msg')
-    if errput: raise svntest.Failure
+    svntest.actions.run_and_verify_svn(None, None, [],
+                                       'ci', '-m', 'empty-msg')
 
-    output, errput = svntest.main.run_svn(None, 'diff', '-r', '1:2')
-    if errput: raise svntest.Failure
+    svntest.actions.run_and_verify_svn(None, None, [],
+                                       'diff', '-r', '1:2')
 
-    output, errput = svntest.main.run_svn(None, 'diff', '-r', '2:1')
-    if errput: raise svntest.Failure
+    svntest.actions.run_and_verify_svn(None, None, [],
+                                       'diff', '-r', '2:1')
 
-    output, errput = svntest.main.run_svn(None, 'diff', '-r', '1')
-    if errput: raise svntest.Failure
+    svntest.actions.run_and_verify_svn(None, None, [],
+                                       'diff', '-r', '1')
 
   finally:
     os.chdir(current_dir)
@@ -652,7 +662,6 @@ def dont_diff_binary_file(sbox):
   "don't diff file marked as binary type"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
   
   # Add a binary file to the project.
@@ -680,10 +689,9 @@ def dont_diff_binary_file(sbox):
     })
 
   # Commit the new binary file, creating revision 2.
-  if svntest.actions.run_and_verify_commit(wc_dir, expected_output,
-                                           expected_status, None,
-                                           None, None, None, None, wc_dir):
-    raise svntest.Failure
+  svntest.actions.run_and_verify_commit(wc_dir, expected_output,
+                                        expected_status, None,
+                                        None, None, None, None, wc_dir)
 
   # Update the whole working copy to HEAD (rev 2)
   expected_output = svntest.wc.State(wc_dir, {})
@@ -699,13 +707,12 @@ def dont_diff_binary_file(sbox):
     'A/theta' : Item(status='  ', wc_rev=2, repos_rev=2),
     })
 
-  if svntest.actions.run_and_verify_update(wc_dir,
-                                           expected_output,
-                                           expected_disk,
-                                           expected_status,
-                                           None, None, None, None, None,
-                                           1):  # verify props, too.
-    raise svntest.Failure
+  svntest.actions.run_and_verify_update(wc_dir,
+                                        expected_output,
+                                        expected_disk,
+                                        expected_status,
+                                        None, None, None, None, None,
+                                        1)  # verify props, too.
 
   # Make a local mod to the binary file.
   svntest.main.file_append(theta_path, "some extra junk")
@@ -718,11 +725,10 @@ def dont_diff_binary_file(sbox):
 
   stdout, stderr = svntest.main.run_svn(None, 'diff', wc_dir)
 
-  failed_to_display = 0;
   for line in stdout:
     if (re_nodisplay.match(line)):
-      failed_to_display = 1;
-  if not failed_to_display:
+      break
+  else:
     raise svntest.Failure
 
   # Second diff use-case: 'svn diff -r1 wc' compares the wc against a
@@ -730,11 +736,10 @@ def dont_diff_binary_file(sbox):
 
   stdout, stderr = svntest.main.run_svn(None, 'diff', '-r', '1', wc_dir)
 
-  failed_to_display = 0;
   for line in stdout:
     if (re_nodisplay.match(line)):
-      failed_to_display = 1;
-  if not failed_to_display:
+      break
+  else:
     raise svntest.Failure
 
   # Now commit the local mod, creating rev 3.
@@ -748,21 +753,19 @@ def dont_diff_binary_file(sbox):
     'A/theta' : Item(status='  ', wc_rev=3, repos_rev=3),
     })
 
-  if svntest.actions.run_and_verify_commit(wc_dir, expected_output,
-                                           expected_status, None,
-                                           None, None, None, None, wc_dir):
-    raise svntest.Failure
+  svntest.actions.run_and_verify_commit(wc_dir, expected_output,
+                                        expected_status, None,
+                                        None, None, None, None, wc_dir)
 
   # Third diff use-case: 'svn diff -r2:3 wc' will compare two
   # repository trees.
 
   stdout, stderr = svntest.main.run_svn(None, 'diff', '-r', '2:3', wc_dir)
 
-  failed_to_display = 0;
   for line in stdout:
     if (re_nodisplay.match(line)):
-      failed_to_display = 1;
-  if not failed_to_display:
+      break
+  else:
     raise svntest.Failure
 
 
@@ -777,7 +780,7 @@ def diff_nonextant_urls(sbox):
                                                  '--old', non_extant_url,
                                                  '--new', extant_url)
   for line in err_output:
-    if re.match('svn: Filesystem has no item$', line):
+    if re.search('was not found in the repository at revision', line):
       break
   else:
     raise svntest.Failure
@@ -786,7 +789,7 @@ def diff_nonextant_urls(sbox):
                                                  '--old', extant_url,
                                                  '--new', non_extant_url)
   for line in err_output:
-    if re.match('svn: Filesystem has no item$', line):
+    if re.search('was not found in the repository at revision', line):
       break
   else:
     raise svntest.Failure
@@ -803,10 +806,9 @@ def diff_head_of_moved_file(sbox):
   # Modify the file to ensure that the diff is non-empty.
   svntest.main.file_append(new_mu_path, "\nActually, it's a new mu.")
 
-  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r', 'HEAD',
-                                                 new_mu_path)
-  if diff_output == []:
-    raise svntest.Failure
+  svntest.actions.run_and_verify_svn(None, SVNAnyOutput, None,
+                                     'diff', '-r', 'HEAD', new_mu_path)
+
 
 
 #----------------------------------------------------------------------
@@ -818,7 +820,6 @@ def diff_base_to_repos(sbox):
   "diff text-bases against repository"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   iota_path = os.path.join(sbox.wc_dir, 'iota')
@@ -854,11 +855,9 @@ def diff_base_to_repos(sbox):
   # If we run 'svn diff -r 1', we should see diffs that include *both*
   # the rev2 changes and local mods.  That's because the working files
   # are being compared to the repository.
-  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r', '1',
-                                                 wc_dir)
-  if err_output:
-    raise svntest.Failure
-
+  diff_output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                        'diff',
+                                                        '-r', '1', wc_dir)
   line1 = "+This is the file 'iota'.some rev2 iota text.an iota local mod.\n"
 
   for line in diff_output:
@@ -870,11 +869,9 @@ def diff_base_to_repos(sbox):
   # If we run 'svn diff -r BASE:1', we should see diffs that only show
   # the rev2 changes and NOT the local mods.  That's because the
   # text-bases are being compared to the repository.
-  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r',
-                                                 'BASE:1', wc_dir)
-  if err_output:
-    raise svntest.Failure
-
+  diff_output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                        'diff', '-r', 'BASE:1',
+                                                        wc_dir)
   line1 = "-This is the file 'iota'.some rev2 iota text.\n"
 
   for line in diff_output:
@@ -889,10 +886,9 @@ def diff_base_to_repos(sbox):
   # For example, we just ran 'svn diff -rBASE:1'.  The output should
   # look exactly the same as 'svn diff -r2:1'.  (If you remove the
   # header commentary)  
-  diff_output2, err_output = svntest.main.run_svn(None, 'diff', '-r',
-                                                 '2:1', wc_dir)
-  if err_output:
-    raise svntest.Failure
+  diff_output2, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                         'diff', '-r', '2:1',
+                                                         wc_dir)
 
   diff_output[2:4] = []
   diff_output2[2:4] = []
@@ -901,15 +897,14 @@ def diff_base_to_repos(sbox):
     raise svntest.Failure
 
   # and similarly, does 'svn diff -r1:2' == 'svn diff -r1:BASE' ?
-  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r',
-                                                 '1:2', wc_dir)
-  if err_output:
-    raise svntest.Failure
+  diff_output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                        'diff',
+                                                        '-r', '1:2', wc_dir)
   
-  diff_output2, err_output = svntest.main.run_svn(None, 'diff', '-r',
-                                                 '1:BASE', wc_dir)
-  if err_output:
-    raise svntest.Failure
+  diff_output2, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                         'diff',
+                                                         '-r', '1:BASE',
+                                                         wc_dir)
 
   diff_output[2:4] = []
   diff_output2[2:4] = []
@@ -933,25 +928,21 @@ def diff_base_to_repos(sbox):
   # once again, verify that -r1:2 and -r1:BASE look the same, as do
   # -r2:1 and -rBASE:1.  None of these diffs should mention the
   # scheduled addition or deletion.
-  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r',
-                                                 '1:2', wc_dir)
-  if err_output:
-    raise svntest.Failure
+  diff_output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                        'diff', '-r',
+                                                        '1:2', wc_dir)
 
-  diff_output2, err_output = svntest.main.run_svn(None, 'diff', '-r',
-                                                 '1:BASE', wc_dir)
-  if err_output:
-    raise svntest.Failure
+  diff_output2, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                         'diff', '-r',
+                                                         '1:BASE', wc_dir)
 
-  diff_output3, err_output = svntest.main.run_svn(None, 'diff', '-r',
-                                                 '2:1', wc_dir)
-  if err_output:
-    raise svntest.Failure
+  diff_output3, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                         'diff', '-r',
+                                                         '2:1', wc_dir)
 
-  diff_output4, err_output = svntest.main.run_svn(None, 'diff', '-r',
-                                                 'BASE:1', wc_dir)
-  if err_output:
-    raise svntest.Failure
+  diff_output4, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                         'diff', '-r',
+                                                         'BASE:1', wc_dir)
 
   diff_output[2:4] = []
   diff_output2[2:4] = []
@@ -999,15 +990,13 @@ def diff_base_to_repos(sbox):
   
   # Now 'svn diff -r3:2' should == 'svn diff -rBASE:2', showing the
   # removal of changes to iota, the adding of mu, and deletion of newfile.
-  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r',
-                                                 '3:2', wc_dir)
-  if err_output:
-    raise svntest.Failure
+  diff_output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                        'diff', '-r',
+                                                        '3:2', wc_dir)
 
-  diff_output2, err_output = svntest.main.run_svn(None, 'diff', '-r',
-                                                 'BASE:2', wc_dir)
-  if err_output:
-    raise svntest.Failure
+  diff_output2, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                         'diff', '-r',
+                                                         'BASE:2', wc_dir)
 
   # to do the comparison, remove all output lines starting with +++ or ---
   re_infoline = re.compile('^(\+\+\+|---).*$')
@@ -1034,7 +1023,6 @@ def diff_deleted_in_head(sbox):
   "repos-repos diff on item deleted from HEAD"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   A_path = os.path.join(sbox.wc_dir, 'A')
@@ -1082,10 +1070,9 @@ def diff_deleted_in_head(sbox):
   # Doing an 'svn diff -r1:2' on the URL of directory A should work,
   # especially over the DAV layer.
   the_url = sbox.repo_url + '/A'
-  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r',
-                                                 '1:2', the_url)
-  if err_output:    
-    raise svntest.Failure
+  diff_output = svntest.actions.run_and_verify_svn(None, None, [],
+                                                   'diff', '-r',
+                                                   '1:2', the_url + "@2")
 
 
 #----------------------------------------------------------------------
@@ -1127,23 +1114,35 @@ def diff_targets(sbox):
     if not check_update_a_file(diff_output) or check_add_a_file(diff_output):
       raise svntest.Failure
 
-    out,err = svntest.main.run_svn(None, 'ci', '-m', 'log msg')
-    if err: raise svntest.Failure
+    diff_output, err_output = svntest.main.run_svn(None, 'ci', '-m', 'log msg')
+    if err_output: raise svntest.Failure
 
-    diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r1:2',
+    diff_output, err_output = svntest.main.run_svn(1, 'diff', '-r1:2',
                                                    update_path, add_path)
-    if check_update_a_file(diff_output) or check_add_a_file(diff_output):
+
+    regex = 'svn: Unable to find repository location for \'.*\''
+    for line in err_output:
+      if re.match(regex, line):
+        break
+    else:
       raise svntest.Failure
 
-    diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r1:2',
+    diff_output, err_output = svntest.main.run_svn(1, 'diff', '-r1:2',
                                                    add_path)
-    if not check_update_a_file(diff_output) or check_add_a_file(diff_output):
+    for line in err_output:
+      if re.match(regex, line):
+        break
+    else:
       raise svntest.Failure
 
-    diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r1:2',
+    diff_output, err_output = svntest.main.run_svn(1, 'diff', '-r1:2',
                                                    '--old', parent_path,
                                                    'alpha', 'theta')
-    if check_update_a_file(diff_output) or check_add_a_file(diff_output):
+    regex = 'svn: \'.*\' was not found in the repository'
+    for line in err_output:
+      if re.match(regex, line):
+        break
+    else:
       raise svntest.Failure
 
     diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r1:2',
@@ -1157,14 +1156,6 @@ def diff_targets(sbox):
   
 
 #----------------------------------------------------------------------
-def verify_expected_output(diff_output, expected):
-  "verify given line exists in diff output"
-  for line in diff_output:
-    if line.find(expected) != -1:
-      break
-  else:
-    raise svntest.Failure
-
 def diff_branches(sbox):
   "diff for branches"
 
@@ -1173,84 +1164,412 @@ def diff_branches(sbox):
   A_url = svntest.main.current_repo_url + '/A'
   A2_url = svntest.main.current_repo_url + '/A2'
 
-  out, err = svntest.main.run_svn(None, 'cp', '-m', 'log msg', A_url, A2_url)
-  if err: raise svntest.Failure
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'cp', '-m', 'log msg',
+                                     A_url, A2_url)
 
-  out, err = svntest.main.run_svn(None, 'up', sbox.wc_dir)
-  if err: raise svntest.Failure
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'up', sbox.wc_dir)
 
   A_alpha = os.path.join(sbox.wc_dir, 'A', 'B', 'E', 'alpha')
   A2_alpha = os.path.join(sbox.wc_dir, 'A2', 'B', 'E', 'alpha')
 
   svntest.main.file_append(A_alpha, "\nfoo\n")
-  out, err = svntest.main.run_svn(None, 'ci', '-m', 'log msg', sbox.wc_dir)
-  if err: raise svntest.Failure
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'log msg', sbox.wc_dir)
 
   svntest.main.file_append(A2_alpha, "\nbar\n")
-  out, err = svntest.main.run_svn(None, 'ci', '-m', 'log msg', sbox.wc_dir)
-  if err: raise svntest.Failure
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'log msg', sbox.wc_dir)
 
   svntest.main.file_append(A_alpha, "zig\n")
 
   # Compare repository file on one branch against repository file on
   # another branch
   rel_path = os.path.join('B', 'E', 'alpha')
-  diff_output, err_output = svntest.main.run_svn(None, 'diff',
-                                                 '--old', A_url,
-                                                 '--new', A2_url,
-                                                 rel_path)
-  if err_output: raise svntest.Failure
+  diff_output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                        'diff',
+                                                        '--old', A_url,
+                                                        '--new', A2_url,
+                                                        rel_path)
   verify_expected_output(diff_output, "-foo")
   verify_expected_output(diff_output, "+bar")
 
   # Same again but using whole branch
-  diff_output, err_output = svntest.main.run_svn(None, 'diff',
-                                                 '--old', A_url,
-                                                 '--new', A2_url)
-  if err_output: raise svntest.Failure
+  diff_output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                        'diff',
+                                                        '--old', A_url,
+                                                        '--new', A2_url)
   verify_expected_output(diff_output, "-foo")
   verify_expected_output(diff_output, "+bar")
 
-  # Compare working file on one branch against repository file on
-  # another branch
-  A_path = os.path.join(sbox.wc_dir, 'A')
-  diff_output, err_output = svntest.main.run_svn(None, 'diff',
-                                                 '--old', A2_url,
-                                                 '--new', A_path,
-                                                 rel_path)
-  if err_output: raise svntest.Failure
-  verify_expected_output(diff_output, "-bar")
-  verify_expected_output(diff_output, "+foo")
-  verify_expected_output(diff_output, "+zig")
-
-  # Same again but using whole branch, hmm this doesn't work
-  #diff_output, err_output = svntest.main.run_svn(None, 'diff',
-  #                                               '--old', A2_url,
-  #                                               '--new', A_path)
-  #verify_expected_output(diff_output, "-bar")
-  #verify_expected_output(diff_output, "+foo")
-  #verify_expected_output(diff_output, "+zig")
-
   # Compare two repository files on different branches
-  diff_output, err_output = svntest.main.run_svn(None, 'diff',
-                                                 A_url + '/B/E/alpha',
-                                                 A2_url + '/B/E/alpha')
-  if err_output: raise svntest.Failure
+  diff_output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                        'diff',
+                                                        A_url + '/B/E/alpha',
+                                                        A2_url + '/B/E/alpha')
   verify_expected_output(diff_output, "-foo")
   verify_expected_output(diff_output, "+bar")
 
   # Compare two versions of a file on a single branch
-  diff_output, err_output = svntest.main.run_svn(None, 'diff',
-                                                 A_url + '/B/E/alpha@2',
-                                                 A_url + '/B/E/alpha@3')
-  if err_output: raise svntest.Failure
+  diff_output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                        'diff',
+                                                        A_url + '/B/E/alpha@2',
+                                                        A_url + '/B/E/alpha@3')
   verify_expected_output(diff_output, "+foo")
 
   # Compare identical files on different branches
+  diff_output, err = svntest.actions.run_and_verify_svn(
+    None, [], [],
+    'diff', A_url + '/B/E/alpha@2', A2_url + '/B/E/alpha@3')
+
+
+#----------------------------------------------------------------------
+def diff_repos_and_wc(sbox):
+  "diff between repos URLs and WC paths"
+
+  sbox.build()
+
+  A_url = svntest.main.current_repo_url + '/A'
+  A2_url = svntest.main.current_repo_url + '/A2'
+
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'cp', '-m', 'log msg',
+                                     A_url, A2_url)
+
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'up', sbox.wc_dir)
+
+  A_alpha = os.path.join(sbox.wc_dir, 'A', 'B', 'E', 'alpha')
+  A2_alpha = os.path.join(sbox.wc_dir, 'A2', 'B', 'E', 'alpha')
+
+  svntest.main.file_append(A_alpha, "\nfoo\n")
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'log msg', sbox.wc_dir)
+
+  svntest.main.file_append(A2_alpha, "\nbar\n")
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'log msg', sbox.wc_dir)
+
+  svntest.main.file_append(A_alpha, "zig\n")
+
+  # Compare working file on one branch against repository file on
+  # another branch
+  A_path = os.path.join(sbox.wc_dir, 'A')
+  rel_path = os.path.join('B', 'E', 'alpha')
+  diff_output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                        'diff',
+                                                        '--old', A2_url,
+                                                        '--new', A_path,
+                                                        rel_path)
+  verify_expected_output(diff_output, "-bar")
+  verify_expected_output(diff_output, "+foo")
+  verify_expected_output(diff_output, "+zig")
+
+  # Same again but using whole branch
+  diff_output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                        'diff',
+                                                        '--old', A2_url,
+                                                        '--new', A_path)
+  verify_expected_output(diff_output, "-bar")
+  verify_expected_output(diff_output, "+foo")
+  verify_expected_output(diff_output, "+zig")
+
+#----------------------------------------------------------------------
+def diff_file_urls(sbox):
+  "diff between two file URLs (issue #1311)"
+
+  sbox.build()
+
+  iota_path = os.path.join(sbox.wc_dir, 'iota')
+  iota_url = svntest.main.current_repo_url + '/iota'
+  iota_copy_path = os.path.join(sbox.wc_dir, 'A', 'iota')
+  iota_copy_url = svntest.main.current_repo_url + '/A/iota'
+  iota_copy2_url = svntest.main.current_repo_url + '/A/iota2'
+
+  # Put some different text into iota, and commit.
+  os.remove(iota_path)
+  svntest.main.file_append(iota_path, "foo\nbar\nsnafu\n")
+  
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'log msg', iota_path)
+
+  # Now, copy the file elsewhere, twice.
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'cp', '-m', 'log msg',
+                                     iota_url, iota_copy_url)
+
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'cp', '-m', 'log msg',
+                                     iota_url, iota_copy2_url)
+
+  # Update (to get the copies)
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'up', sbox.wc_dir)
+
+  # Now, make edits to one of the copies of iota, and commit.
+  os.remove(iota_copy_path)
+  svntest.main.file_append(iota_copy_path, "foo\nsnafu\nabcdefg\nopqrstuv\n")
+
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'log msg', iota_copy_path)
+
+  # Finally, do a diff between the first and second copies of iota,
+  # and verify that we got the expected lines.  And then do it in reverse!
+  out, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                'diff',
+                                                iota_copy_url, iota_copy2_url)
+
+  verify_expected_output(out, "+bar")
+  verify_expected_output(out, "-abcdefg")
+  verify_expected_output(out, "-opqrstuv")
+
+  out, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                'diff',
+                                                iota_copy2_url, iota_copy_url)
+
+  verify_expected_output(out, "-bar")
+  verify_expected_output(out, "+abcdefg")
+  verify_expected_output(out, "+opqrstuv")
+  
+#----------------------------------------------------------------------
+def diff_prop_change_local_edit(sbox):
+  "diff a property change plus a local edit"
+
+  sbox.build()
+
+  iota_path = os.path.join(sbox.wc_dir, 'iota')
+  iota_url = svntest.main.current_repo_url + '/iota'
+
+  # Change a property on iota, and commit.
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'propset', 'pname', 'pvalue', iota_path)
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'log msg', iota_path)
+
+  # Make local edits to iota.
+  svntest.main.file_append(iota_path, "\nMore text.\n")
+
+  # diff r0:PREV should show neither the property change nor the local edit.
+  out, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                'diff', '-r0:PREV', iota_path)
+  for line in out:
+    if line.find("+More text.") != -1:
+      raise svntest.Failure
+    if line.find("   + pvalue") != -1:
+      raise svntest.Failure
+
+  # diff r0:COMMITTED should show the property change but not the local edit.
+  out, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                'diff', '-r0:COMMITTED', iota_path)
+  for line in out:
+    if line.find("+More text.") != -1:
+      raise svntest.Failure
+  verify_expected_output(out, "   + pvalue")
+
+  # diff r0:BASE should show the property change but not the local edit.
+  out, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                'diff', '-r0:BASE', iota_path)
+  for line in out:
+    if line.find("+More text.") != -1:
+      raise svntest.Failure                   # fails at r7481
+  verify_expected_output(out, "   + pvalue")  # fails at r7481
+
+  # diff r0:WC should show the property change as well as the local edit.
+  out, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                'diff', '-r0', iota_path)
+  verify_expected_output(out, "+More text.")
+  verify_expected_output(out, "   + pvalue")  # fails at r7481
+
+  # diff r1:WC should show the local edit as well as the property change.
+  out, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                'diff', '-r1', iota_path)
+  verify_expected_output(out, "+More text.")  # fails at r7481
+  verify_expected_output(out, "   + pvalue")
+
+#----------------------------------------------------------------------
+def check_for_omitted_prefix_in_path_component(sbox):
+  "check for omitted prefix in path component"
+
+  sbox.build()
+
+  prefix_path = os.path.join(sbox.wc_dir, 'prefix_mydir')
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'mkdir', prefix_path)
+  other_prefix_path = os.path.join(sbox.wc_dir, 'prefix_other')
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'mkdir', other_prefix_path)
+
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'log msg', sbox.wc_dir)
+
+
+  file_path = os.path.join(prefix_path, "test.txt")
+  f = open(file_path, "w")
+  f.write("Hello\nThere\nIota\n")
+  f.close()
+
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'add', file_path)
+
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'log msg', sbox.wc_dir)
+
+
+  prefix_url = svntest.main.current_repo_url + "/prefix_mydir"
+  other_prefix_url = svntest.main.current_repo_url + "/prefix_other/mytag"
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'cp', '-m', 'log msg', prefix_url,
+                                     other_prefix_url)
+
+  f = open(file_path, "w")
+  f.write("Hello\nWorld\nIota\n")
+  f.close()
+
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'log msg', prefix_path)
+
+  out, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                'diff', prefix_url,
+                                                other_prefix_url)
+
+  src = extract_diff_path(out[2])
+  dest = extract_diff_path(out[3])
+
+  good_src = ".../prefix_mydir"
+  good_dest = ".../prefix_other/mytag"
+
+  if ((src != good_src) or (dest != good_dest)):
+    print("src is '%s' instead of '%s' and dest is '%s' instead of '%s'" %
+          (src, good_src, dest, good_dest))
+    raise svntest.Failure
+
+#----------------------------------------------------------------------
+def diff_renamed_file(sbox):
+  "diff a file that has been renamed"
+
+  sbox.build()
+
+  was_cwd = os.getcwd()
+  os.chdir(sbox.wc_dir)
+
+  
+  svntest.main.file_append(os.path.join('A', 'D', 'G', 'pi'), "more pi")
+  
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'log msg')
+  
+  svntest.main.file_append(os.path.join('A', 'D', 'G', 'pi'), "even more pi")
+  
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'log msg')
+  
+  svntest.main.run_svn(None, 'mv', os.path.join('A', 'D', 'G', 'pi'),
+                                   os.path.join('A', 'D', 'pi2'))
+
+  # Repos->WC diff of the file
+  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r', '1',
+                                                 os.path.join('A', 'D', 'pi2'))
+
+  if check_diff_output(diff_output,
+                       os.path.join('A', 'D', 'pi2'),
+                       'M') :
+    raise svntest.Failure
+                                   
+  svntest.main.file_append(os.path.join('A', 'D', 'pi2'), "new pi")
+
+  # Repos->WC of the directory
+  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r', '1',
+                                                 os.path.join('A', 'D'))
+
+  if check_diff_output(diff_output,
+                       os.path.join('A', 'D', 'G', 'pi'),
+                       'D') :
+    raise svntest.Failure
+    
+  if check_diff_output(diff_output,
+                       os.path.join('A', 'D', 'pi2'),
+                       'M') :
+    raise svntest.Failure
+  
+  # WC->WC of the file
   diff_output, err_output = svntest.main.run_svn(None, 'diff',
-                                                 A_url + '/B/E/alpha@2',
-                                                 A2_url + '/B/E/alpha@3')
-  if diff_output or err_output: raise svntest.Failure
+                                                 os.path.join('A', 'D', 'pi2'))
+  if check_diff_output(diff_output,
+                       os.path.join('A', 'D', 'pi2'),
+                       'M') :
+    raise svntest.Failure
+
+    
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'log msg')
+
+  # Repos->WC diff of file after the rename.
+  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r', '1',
+                                                 os.path.join('A', 'D', 'pi2'))
+  if check_diff_output(diff_output,
+                       os.path.join('A', 'D', 'pi2'),
+                       'M') :
+    raise svntest.Failure
+
+  # Repos->repos diff after the rename.
+  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r', '2:3',
+                                                 os.path.join('A', 'D', 'pi2'))
+  if check_diff_output(diff_output,
+                       os.path.join('A', 'D', 'pi'),
+                       'M') :
+    raise svntest.Failure
+
+  os.chdir(was_cwd)
+
+#----------------------------------------------------------------------
+def diff_within_renamed_dir(sbox):
+  "diff a file within a renamed directory"
+
+  sbox.build()
+
+  was_cwd = os.getcwd()
+  os.chdir(sbox.wc_dir)
+
+  svntest.main.run_svn(None, 'mv', os.path.join('A', 'D', 'G'),
+                                   os.path.join('A', 'D', 'I'))
+  # svntest.main.run_svn(None, 'ci', '-m', 'log_msg')
+  svntest.main.file_append(os.path.join('A', 'D', 'I', 'pi'), "new pi")
+
+  # Check a repos->wc diff
+  diff_output, err_output = svntest.main.run_svn(None, 'diff',
+                                                 os.path.join('A', 'D', 'I', 'pi'))
+  if check_diff_output(diff_output,
+                       os.path.join('A', 'D', 'I', 'pi'),
+                       'M') :
+    raise svntest.Failure
+
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'log msg')
+
+  # Check repos->wc after commit
+  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r', '1',
+                                                 os.path.join('A', 'D', 'I', 'pi'))
+  if check_diff_output(diff_output,
+                       os.path.join('A', 'D', 'I', 'pi'),
+                       'M') :
+    raise svntest.Failure
+
+  # Test the diff while within the moved directory
+  os.chdir(os.path.join('A','D','I'))
+
+  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r', '1')
+
+  if check_diff_output(diff_output, 'pi', 'M') :
+    raise svntest.Failure
+
+  # Test a repos->repos diff while within the moved directory
+  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r', '1:2')
+
+  if check_diff_output(diff_output, 'pi', 'M') :
+    raise svntest.Failure
+
+  os.chdir(was_cwd)
 
 ########################################################################
 #Run the tests
@@ -1275,6 +1594,12 @@ test_list = [ None,
               diff_deleted_in_head,
               diff_targets,
               diff_branches,
+              diff_repos_and_wc,
+              diff_file_urls,
+              XFail(diff_prop_change_local_edit),
+              check_for_omitted_prefix_in_path_component,
+              diff_renamed_file,
+              diff_within_renamed_dir,
               ]
 
 if __name__ == '__main__':
