@@ -6,7 +6,7 @@
 #  See http://subversion.tigris.org for more information.
 #    
 # ====================================================================
-# Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2004 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -46,6 +46,12 @@ class SVNExpectedStderr(SVNUnexpectedOutput):
   STDERR when output was expected."""
   pass
 
+class SVNIncorrectDatatype(SVNUnexpectedOutput):
+  """Exception raised if invalid input is passed to the 
+  run_and_verify_* API"""
+  pass
+
+
 ######################################################################
 # Used by every test, so that they can run independently of
 # one another.  The first time it's run, it executes 'svnadmin' to
@@ -70,6 +76,8 @@ def guarantee_greek_repository(path):
 
     # build a URL for doing an import.
     url = main.test_area_url + '/' + main.pristine_dir
+    if main.windows == 1:
+      url = string.replace(url, '\\', '/')
 
     # import the greek tree, using l:foo/p:bar
     ### todo: svn should not be prompting for auth info when using
@@ -78,7 +86,7 @@ def guarantee_greek_repository(path):
                                   '--username', main.wc_author,
                                   '--password', main.wc_passwd,
                                   '-m', 'Log message for revision 1.',
-                                  url, main.greek_dump_dir)
+                                  main.greek_dump_dir, url)
 
     # check for any errors from the import
     if len(errput):
@@ -114,8 +122,7 @@ def guarantee_greek_repository(path):
       sys.exit(1)
 
   # Now that the pristine repos exists, copy it to PATH.
-  if os.path.exists(path):
-    shutil.rmtree(path)
+  main.safe_rmtree(path)
   if main.copy_repos(main.pristine_dir, path, 1):
     print "ERROR:  copying repository failed."
     sys.exit(1)
@@ -123,6 +130,31 @@ def guarantee_greek_repository(path):
   # make the repos world-writeable, for mod_dav_svn's sake.
   main.chmod_tree(path, 0666, 0666)
 
+def run_and_verify_svnversion(message, wc_dir, repo_url,
+                              expected_stdout, expected_stderr):
+  "Run svnversion command and check its output"
+
+  out, err = main.run_svnversion(wc_dir, repo_url)
+  
+  if type(expected_stdout) is type([]):
+    compare_and_display_lines(message, 'STDOUT', expected_stdout, out)
+  elif expected_stdout == SVNAnyOutput:
+    if len(out) == 0:
+      if message is not None: print message
+      raise SVNExpectedStdout
+  elif expected_stdout is not None:
+    raise SVNIncorrectDatatype("Unexpected specification for stdout data")
+  
+  if type(expected_stderr) is type([]):
+    compare_and_display_lines(message, 'STDERR', expected_stderr, err)
+  elif expected_stderr == SVNAnyOutput:
+    if len(err) == 0:
+      if message is not None: print message
+      raise SVNExpectedStderr
+  elif expected_stderr is not None:
+    raise SVNIncorrectDatatype("Unexpected specification for stderr data")
+  return out, err
+  
 
 def run_and_verify_svn(message, expected_stdout, expected_stderr, *varargs):
   """Invokes main.run_svn with *VARARGS.  If EXPECTED_STDOUT or
@@ -138,17 +170,21 @@ def run_and_verify_svn(message, expected_stdout, expected_stderr, *varargs):
 
   if type(expected_stdout) is type([]):
     compare_and_display_lines(message, 'STDOUT', expected_stdout, out)
-  if expected_stdout == SVNAnyOutput:
+  elif expected_stdout == SVNAnyOutput:
     if len(out) == 0:
       if message is not None: print message
       raise SVNExpectedStdout
-
+  elif expected_stdout is not None:
+    raise SVNIncorrectDatatype("Unexpected specification for stdout data")
+  
   if type(expected_stderr) is type([]):
     compare_and_display_lines(message, 'STDERR', expected_stderr, err)
-  if expected_stderr == SVNAnyOutput:
+  elif expected_stderr == SVNAnyOutput:
     if len(err) == 0:
       if message is not None: print message
       raise SVNExpectedStderr
+  elif expected_stderr is not None:
+    raise SVNIncorrectDatatype("Unexpected specification for stderr data")
   return out, err
 
 
@@ -207,6 +243,48 @@ def run_and_verify_checkout(URL, wc_dir_name, output_tree, disk_tree,
                       singleton_handler_b, b_baton)
 
 
+def run_and_verify_export(URL, export_dir_name, output_tree, disk_tree,
+                          singleton_handler_a = None,
+                          a_baton = None,
+                          singleton_handler_b = None,
+                          b_baton = None,
+                          *args):
+  """Export the URL into a new directory WC_DIR_NAME.
+
+  The subcommand output will be verified against OUTPUT_TREE,
+  and the exported copy itself will be verified against DISK_TREE.
+  SINGLETON_HANDLER_A and SINGLETON_HANDLER_B will be passed to
+  tree.compare_trees - see that function's doc string for more details.
+  Returns if successful and raise on failure."""
+
+  if isinstance(output_tree, wc.State):
+    output_tree = output_tree.old_tree()
+  if isinstance(disk_tree, wc.State):
+    disk_tree = disk_tree.old_tree()
+
+  # Export and make a tree of the output, using l:foo/p:bar
+  ### todo: svn should not be prompting for auth info when using
+  ### repositories with no auth/auth requirements
+  output, errput = main.run_svn (None, 'export',
+                                 '--username', main.wc_author,
+                                 '--password', main.wc_passwd,
+                                 URL, export_dir_name, *args)
+  mytree = tree.build_tree_from_checkout (output)
+
+  # Verify actual output against expected output.
+  tree.compare_trees (mytree, output_tree)
+
+  # Create a tree by scanning the working copy.  Don't ignore
+  # the .svn directories so that we generate an error if they
+  # happen to show up.
+  mytree = tree.build_tree_from_wc (export_dir_name, ignore_svn=0)
+
+  # Verify expected disk against actual disk.
+  tree.compare_trees (mytree, disk_tree,
+                      singleton_handler_a, a_baton,
+                      singleton_handler_b, b_baton)
+
+
 def verify_update(actual_output, wc_dir_name,
                   output_tree, disk_tree, status_tree,
                   singleton_handler_a, a_baton,
@@ -303,13 +381,15 @@ def run_and_verify_update(wc_dir_name,
 
 
 def run_and_verify_merge(dir, rev1, rev2, url,
-                         output_tree, disk_tree, status_tree,
+                         output_tree, disk_tree, status_tree, skip_tree,
                          error_re_string = None,
                          singleton_handler_a = None,
                          a_baton = None,
                          singleton_handler_b = None,
                          b_baton = None,
-                         check_props = 0):
+                         check_props = 0,
+                         dry_run = 1,
+                         *args):
 
   """Run 'svn merge -rREV1:REV2 URL DIR'
 
@@ -321,12 +401,16 @@ def run_and_verify_merge(dir, rev1, rev2, url,
   The subcommand output will be verified against OUTPUT_TREE, and the
   working copy itself will be verified against DISK_TREE.  If optional
   STATUS_TREE is given, then 'svn status' output will be compared.
-  (This is a good way to check that revision numbers were bumped.)
+  The 'skipped' merge output will be compared to SKIP_TREE.
   SINGLETON_HANDLER_A and SINGLETON_HANDLER_B will be passed to
   tree.compare_trees - see that function's doc string for more
   details.
   
   If CHECK_PROPS is set, then disk comparison will examine props.
+
+  If DRY_RUN is set then a --dry-run merge will be carried out first and
+  the output compared with that of the full merge.
+  
   Returns if successful, raises on failure."""
 
   if isinstance(output_tree, wc.State):
@@ -335,19 +419,29 @@ def run_and_verify_merge(dir, rev1, rev2, url,
     disk_tree = disk_tree.old_tree()
   if isinstance(status_tree, wc.State):
     status_tree = status_tree.old_tree()
+  if isinstance(skip_tree, wc.State):
+    skip_tree = skip_tree.old_tree()
 
-  ### See http://subversion.tigris.org/issues/show_bug.cgi?id=748
-  ### "svn merge" only works in "." right now, unlike all the other
-  ### commands.  This means that people building expected_output trees
-  ### should pass "" as the wc_dir for now, until we can run merge on
-  ### a target deeper than ".".
+  merge_command = ('merge', '-r', rev1 + ':' + rev2, url, dir)
+
+  if dry_run:
+    pre_disk = tree.build_tree_from_wc(dir)
+    dry_run_command = merge_command + ('--dry-run',)
+    dry_run_command = dry_run_command + args
+    out_dry, err_dry = main.run_svn(error_re_string, *dry_run_command)
+    post_disk = tree.build_tree_from_wc(dir)
+    try:
+      tree.compare_trees(post_disk, pre_disk)
+    except tree.SVNTreeError:
+      print "============================================================="
+      print "Dry-run merge altered working copy"
+      print "============================================================="
+      raise
+      
 
   # Update and make a tree of the output.
-  dry_out, dry_err = main.run_svn (error_re_string,
-                                   'merge', '--dry-run',
-                                   '-r', rev1 + ':' + rev2, url, dir)
-  out, err = main.run_svn (error_re_string,
-                           'merge', '-r', rev1 + ':' + rev2, url, dir)
+  merge_command = merge_command + args
+  out, err = main.run_svn (error_re_string, *merge_command)
 
   if (error_re_string):
     rm = re.compile(error_re_string)
@@ -360,18 +454,30 @@ def run_and_verify_merge(dir, rev1, rev2, url,
     ### we should raise a less generic error here. which?
     raise Failure(err)
 
-  if dry_out != out:
+  if dry_run and out != out_dry:
     print "============================================================="
-    print "The merge dry-run output didn't match that of the real merge"
-    print "============================================================="
-    print "The merge dry-run output:"
-    map(sys.stdout.write, dry_out)
-    print "============================================================="
-    print "The merge output:"
+    print "Merge outputs differ"
+    print "The dry-run merge output:"
+    map(sys.stdout.write, out_dry)
+    print "The full merge output:"
     map(sys.stdout.write, out)
     print "============================================================="
-    ### we should raise a less generic error here. which?
+    raise main.SVNUnmatchedError
+
+  def missing_skip(a, b):
+    print "============================================================="
+    print "Merge failed to skip: " + a.path
+    print "============================================================="
     raise Failure
+  def extra_skip(a, b):
+    print "============================================================="
+    print "Merge unexpectedly skipped: " + a.path
+    print "============================================================="
+    raise Failure
+
+  myskiptree = tree.build_tree_from_skipped(out)
+  tree.compare_trees(myskiptree, skip_tree,
+                     extra_skip, None, missing_skip, None)
 
   mytree = tree.build_tree_from_checkout(out)
   verify_update (mytree, dir,
@@ -492,14 +598,14 @@ def run_and_verify_commit(wc_dir_name, output_tree, status_output_tree,
       output.append(lastline)
     
   # Convert the output into a tree.
-  expected_tree = tree.build_tree_from_commit (output)
+  mytree = tree.build_tree_from_commit (output)
     
   # Verify actual output against expected output.
   try:
-    tree.compare_trees (expected_tree, output_tree)
+    tree.compare_trees (mytree, output_tree)
   except tree.SVNTreeError:
       display_trees("Output of commit is unexpected.",
-                    "OUTPUT TREE", expected_tree, output_tree)
+                    "OUTPUT TREE", output_tree, mytree)
       raise
     
   # Verify via 'status' command too, if possible.
@@ -651,8 +757,7 @@ def duplicate_dir(wc_name, wc_copy_name):
   """Copy the working copy WC_NAME to WC_COPY_NAME.  Overwrite any
   existing tree at that location."""
 
-  if os.path.exists(wc_copy_name):
-    main.safe_rmtree(wc_copy_name)
+  main.safe_rmtree(wc_copy_name)
   shutil.copytree(wc_name, wc_copy_name)
   
 
