@@ -97,17 +97,15 @@ test_path_split (const char **msg,
 
   static const char * const paths[][3] = { 
     { "/foo/bar",        "/foo",          "bar" },
-    { "/foo/bar/ ",      "/foo/bar",      " " },
+    { "/foo/bar/ ",       "/foo/bar",      " " },
     { "/foo",            "/",             "foo" },
     { "foo",             SVN_EMPTY_PATH,  "foo" },
-    { "./foo",           SVN_EMPTY_PATH,  "foo" },
-    { "././.bar",        SVN_EMPTY_PATH,  ".bar" },
-    { "/././.bar",       "/",             ".bar" },
-    { "foo///bar",       "foo",           "bar" },
-    { "/foo///bar",      "/foo",          "bar" },
-    { "foo//.//bar",     "foo",           "bar" },
+    { ".bar",            SVN_EMPTY_PATH,  ".bar" },
+    { "/.bar",           "/",             ".bar" },
+    { "foo/bar",         "foo",           "bar" },
+    { "/foo/bar",        "/foo",          "bar" },
+    { "foo/bar",         "foo",           "bar" },
     { "foo./.bar",       "foo.",          ".bar" },
-    { "foo././.bar",     "foo.",          ".bar" },
     { "../foo",          "..",            "foo" },
     { SVN_EMPTY_PATH,   SVN_EMPTY_PATH,   SVN_EMPTY_PATH },
     { "/flu\\b/\\blarg", "/flu\\b",       "\\blarg" },
@@ -293,6 +291,129 @@ test_uri_encode (const char **msg,
 }
 
 
+static svn_error_t *
+test_uri_decode (const char **msg,
+                 svn_boolean_t msg_only,
+                 apr_pool_t *pool)
+{
+  int i;
+
+  const char *paths[3][2] = { 
+    { "http://c.r.a/s%\0008me", 
+         "http://c.r.a/s%"},
+    { "http://c.r.a/s%6\000me",
+         "http://c.r.a/s%6" },
+    { "http://c.r.a/s%68me",
+         "http://c.r.a/shme" },
+  };
+  
+  *msg = "test svn_path_uri_decode with invalid escape";
+
+  if (msg_only)
+    return SVN_NO_ERROR;
+
+  for (i = 0; i < 3; i++)
+    {
+      const char *de_path;
+
+      /* URI-decode the path, and verify the results. */
+      de_path = svn_path_uri_decode (paths[i][0], pool);
+      if (strcmp (de_path, paths[i][1]))
+        {
+          return svn_error_createf
+            (SVN_ERR_TEST_FAILED, NULL,
+             "svn_path_uri_decode ('%s') returned '%s' instead of '%s'",
+             paths[i][0], de_path, paths[i][1]);
+        }
+    }
+  return SVN_NO_ERROR;
+}
+
+
+static svn_error_t *
+test_uri_autoescape (const char **msg,
+                     svn_boolean_t msg_only,
+                     apr_pool_t *pool)
+{
+  static const char *paths[3][2] = {
+    { "http://svn.collab.net/", "http://svn.collab.net/" },
+    { "file:///<>\" {}|\\^`", "file:///%3C%3E%22%20%7B%7D%7C%5C%5E%60" },
+    { "http://[::1]", "http://[::1]" }
+  };
+  int i;
+
+  *msg = "test svn_path_uri_autoescape";
+
+  if (msg_only)
+    return SVN_NO_ERROR;
+
+  for (i = 0; i < 3; ++i)
+    {
+      const char* uri = svn_path_uri_autoescape (paths[i][0], pool);
+      if (strcmp (uri, paths[i][1]) != 0)
+        return svn_error_createf
+          (SVN_ERR_TEST_FAILED, NULL,
+           "svn_path_uri_autoescape on '%s' returned '%s' instead of '%s'",
+           paths[i][0], uri, paths[i][1]);
+      if (strcmp (paths[i][0], paths[i][1]) == 0
+          && paths[i][0] != uri)
+        return svn_error_createf
+          (SVN_ERR_TEST_FAILED, NULL,
+           "svn_path_uri_autoescape on '%s' returned identical but not same"
+           " string", paths[i][0]);
+    }
+                                  
+  return SVN_NO_ERROR;
+}
+
+static svn_error_t *
+test_uri_from_iri (const char **msg,
+                   svn_boolean_t msg_only,
+                   apr_pool_t *pool)
+{
+  /* We have to code the IRIs like this because the compiler might translate
+     character and string literals outside of ASCII to some character set,
+     but here we are hard-coding UTF-8.  But we all read UTF-8 codes like
+     poetry, don't we. */
+  static const char p1[] = {
+    0x66, 0x69, 0x6C, 0x65, 0x3A, 0x2F, 0x2F, 0x2F,
+    0x72, 0xC3, 0xA4, 0x6B, 0x73, 0x6D, 0xC3, 0xB6, 0x72,
+    0x67, 0xC3, 0xA5, 0x73, 0 };
+  static const char p2[] = {
+    0x66, 0x69, 0x6C, 0x65, 0x3A, 0x2F, 0x2F, 0x2F,
+    0x61, 0x62, 0x25, 0x32, 0x30, 0x63, 0x64, 0 };
+  static const char *paths[2][2] = {
+    { p1,
+      "file:///r%C3%A4ksm%C3%B6rg%C3%A5s" },
+    { p2,
+      "file:///ab%20cd" }
+  };
+  int i;
+
+  *msg = "testing svn_uri_from_iri";
+
+  if (msg_only)
+    return SVN_NO_ERROR;
+
+  for (i = 0; i < 2; ++i)
+    {
+      const char *uri = svn_path_uri_from_iri (paths[i][0], pool);
+      if (strcmp (paths[i][1], uri) != 0)
+        return svn_error_createf
+          (SVN_ERR_TEST_FAILED, NULL,
+           "svn_uri_from_iri on '%s' returned '%s' instead of '%s'",
+           paths[i][0], uri, paths[i][1]);
+      if (strcmp (paths[i][0], uri) == 0
+          && paths[i][0] != uri)
+        return svn_error_createf
+          (SVN_ERR_TEST_FAILED, NULL,
+           "svn_path_uri_from_iri on '%s' returned identical but not same"
+           " string", paths[i][0]);
+    }
+
+  return SVN_NO_ERROR;
+}
+
 static svn_error_t *
 test_join (const char **msg,
            svn_boolean_t msg_only,
@@ -402,15 +523,15 @@ test_basename (const char **msg,
   static const char * const paths[][2] = {
     { "abc", "abc" },
     { "/abc", "abc" },
-    { "//abc", "abc" },
+    { "/abc", "abc" },
     { "/x/abc", "abc" },
     { "/xx/abc", "abc" },
-    { "/xx//abc", "abc" },
-    { "/xx//abc", "abc" },
+    { "/xx/abc", "abc" },
+    { "/xx/abc", "abc" },
     { "a", "a" },
     { "/a", "a" },
     { "/b/a", "a" },
-    { "/b//a", "a" },
+    { "/b/a", "a" },
     { "/", "/" },
     { SVN_EMPTY_PATH, SVN_EMPTY_PATH }
   };
@@ -447,14 +568,9 @@ test_decompose (const char **msg,
     "/foo", "/", "foo", NULL,
     "/foo/bar", "/", "foo", "bar", NULL,
     "foo/bar", "foo", "bar", NULL,
-    "foo/../bar", "foo", "..", "bar", NULL,
-    "./foo", SVN_EMPTY_PATH, "foo", NULL,
 
     /* Are these canonical? Should the middle bits produce SVN_EMPTY_PATH? */
-    "foo//bar", "foo", SVN_EMPTY_PATH, "bar", NULL,
-    "foo//.//bar",
-    "foo", SVN_EMPTY_PATH, SVN_EMPTY_PATH, SVN_EMPTY_PATH, "bar", NULL,
-
+    "foo/bar", "foo", "bar", NULL,
     NULL,
   };
   int i = 0;
@@ -525,12 +641,25 @@ test_canonicalize (const char **msg,
     { "foo/..",               "foo/.." },
     { "foo/../",              "foo/.." },
     { "foo/../.",             "foo/.." },
-
-    /* Should canonicalization do these? */
-    { "foo//.//bar",          "foo//.//bar" },
-    { "///foo",               "///foo" },
-    { "/.//./.foo",           "/.//./.foo" },
-    { ".///.foo",             ".///.foo" },
+    { "foo//.//bar",          "foo/bar" },
+    { "///foo",               "/foo" },
+    { "/.//./.foo",           "/.foo" },
+    { ".///.foo",             ".foo" },
+    { "../foo",               "../foo" },
+    { "../../foo/",           "../../foo" },
+    { "../../foo/..",         "../../foo/.." },
+    { "/../../",              "/../.." },
+    { "http://hst",           "http://hst" },
+    { "http://hst/foo/../bar","http://hst/foo/../bar" },
+    { "http://hst/",          "http://hst" },
+#ifdef WIN32 
+    /* We permit UNC paths on Windows.  By definition UNC
+     * paths must have two components so we should remove the
+     * double slash if there is only one component. */
+    { "//hst/foo",            "//hst/foo" },
+    { "//hst",                "/hst" },
+    { "//hst/./",             "/hst" },
+#endif /* WIN32 */
     { NULL, NULL }
   };
   int i;
@@ -549,12 +678,47 @@ test_canonicalize (const char **msg,
                                   "svn_path_canonicalize(\"%s\") returned "
                                   "\"%s\" expected \"%s\"",
                                   paths[i][0], canonical, paths[i][1]);
+      ++i;
+    }
 
-      if (strcmp (paths[i][0], paths[i][1]) == 0 && canonical != paths[i][0])
+  return SVN_NO_ERROR;
+}
+
+static svn_error_t *
+test_remove_component (const char **msg,
+                       svn_boolean_t msg_only,
+                       apr_pool_t *pool)
+{
+  const char *paths[][2] = {
+    { "",                     "" },
+    { "/",                    "/" },
+    { "foo",                  "" },
+    { "foo/bar",              "foo" },
+    { "/foo/bar",             "/foo" },
+    { "/foo",                 "/" },
+    { NULL, NULL }
+  };
+  int i;
+  svn_stringbuf_t *buf;
+
+  *msg = "test svn_path_remove_component";
+  if (msg_only)
+    return SVN_NO_ERROR;
+
+  buf = svn_stringbuf_create ("", pool);
+  
+  i = 0;
+  while (paths[i][0])
+    {
+      svn_stringbuf_set (buf, paths[i][0]);
+
+      svn_path_remove_component (buf);
+      
+      if (strcmp (buf->data, paths[i][1]))
         return svn_error_createf (SVN_ERR_TEST_FAILED, NULL,
-                                  "svn_path_canonicalize(\"%s\") alloc'd",
-                                  paths[i][0]);
-
+                                  "svn_path_remove_component(\"%s\") returned "
+                                  "\"%s\" expected \"%s\"",
+                                  paths[i][0], buf->data, paths[i][1]);
       ++i;
     }
 
@@ -572,9 +736,13 @@ struct svn_test_descriptor_t test_funcs[] =
     SVN_TEST_PASS (test_is_url),
     SVN_TEST_PASS (test_is_uri_safe),
     SVN_TEST_PASS (test_uri_encode),
+    SVN_TEST_PASS (test_uri_decode),
+    SVN_TEST_PASS (test_uri_autoescape),
+    SVN_TEST_PASS (test_uri_from_iri),
     SVN_TEST_PASS (test_join),
     SVN_TEST_PASS (test_basename),
     SVN_TEST_PASS (test_decompose),
     SVN_TEST_PASS (test_canonicalize),
+    SVN_TEST_PASS (test_remove_component),
     SVN_TEST_NULL
   };
