@@ -5,7 +5,7 @@
 #
 TEST_ROOT="/home/brane/svn"
 
-# Installation path, everything under that is considered 
+# Installation path, everything under that is considered
 # to be temporary
 INST_DIR="$TEST_ROOT/inst"
 
@@ -36,6 +36,16 @@ MAKE_OPTS=
 # RAMDISK=<yes|no>
 RAMDISK=no
 
+#
+# Whether to test the BDB backend, TEST_FSFS=<yes|no>
+#
+TEST_BDB=yes
+
+#
+# Whether to test the FSFS backend, TEST_FSFS=<yes|no>
+#
+TEST_FSFS=yes
+
 # This should correspond with your httpd Listen directive
 RA_DAV_CHECK_ARGS="BASE_URL=http://localhost:42024"
 
@@ -66,6 +76,7 @@ BIN="/bin"
 USRBIN="/usr/bin"
 LOCALBIN="/usr/local/bin"
 OPTBIN="/opt/bin"
+PERLBIN="/usr/bin"
 
 # Statically linked svn binary (used for repository updates)
 SVN="$TEST_ROOT/static/bin/svn"
@@ -79,9 +90,35 @@ GUESS="/usr/share/libtool/config.guess"
 # Path to sendmail
 SENDMAIL="/usr/sbin/sendmail"
 
+# A program used to base64 encode standard input.  Two choices are
+# 1) The encode-base64 script that comes with the Perl MIME::Base64
+#    module on CPAN at
+#    ftp://ftp.funet.fi/pub/CPAN/modules/by-module/MIME/
+#
+#    To install, get the latest MIME-Base64-X.YY.tar.gz and run
+#    perl Makefile.PL
+#    make
+#    make test
+#    make install UNINST=1
+#
+#    Uncomment here if you use this base64 encoder:
+#
+#    BASE64="$PERLBIN/encode-base64"
+#    BASE64_E="$BASE64"
+#
+# 2) A pre-compiled Windows base64.exe binary for Windows users.  C
+#    source code for other OSes.  Available at
+#    http://www.fourmilab.ch/webtools/base64/
+#
+#    No instructions needed.  If you can compile and test Subversion,
+#    then you can compile this :)
+#
+#    Uncomment here if you use this base64 encoder:
+#
+#    BASE64="$USRBIN/base64"
+#    BASE64_E="$BASE64 -e - -"
+
 # Other stuff
-BASE64="$USRBIN/base64"
-BASE64_E="$BASE64 -e - -"
 CAT="$BIN/cat"
 CP="$BIN/cp"
 CP_F="$CP -f"
@@ -109,6 +146,11 @@ TAIL_100="$TAIL -n 100"
 TOUCH="$USRBIN/touch"
 UMOUNT="$BIN/umount"
 
+#
+# Branch prefix for the e-mail subject
+#
+REVPREFIX=`$SVN info $SVN_REPO | $SED -ne 's@^URL:.*/repos/svn/\(.*\)$@\1 r@p'`
+
 #
 # Helper functions
 #
@@ -155,24 +197,24 @@ FAIL_LOG() {
 # ii) Ramdisk isn't already mounted
 mount_ramdisk() {
     local mount_dir="$1"
-    if test "xyes" == "x$RAMDISK";
+    if test "xyes" = "x$RAMDISK";
     then
         test -z "$mount_dir" && return 1
-        
+
         test -f "$mount_dir/.ramdisk" && {
             echo "Warning: ramdisk exists"
             return 0
         }
-    
+
         $MOUNT "$mount_dir" || return 1
         $TOUCH "$mount_dir/.ramdisk" || return 1
-    fi 
+    fi
     return 0
 }
 
 umount_ramdisk() {
     local mount_dir="$1"
-    if test "xyes" == "x$RAMDISK";
+    if test "xyes" = "x$RAMDISK";
     then
         test -z "$mount_dir" && return 1
 
@@ -182,4 +224,3 @@ umount_ramdisk() {
     fi
     return 0
 }
-
