@@ -12,7 +12,7 @@ BUILD="`$GUESS` $BUILD_TYPE"
 REV="`$SVN st -v $SVN_REPO/README | $CUT -c 12-17 | $SED -e 's/^ *//'`"
 
 # Initialize the log file
-echo "BUILD: Revision $REV on $BUILD" >> $LOG_FILE
+echo "BUILD: $REVPREFIX$REV on $BUILD" >> $LOG_FILE
 echo >> $LOG_FILE
 
 # Check the build type
@@ -28,8 +28,11 @@ PASS
 
 # Create the object directory
 START "create object directory" "Creating object directory..."
-$RM_RF $TEST_ROOT/$OBJ >> $LOG_FILE 2>&1 || FAIL
-$MKDIR $TEST_ROOT/$OBJ >> $LOG_FILE 2>&1 || FAIL
+umount_ramdisk "$TEST_ROOT/$OBJ/subversion/tests" >> $LOG_FILE 2>&1 || FAIL 
+$RM_RF "$TEST_ROOT/$OBJ" >> $LOG_FILE 2>&1 || FAIL
+$MKDIR "$TEST_ROOT/$OBJ" >> $LOG_FILE 2>&1 || FAIL
+$MKDIR_P "$TEST_ROOT/$OBJ/subversion/tests" >> $LOG_FILE 2>&1 || FAIL
+mount_ramdisk "$TEST_ROOT/$OBJ/subversion/tests" >> $LOG_FILE 2>&1 || FAIL 
 PASS
 
 # Configure
@@ -50,7 +53,19 @@ PASS
 # Build
 START "build" "Building..."
 cd $TEST_ROOT/$OBJ
-$MAKE > "$TEST_ROOT/LOG_svn_build_$BUILD_TYPE" 2>&1
+$MAKE $MAKE_OPTS > "$TEST_ROOT/LOG_svn_build_$BUILD_TYPE" 2>&1
+test $? = 0 || {
+    FAIL_LOG "$TEST_ROOT/LOG_svn_build_$BUILD_TYPE"
+    FAIL
+}
+PASS
+
+
+# Install (bc mod_dav_svn.so)
+START "install" "Installing..."
+cd $TEST_ROOT/$OBJ
+$RM_RF "$INST_DIR/$SVN_NAME"
+$MAKE install > "$TEST_ROOT/LOG_svn_install_$BUILD_TYPE" 2>&1
 test $? = 0 || {
     FAIL_LOG "$TEST_ROOT/LOG_svn_build_$BUILD_TYPE"
     FAIL
